
local tinsert, tremove = table.insert, table.remove
local maxFonts = #SYNCUI_FONTLIST
local lineOnFocus

-- General Stuff --
local function LoadDefaultOptions(perChar)
	if not perChar then
		SyncUI_Data = {
			["Options"] = {
				["Auras"] = {},
				["UsedFont"] = 4,
				["Quest"] = {},
				["InfoSelection"] = 1,
			},
			["Bags"] = {
				["ItemList"] = {},
				["ContainerList"] = {},
			},
		}
	end
	
	if perChar then
		SyncUI_CharData = {
			["WeakAuras"] = {},
			["GroupControl"] = {
				["Layout"] = "Normal",
			},
			["FramePos"] = {
				[1] = {},
				[2] = {},
			},
		}
	end
end

local function LoadSavedOptions(perChar)
	if not perChar then
		-- Options
		if SyncUI_Data.Options.Auras then
			if SyncUI_Data.Options.Auras.DurationTimer then
				SyncUI_OptionPanel_Aura.DurationTimer:SetChecked(true)
			else
				SyncUI_OptionPanel_Aura.DurationColor:SetEnabled(false)
				SyncUI_OptionPanel_Aura.DurationSpiral:SetEnabled(false)
			end
			if SyncUI_Data.Options.Auras.DurationColor then
				SyncUI_OptionPanel_Aura.DurationColor:SetChecked(true)
			end
			if SyncUI_Data.Options.Auras.DurationSpiral then
				SyncUI_OptionPanel_Aura.DurationSpiral:SetChecked(true)
			end
			if SyncUI_Data.Options.Auras.DebuffFilter then
				SyncUI_OptionPanel_Aura.DebuffFilter:SetChecked(true)
			end
		else
			SyncUI_Data.Options.Auras = {}
		end
		if SyncUI_Data.Options.UsedFont then
			if SyncUI_Data.Options.UsedFont > maxFonts then
				SyncUI_Data.Options.UsedFont = 4
			end
			
			SyncUI_SetUIFont()
		end
		if SyncUI_Data.Options.AutoWorkOrders then
			SyncUI_OptionPanel_Misc.AutoWorkOrders:SetChecked(true)
		end
		if SyncUI_Data.Options.SkipCinematics then
			SyncUI_OptionPanel_Misc.SkipCinematics:SetChecked(true)
		end
		if SyncUI_Data.Options.DisableAFK then
			SyncUI_OptionPanel_Misc.DisableAFK:SetChecked(true)
		end
		if SyncUI_Data.Options.AutoSellJunk then
			SyncUI_OptionPanel_Misc.AutoSellJunk:SetChecked(true)
		end
		if SyncUI_Data.Options.ShowNewItems then
			SyncUI_OptionPanel_Misc.ShowNewItems:SetChecked(true)
		else
			SyncUI_OptionPanel_Misc.ClearNewItems:SetEnabled(false)
		end
		if SyncUI_Data.Options.ClearNewItems then
			SyncUI_OptionPanel_Misc.ClearNewItems:SetChecked(true)
		end
		if SyncUI_Data.Options.Quest then
			if SyncUI_Data.Options.Quest.Accept then
				SyncUI_OptionPanel_Quest.Accept:SetChecked(true)
			else
				SyncUI_OptionPanel_Quest.Daily:SetEnabled(false)
			end
			if SyncUI_Data.Options.Quest.Daily then
				SyncUI_OptionPanel_Quest.Daily:SetChecked(true)
			end
			if SyncUI_Data.Options.Quest.TurnIn then
				SyncUI_OptionPanel_Quest.TurnIn:SetChecked(true)
			end
			if SyncUI_Data.Options.Quest.Share then
				SyncUI_OptionPanel_Quest.Share:SetChecked(true)
			end
			if SyncUI_Data.Options.Quest.DisableTracker then
				SyncUI_OptionPanel_Quest.DisableTracker:SetChecked(true)
			end
		else
			SyncUI_Data.Options.Quest = {}
		end
		if SyncUI_Data.Options.DefaultRaid then
			SyncUI_OptionPanel_UnitFrames.DefaultRaid:SetChecked(true)
		end
		if SyncUI_Data.Options.GroupSortMethod then
			SyncUI_OptionPanel_UnitFrames_SelectSortMethod(SyncUI_Data.Options.GroupSortMethod)
		else
			SyncUI_OptionPanel_UnitFrames_SelectSortMethod("GROUP")
		end
		if SyncUI_Data.Options.SkadaChat then
			SyncUI_OptionPanel_Meter.Skada:SetChecked(true)
		end
		if SyncUI_Data.Options.RecountChat then
			SyncUI_OptionPanel_Meter.Recount:SetChecked(true)
		end
		
		-- Bags
		if not SyncUI_Data.Bags then
			SyncUI_Data.Bags = {
				["ItemList"] = {},
				["ContainerList"] = {},
			}
		end
	end

	if perChar then
		if not SyncUI_CharData then
			SyncUI_CharData = {}
			LoadSavedOptions(true)
		else
			if SyncUI_CharData.GroupControl then
				if SyncUI_CharData.GroupControl.Layout then
					local type = SyncUI_CharData.GroupControl.Layout
					
					if type ~= "Normal" and type ~= "Heal" then
						type = "Normal"
					end

					if type == "Heal" then
						SyncUI_PartyFrameContainer.Normal:Hide()
						SyncUI_RaidFrameContainer.Normal:Hide()
						
						SyncUI_PartyFrameContainer.Heal:Show()
						SyncUI_RaidFrameContainer.Heal:Show()
					end
					
					SyncUI_GroupControl.LayoutButton:SetText("Layout: |cFF00FF99"..type)
				else
					SyncUI_CharData.GroupControl.Layout = "Normal"
					SyncUI_GroupControl.LayoutButton:SetText("Layout: |cFF00FF99".."Normal")
				end
				if SyncUI_CharData.GroupControl.Toggle then
					SyncUI_RaidFrameContainer:Hide()
					SyncUI_GroupControl.ToggleButton.Normal:SetTexCoord(0,0.328125,0.25,0.4140625)
					SyncUI_GroupControl.ToggleButton.Pushed:SetTexCoord(0.5,0.828125,0.25,0.4140625)
				end
			else
				SyncUI_CharData.GroupControl = {}
				LoadSavedOptions(true)
			end
			if SyncUI_CharData.SideBarToggle then
				SyncUI_SideBar:Hide()
			end
		end
	end
end


function SyncUI_OptionsMenu_OnLoad(self)
	self:SetScale(SyncUI_UIScale)
	self:RegisterEvent("ADDON_LOADED")
	
	GameMenuFrame:SetAlpha(0)
	GameMenuFrame:EnableMouse(false)
	GameMenuFrame:HookScript("OnShow",function()
		SyncUI_SlideIn(self)
	end)
	GameMenuFrame:HookScript("OnHide",function()
		SyncUI_SlideOut(self)
	end)
	
	for _, child in pairs({GameMenuFrame:GetChildren()}) do
		_G[child:GetName()]:EnableMouse(false)
	end
	
	tinsert(UISpecialFrames,"AddonList")
	
	do	-- Disable InterfaceOptionCategories
		local CategoriesToDisable = {
			[1] = 9,	-- Status Text
			[2] = 10,	-- Unit Frames
			[3] = 11,	-- Buffs & Debuffs
		}
		
		-- fix value if default raid frames are enabled
		if IsAddOnLoaded("Blizzard_CUFProfiles") then
			CategoriesToDisable[3] = 12
		end
		
		for index, id in pairs(CategoriesToDisable) do
			local button = _G["InterfaceOptionsFrameCategoriesButton"..id]
			local text = _G[button:GetName().."Text"]
			
			if button then
				button:Disable()
				text:SetVertexColor(1,0.25,0.25)
			end
		end
		
	end
end

function SyncUI_OptionsMenu_OnEvent(self,event,addOn)
	if event == "ADDON_LOADED" and addOn == "SyncUI" then 
		-- load general settings
		if SyncUI_Data and SyncUI_Data.Options then
			LoadSavedOptions()
		else
			LoadDefaultOptions()
		end
		
		-- load char specific settings
		if not SyncUI_CharData or (SyncUI_CharData and not SyncUI_CharData.Installed) then
			LoadDefaultOptions(true)
			SyncUI_Installer:Show()
		elseif SyncUI_CharData then
			LoadSavedOptions(true)
		else
			LoadDefaultOptions(true)
		end
		
		do	-- call Damage Meter Chat Function
			SyncUI_ChatFrame_DamageMeter_OnLoad()
		end
	end
end

function SyncUI_OptionPanelFrame_HidePanels()
	local children = {SyncUI_OptionPanelFrame:GetChildren()}
	
	for _, child in pairs(children) do
		child:Hide()
	end
end


-- Installer --
function SyncUI_Installer_UpdateProgress(self,elapsed)
	if self.StartProgress then
		local barValue = self.LoadingBar.Progress:GetValue()
		
		self.update = self.update + elapsed

		while self.update > 0.3 do
			self.progress = self.progress + math.random(1,10)
			self.update = self.update - 0.3
		end

		SmoothBar(self.LoadingBar.Progress,self.progress)

		self.LoadingBar.Progress.Text:SetText(math.ceil(barValue).."%")

		if barValue >= 100 then
			self.StartProgress = false
			self.FinProgress = true
			self.LoadingBar.Despawn:Play()
			self:SetScript("OnUpdate",nil)
		end
	end
end


-- Reactive Aura Panel --
local function SortAuras(a,b)
	local aName = GetSpellInfo(a)
	local bName = GetSpellInfo(b)
	
	if aName and bName then
		return aName < bName
	end
end

function SyncUI_OptionPanel_ReactiveAura_OnShow(self)
	table.sort(SyncUI_CharData["WeakAuras"],SortAuras)
	
	self.ScrollFrame.ScrollBar:SetValue(0)
	SyncUI_OptionPanel_ReactiveAura_UpdateScrollFrame(self.ScrollFrame)
end

function SyncUI_OptionPanel_ReactiveAura_AuraLine_DeleteButton_OnClick(self,button)
	local line = self:GetParent()
	local spellID = SyncUI_CharData["WeakAuras"][line:GetID()]
	local scrollFrame = line:GetParent().ScrollFrame
	
	-- Remove Aura
	tremove(SyncUI_CharData["WeakAuras"], line:GetID())
	
	-- Update Scroll Frame
	scrollFrame.ScrollBar:SetValue(0)
	SyncUI_OptionPanel_ReactiveAura_UpdateScrollFrame(scrollFrame)
	
	-- Print out
	if spellID then
		local name, _, icon = GetSpellInfo(spellID)
		
		if name and icon then
			print("|T"..icon..":15:15:0:0:64:64:5:59:5:59|t",name.." - has been removed!")
		end
	end
end

function SyncUI_OptionPanel_ReactiveAura_AuraLine_Update(self)
	if not SyncUI_CharData["WeakAuras"][self:GetID()] then self:Hide() return end

	local spellID = SyncUI_CharData["WeakAuras"][self:GetID()]
	local name = select(1, GetSpellInfo(spellID))
	local icon = select(3, GetSpellInfo(spellID))

	if name then
		self.Icon:SetTexture(icon)
		self.Name:SetText(name)
	end
	
	if not self:IsShown() then self:Show() end
end

function SyncUI_OptionPanel_ReactiveAura_UpdateScrollFrame(self)
	local offset = FauxScrollFrame_GetOffset(self)
	local maxLines = 7
	local size = #SyncUI_CharData["WeakAuras"]
	
	FauxScrollFrame_Update(self, size, maxLines, 20)
	
	for i = 1, maxLines do
		local line = SyncUI_OptionPanel_ReactiveAura["Line"..i]
		local idx = offset + i

		line:SetID(idx)
		
		SyncUI_OptionPanel_ReactiveAura_AuraLine_Update(line)
	end
	
	if GetMouseFocus() == lineOnFocus then
		SyncUI_OptionPanel_Reactiveaura_AuraLine_DeleteButton_OnEnter(lineOnFocus)
	end
end

function SyncUI_OptionPanel_ReactiveAura_ApplyAura(self)
	local editBox = self:GetParent().Enter
	local text = editBox:GetText()
	local spellID = gsub(text,"%a+","")
	
	if spellID then
		local name, _, icon = GetSpellInfo(spellID)
		
		for _, spell in pairs(SyncUI_CharData["WeakAuras"]) do
			if spell == spellID then
				editBox:SetText("")

				return print("|T"..icon..":15:15:0:0:64:64:5:59:5:59|t",name.." - is already in the database!")
			end
		end

		if name then
			local scrollFrame = self:GetParent().ScrollFrame

			-- Add Aura
			tinsert(SyncUI_CharData["WeakAuras"],spellID)
			
			-- Sort Table
			table.sort(SyncUI_CharData["WeakAuras"],SortAuras)
			
			-- Update Scroll Frame
			scrollFrame.ScrollBar:SetValue(0)
			SyncUI_OptionPanel_ReactiveAura_UpdateScrollFrame(scrollFrame)
			
			-- Clear EditBox
			editBox:SetText("")
			
			-- Print
			print("|T"..icon..":15:15:0:0:64:64:5:59:5:59|t",name.." - has been added!")
		else
			if spellID == "" then
				print("Enter a spellID to add an aura!")
			else
				print("|cFF00FF99"..spellID.."|cFFFFFFFF - is no valid spellID!")
			end
		end
	end
end


-- Font Panel --
function SyncUI_SetUIFont()
	local index = SyncUI_Data.Options.UsedFont
	local name,tiny,small,medium,large,huge = unpack(SYNCUI_FONTLIST[index])
	local path = [[Interface\AddOns\SyncUI\Media\Fonts\]]..name

	SyncUI_FontPath = path
	
	do	-- SyncUI Fonts
		SyncUI_GameFontNormal_Tiny:SetFont(path,tiny,"OUTLINE")
		SyncUI_GameFontNormal_Small:SetFont(path,small,"OUTLINE")
		SyncUI_GameFontNormal_Medium:SetFont(path,medium,"OUTLINE")
		SyncUI_GameFontNormal_Large:SetFont(path,large,"OUTLINE")
		SyncUI_GameFontNormal_Huge:SetFont(path,huge,"OUTLINE")

		SyncUI_GameFontNormal_Small:SetShadowColor(0,0,0)
		SyncUI_GameFontNormal_Small:SetShadowOffset(0,0)
		SyncUI_GameFontNormal_Medium:SetShadowColor(0,0,0)
		SyncUI_GameFontNormal_Medium:SetShadowOffset(0,0)
		SyncUI_GameFontNormal_Large:SetShadowColor(0,0,0)
		SyncUI_GameFontNormal_Large:SetShadowOffset(0,0)
		SyncUI_GameFontNormal_Huge:SetShadowColor(0,0,0)
		SyncUI_GameFontNormal_Huge:SetShadowOffset(0,0)
		
		SyncUI_GameFontOutline_Small:SetFont(path,small,"NONE")
		SyncUI_GameFontOutline_Medium:SetFont(path,medium,"NONE")
		SyncUI_GameFontOutline_Large:SetFont(path,large,"NONE")
		SyncUI_GameFontOutline_Huge:SetFont(path,huge,"NONE")
		
		SyncUI_GameFontShadow_Small:SetFont(path,small,"NONE")
		SyncUI_GameFontShadow_Medium:SetFont(path,medium,"NONE")
		SyncUI_GameFontShadow_Large:SetFont(path,large,"NONE")
		SyncUI_GameFontShadow_Huge:SetFont(path,huge,"NONE")
		
		SyncUI_GameFontShadow_Small:SetShadowColor(0,0,0)
		SyncUI_GameFontShadow_Small:SetShadowOffset(1,-1)
		SyncUI_GameFontShadow_Medium:SetShadowColor(0,0,0)
		SyncUI_GameFontShadow_Medium:SetShadowOffset(1,-1)
		SyncUI_GameFontShadow_Large:SetShadowColor(0,0,0)
		SyncUI_GameFontShadow_Large:SetShadowOffset(1,-1)
		SyncUI_GameFontShadow_Huge:SetShadowColor(0,0,0)
		SyncUI_GameFontShadow_Huge:SetShadowOffset(1,-1)
	end
	
	do	-- Blizzard Fonts
		for i = 1, NUM_CHAT_WINDOWS do
			local chatWindow = _G["ChatFrame"..i]
			
			if chatWindow then
				local height = select(2, chatWindow:GetFont())
				local isDev = SyncUI_IsDev()
				
				if isDev then
					path = [[Interface\AddOns\SyncUI\Media\Fonts\]].."Haas.ttf"
				end
				
				chatWindow:SetFont(path,height,"NONE")
				chatWindow:SetShadowColor(0,0,0)
				chatWindow:SetShadowOffset(1,-1)
			end
		end
	end
end

function SyncUI_OptionPanel_Font_FontCheckButton_OnClick(self,button)
	local parent = self:GetParent()
	local maxButtons = 6
	
	for index = 1, maxButtons do
		local button = _G[parent:GetName()]["Button"..index]
		button:SetChecked(false)
	end
	
	self:SetChecked(true)
	
	SyncUI_Data.Options.UsedFont = self:GetID()
	SyncUI_SetUIFont()
end

function SyncUI_OptionPanel_Font_FontCheckButton_Update(self)
	local index = self:GetID()
	local text = select(1, unpack(SYNCUI_FONTLIST[index]))
	local language = select(7, unpack(SYNCUI_FONTLIST[index]))
	
	if index <= maxFonts then
		self:Show()
	else
		self:Hide()
	end
	
	if text then
		--text = gsub(text,".otf","")
		--text = gsub(text,".ttf","")
		if language then
			text = text.." - "..language
		end
		self.Text:SetText(text)
	end
	
	if index == SyncUI_Data.Options.UsedFont then
		self:SetChecked(true)
	else
		self:SetChecked(false)
	end
end

function SyncUI_OptionPanel_Font_UpdateScrollFrame(self)
	local offset = FauxScrollFrame_GetOffset(self)
	local maxButtons = 6
	
	FauxScrollFrame_Update(self, maxFonts, maxButtons, 20)
	
	for i = 1, maxButtons do
		local button = SyncUI_OptionPanel_Font["Button"..i]
		local idx = offset + i

		button:SetID(idx)
		
		SyncUI_OptionPanel_Font_FontCheckButton_Update(button)
	end
end



-- Unit Frames Panel --
local function GetMethodID(sortBy)
	if sortBy == "GROUP" then
		return 1
	elseif sortBy == "CLASS" then
		return 2
	elseif sortBy == "ROLE" then
		return 3
	else
		return 0
	end
end

function SyncUI_OptionPanel_UnitFrames_SelectSortMethod(sortBy)
	local numMethods = 3
	local methodID = GetMethodID(sortBy)

	for i = 1, numMethods do
		local frame = SyncUI_OptionPanel_UnitFrames["SortMethod"..i]

		if i == methodID then
			frame:SetChecked(true)
		else
			frame:SetChecked(false)
		end
	end

	SyncUI_RaidFrameContainer_UpdateSorting(sortBy)
	SyncUI_Data.Options.GroupSortMethod = sortBy
end


-- Popup Dialogs
StaticPopupDialogs["SYNCUI_USEDEFAULTRAID"] = {
	text = "Do you want to use blizzard raid frames instead?",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function()
		EnableAddOn("Blizzard_CompactRaidFrames")
		EnableAddOn("Blizzard_CUFProfiles")
		SyncUI_Data.Options.DefaultRaid = true
		ReloadUI()
	end,
	OnCancel = function()
		SyncUI_Data.Options.DefaultRaid = false
		SyncUI_OptionPanel_UnitFrames.DefaultRaid:SetChecked(false)
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,
}
StaticPopupDialogs["SYNCUI_NOTUSEDEFAULTRAID"] = {
	text = "Do you want to disable blizzard raid frames?",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function()
		DisableAddOn("Blizzard_CompactRaidFrames")
		DisableAddOn("Blizzard_CUFProfiles")
		SyncUI_Data.Options.DefaultRaid = false
		ReloadUI()
	end,
	OnCancel = function()
		SyncUI_Data.Options.DefaultRaid = true
		SyncUI_OptionPanel_UnitFrames.DefaultRaid:SetChecked(true)
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,
}
StaticPopupDialogs["SYNCUI_INTEGRATESKADA"] = {
	text = "Do you want to integrate Skada into the Chat?",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function()
		SyncUI_Data.Options.SkadaChat = true
		ReloadUI()
	end,
	OnCancel = function()
		SyncUI_Data.Options.SkadaChat = false
		SyncUI_OptionPanel_Meter.Skada:SetChecked(false)
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,
}
StaticPopupDialogs["SYNCUI_DEFAULTSKADA"] = {
	text = "Do you want to remove Skada from the Chat?",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function()
		SyncUI_Data.Options.SkadaChat = false
		ReloadUI()
	end,
	OnCancel = function()
		SyncUI_Data.Options.SkadaChat = true
		SyncUI_OptionPanel_Meter.Skada:SetChecked(true)
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,
}
StaticPopupDialogs["SYNCUI_INTEGRATERECOUNT"] = {
	text = "Do you want to integrate Recount into the Chat?",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function()
		SyncUI_Data.Options.RecountChat = true
		ReloadUI()
	end,
	OnCancel = function()
		SyncUI_Data.Options.RecountChat = false
		SyncUI_OptionPanel_Meter.Recount:SetChecked(false)
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,
}
StaticPopupDialogs["SYNCUI_DEFAULTRECOUNT"] = {
	text = "Do you want to remove Recount from the Chat?",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function()
		SyncUI_Data.Options.RecountChat = false
		ReloadUI()
	end,
	OnCancel = function()
		SyncUI_Data.Options.RecountChat = true
		SyncUI_OptionPanel_Meter.Recount:SetChecked(true)
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,
}
