
local specID;
local DragFrames = {}
local defaultPos = {
	-- UnitFrames
	["SyncUI_PlayerFrame"] = {"BOTTOM","SyncUI_MainMenuBar","TOP",-200,100},
	["SyncUI_TargetFrame"] = {"BOTTOM","SyncUI_MainMenuBar","TOP",200,100},
	["SyncUI_FocusFrame"] = {"LEFT","SyncUI_UIParent","LEFT",250,0},
	["SyncUI_PetFrame"] = {"BOTTOM","SyncUI_MainMenuBar","BOTTOM",-296,96},	
	["SyncUI_ToTFrame"] = {"TOPLEFT","SyncUI_TargetFrame","BOTTOMRIGHT",-5,5},
	["SyncUI_ToFFrame"] = {"BOTTOMRIGHT","SyncUI_FocusFrame","TOPLEFT",5,-5},
	["SyncUI_BossFrameContainer"] = {"RIGHT","SyncUI_UIParent","RIGHT",-150,50},
	["SyncUI_ArenaFrameContainer"] = {"RIGHT","SyncUI_UIParent","RIGHT",-150,50},
	["SyncUI_PartyFrameContainer"] = {"LEFT","SyncUI_UIParent","LEFT",60,60},
	["SyncUI_PartyFrameContainer_Heal"] = {"BOTTOM","SyncUI_MainMenuBar","TOP",0,50},
	["SyncUI_RaidFrameContainer_Heal"] = {"BOTTOMLEFT","SyncUI_UIParent","BOTTOMLEFT",20,275},
	
	-- Other UI Elements
	["SyncUI_PlayerFrame_CastingBar"] = {"BOTTOM","SyncUI_MainMenuBar","TOP",0,5},
	["SyncUI_PlayerFrame_CastHistory"] = {"BOTTOMLEFT","SyncUI_SideBar","TOPLEFT",0,0},
	["SyncUI_PlayerFrame_AuraFrame"] = {"BOTTOMLEFT","SyncUI_PlayerFrame","TOPLEFT",0,-2},
	["SyncUI_EnvironmentFrame"] = {"TOP","SyncUI_UIParent","TOP",0,-45},
	["SyncUI_AlternatePowerBarFrame"] = {"BOTTOM","SyncUI_MainMenuBar","TOP",0,0},
	["SyncUI_DraenorZoneAbility"] = {"BOTTOM","SyncUI_MainMenuBar","TOP",0,130},
	["SyncUI_ReadyCheckWindow"] = {"TOPLEFT","SyncUI_UIParent","TOPLEFT",230,-15},
	["SyncUI_WorldMarkerBar"] = {"BOTTOMRIGHT","SyncUI_SideBar","TOPRIGHT",0,0},
	["SyncUI_QuestHub"] = {"BOTTOMRIGHT","SyncUI_UIParent","BOTTOMRIGHT",-65,138},
	
	-- Ultra Wide Resolutions (Only)
	["SyncUI_Minimap"] = {"TOPRIGHT","SyncUI_UIParent","TOPRIGHT",-5,-5},
	["SyncUI_ChatFrame"] = {"BOTTOMLEFT","SyncUI_UIParent","BOTTOMLEFT",25,45},
	
	-- Blizzard Frames
	["VehicleSeatIndicator"] = {"BOTTOMLEFT","SyncUI_VehicleBar","TOPLEFT",-160,-60},
}
local specialPos = {
	["SyncUI_PlayerFrame_CastingBar"] = {"BOTTOM","SyncUI_AlternatePowerBarFrame","TOP"}
}

local function isSpecial(frameName)
	local info = specialPos[frameName]
	
	if info then
		local point, relativeTo, relativePoint = unpack(info)
		return true, point, _G[relativeTo], relativePoint
	end
end

local function UpdateButtonText()
	local button = SyncUI_PlacementTool.SpecButton
	local activeSpec = GetSpecialization(nil,nil,specID)
	local text;
	
	if activeSpec then
		local icon = select(4, GetSpecializationInfo(activeSpec))
		local name = select(2, GetSpecializationInfo(activeSpec))
		
		text = tostring("|T"..icon..":20:20:0:0:64:64:4:60:4:60|t "..name)
	elseif not activeSpec then
		local icon = "Interface\\Icons\\trade_engineering"
		
		text = "|T"..icon..":20:20:0:0:64:64:4:60:4:60|t ".."Unknown"
	end
	
	button:SetText(text)
end

local function LoadFramePositions(specID)
	local dataBase = SyncUI_CharData.FramePos[specID]

	-- Pre Reset All Frame Positions
	for frameName, info in pairs(defaultPos) do
		local frame = _G[frameName]
		local isSpecial, point, relativeTo, relativePoint = isSpecial(frameName)
		
		if frame then
			frame:ClearAllPoints()
			
			if isSpecial and relativeTo and relativeTo:IsShown() then 
				frame:SetPoint(point, relativeTo, relativePoint)
			else
				frame:SetPoint(unpack(info))
			end
			
			frame.isPlaced = nil
		end
	end
	
	-- Load Saved Frame Positions
	for frameName, info in pairs(dataBase) do
		local frame = _G[frameName]

		if frame then
			frame:ClearAllPoints()
			frame:SetPoint(unpack(info))
			frame.isPlaced = true
		end
	end
end

local function ResetFramePositions()
	PlaySound("igMainMenuOptionCheckBoxOn")
	
	for parent, dragFrame in pairs(DragFrames) do
		local info = defaultPos[parent:GetName()]
		
		if info then
			SyncUI_CharData.FramePos[specID][parent:GetName()] = nil
			parent.isPlaced = nil
			parent:ClearAllPoints()
			parent:SetPoint(unpack(info))
		end
	end
end

local function OnDragStart(self)
	local parent = self.parent
	
	self.Flashing:Play()
	
	parent:StartMoving()
	parent.isMoving = true	
end

local function OnDragStop(self)
	local parent = self.parent
	
	self.Flashing:Stop()
	
	parent:StopMovingOrSizing()
	parent:SetUserPlaced(false)
	parent.isMoving = false
	parent.isPlaced = true
	
	SyncUI_CharData.FramePos[specID][parent:GetName()] = {parent:GetPoint()}
end


-- Drag Frames
function SyncUI_RegisterDragFrame(parent,name,securefunc)
	local dragFrame = CreateFrame("Button",nil,SyncUI_UIParent,"SyncUI_DragFrameTemplate")
	
	dragFrame:Hide()
	dragFrame:SetPoint("TOPLEFT",parent)
	dragFrame:SetPoint("BOTTOMRIGHT",parent)
	dragFrame:SetScript("OnDragStart", OnDragStart)
	dragFrame:SetScript("OnDragStop", OnDragStop)
	dragFrame.Header:SetText(name)
	dragFrame.parent = parent
	dragFrame.name = name

	if securefunc then
		hooksecurefunc(parent, "SetPoint", function(self)
			local dataBase = SyncUI_CharData.FramePos[specID][self:GetName()]
			local point, relativeTo, relativePoint, xPos, yPos

			if dataBase then	
				point, relativeTo, relativePoint, xPos, yPos = unpack(dataBase)
			else
				point, relativeTo, relativePoint, xPos, yPos = unpack(defaultPos[self:GetName()])
			end

			self:ClearAllPoints()
			getmetatable(self).__index.SetPoint(self,point, relativeTo, relativePoint, xPos, yPos)
		end)
	end
	
	parent:SetClampedToScreen(true)
	parent:SetMovable(true)
	parent.DragFrame = dragFrame

	DragFrames[parent] = parent.DragFrame
end

function SyncUI_UnregisterDragFrame(parent)
	DragFrames[parent] = nil
end

function SyncUI_DragFrame_OnEnter(self)
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	GameTooltip:AddLine(self.name,1,1,1)
	GameTooltip:SetPrevLineJustify("CENTER")
	GameTooltip:AddDivider()
	GameTooltip:AddLine("|cFF00FF99Drag |r".."to move frame.",1,1,1)
	GameTooltip:AddLine("|cFF00FF99Shift + Right-Click |r".."to reset position.",1,1,1)
	GameTooltip:Show()
	
	self.Border:SetTexture(1,1,1,0.5)
end

function SyncUI_DragFrame_OnLeave(self)
	GameTooltip_Hide()
	
	self.Border:SetTexture(0,1,0.6,0.5)
end

function SyncUI_DragFrame_OnClick(self,button)
	if button == "RightButton" and IsShiftKeyDown() then
		for parent, dragFrame in pairs(DragFrames) do
			if self == dragFrame then
				local info = defaultPos[parent:GetName()]
			
				if info then
					SyncUI_CharData.FramePos[specID][parent:GetName()] = nil
					parent.isPlaced = false
					parent:ClearAllPoints()
					parent:SetPoint(unpack(info))
				end
			end
		end
	end
end


-- Placement Tool
function SyncUI_PlacementTool_OnLoad(self)
	self:SetScale(SyncUI_UIScale)
	self:RegisterForDrag("LeftButton")
	self:RegisterEvent("ADDON_LOADED")
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	UIPanelWindows[self:GetName()] = { area = "center", pushable = 0, whileDead = 1 }
end

function SyncUI_PlacementTool_OnEvent(self,event,arg1)
	if event == "ADDON_LOADED" and arg1:find("SyncUI") then
		if not SyncUI_CharData.FramePos then
			SyncUI_CharData.FramePos = {
				[1] = {},
				[2] = {},
			}
		end

		specID = GetActiveSpecGroup()
		
		LoadFramePositions(specID)
	end
	
	if event == "ACTIVE_TALENT_GROUP_CHANGED" then
		specID = arg1
		
		LoadFramePositions(specID)
	end
	
	if event == "PLAYER_REGEN_DISABLED" then
		HideUIPanel(self)
	end
end

function SyncUI_PlacementTool_OnShow(self)
	for parent, dragFrame in pairs(DragFrames) do
		dragFrame:Show()
	end
	
	specID = GetActiveSpecGroup()

	UpdateButtonText()
	
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:ClearAllPoints()
	self:SetPoint("TOP",0,-200)
end

function SyncUI_PlacementTool_OnHide(self)
	for parent, dragFrame in pairs(DragFrames) do
		dragFrame:Hide()
		parent:StopMovingOrSizing()
		parent:SetUserPlaced(false)
	end

	if not InCombatLockdown() then
		specID = GetActiveSpecGroup()
		LoadFramePositions(specID)
	end
	
	self.ChooseSpec:Hide()
	self:UnregisterEvent("PLAYER_REGEN_DISABLED")
	StaticPopup_Hide("SYNCI_RESET_POSITIONS")
	OptionsFrame_OnHide(self)
end

function SyncUI_PlacementTool_SelectSpec(self)
	local parent = self:GetParent()
	
	specID = self:GetID()
	
	UpdateButtonText()
	LoadFramePositions(specID)

	parent:Hide()
end

StaticPopupDialogs["SYNCI_RESET_POSITIONS"] = {
	text = "Do you want to reset the position of all frames?",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function()
		ResetFramePositions()
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,
}