
local L = {}
local locale = GetLocale()

if locale == "deDE" then
	L.Consumable = "Verbrauchbar"
	L.TradeGoods = "Handwerkswaren"
elseif locale == "ptBR" then
	L.Consumable = "Consumível"
	L.TradeGoods = "Mercadorias"
elseif locale == "ruRU" then
	L.Consumable = "Расходуемые"
	L.TradeGoods = "Хозяйственные товары"
elseif locale == "esES" then
	L.Consumable = "Consumible"
	L.TradeGoods = "Objeto comerciable"
elseif locale == "itIT" then
	L.Consumable = "Consumabili"
	L.TradeGoods = "Beni commerciali"
elseif locale == "frFR" then
	L.Consumable = "Consommable"
	L.TradeGoods = "Artisanat"
else
	L.Consumable = "Consumable"
	L.TradeGoods = "Trade Goods"
end

local _G = _G
local tinsert, tremove = table.insert, table.remove
local ItemCategories = {
	[1] = "Inventory",
	[2] = "New",
	[3] = "Trade Goods",
	[4] = "Consumables",
	[5] = "Equipment",
}

-- Misc functions
local function SellGreyItems()
	for bagID = 0, NUM_BAG_SLOTS do
		local numSlots = GetContainerNumSlots(bagID)
		for slot = 1, numSlots do
			local itemID = GetContainerItemID(bagID,slot)
			
			if itemID then
				local quality = select(3, GetItemInfo(itemID))
				
				if quality and quality == 0 then
					UseContainerItem(bagID,slot)
				end
			end
		end
	end
end

local function SortContainer(a,b)
	if a == nil or b == nil then
		return false
	end
	
	if a.rarity ~= b.rarity then
		if a.rarity and b.rarity then
			return a.rarity > b.rarity
		elseif a.rarity == nil or b.rarity == nil then
			return a.rarity and true or false
		else
			return false
		end
	elseif a.ilvl ~= b.ilvl then
		if a.ilvl and b.ilvl then
			return a.ilvl > b.ilvl
		elseif a.ilvl == nil or b.ilvl == nil then
			return a.ilvl and true or false
		else
			return false
		end
	elseif a.type ~= b.type then
		return a.type < b.type
	elseif a.name ~= b.name then
		return a.name < b.name
	else
		return a.stack < b.stack
	end
end

local function SortInventoryFreeSlot(self)
	if not self.FreeSlot then return end
	
	for k, v in pairs(self.buttons) do
		if v == self.FreeSlot then
			tremove(self.buttons,k)
		end
	end
	
	tinsert(self.buttons,self.FreeSlot)
end

local function isBank(bagID)
	for _, value in pairs({-1,5,6,7,8,9,10,11}) do
		if bagID == value then
			return true
		end
	end
	
	return false
end

local function isEnabled(container,button)
	if container.buttons then
		for _, ref in pairs(container.buttons) do
			if ref == button then
				return false
			end
		end

		return true
	end
end


-- Assign / Remove Functions
local function CreateNewContainer()
	local dataBase = SyncUI_Data["Bags"]["ContainerList"]
	local numDefault = #ItemCategories
	local numTotal = numDefault + #dataBase or 0
	
	-- Bags
	for index = numDefault + 1, numTotal do
		do -- Bags
			local container = _G["SyncUI_BagFrame_Container"..index]
			local parent = SyncUI_BagFrame_ScrollFrame.ScrollChild
			
			if not container then
				local prevFrame = _G["SyncUI_BankFrame_Container"..index-1]
				
				container = CreateFrame("Frame", "SyncUI_BagFrame_Container"..index, parent, "SyncUI_ItemContainerTemplate")
				container:SetPoint("TOPLEFT", prevFrame, "BOTTOMLEFT")
				container:SetID(index)
				container:Hide()
			end
		end
		
		do -- Bank
			local container = _G["SyncUI_BankFrame_Container"..index]
			local parent = SyncUI_BankFrame_ScrollFrame.ScrollChild
			
			if not container then
				container = CreateFrame("Frame","SyncUI_BankFrame_Container"..index,parent,"SyncUI_ItemContainerTemplate")
				container:SetPoint("TOPLEFT",_G["SyncUI_BankFrame_Container"..index-1],"BOTTOMLEFT")
				container:SetID(index)
				container:Hide()
			end
		end
	end
end

local function AddButtonToContainer(container,button)
	table.insert(container.buttons,button)
	button.container = container
end

local function RemoveButtonFromContainer(container,button)
	for i, ref in pairs(container.buttons) do
		if button == ref then
			table.remove(container.buttons,i)
		end
	end
end


-- Get Info Functions
local function isEquip(itemID)
	local isEquip = select(9, GetItemInfo(itemID))
	
	if isEquip ~= "" and isEquip ~= "INVTYPE_BAG" then
		return true
	end
	
	return
end

local function GetNumContainers()
	local dataBase = SyncUI_Data["Bags"]["ContainerList"]
	local standard = #ItemCategories
	local custom = #dataBase or 0
	
	return standard + custom
end

local function GetDefaultContainerID(itemID)
	local quality = select(3, GetItemInfo(itemID))
	local itemType = select(6, GetItemInfo(itemID))
	local isEquip = isEquip(itemID)
	
	if quality == 0 then
		return 1
	elseif itemType == L.TradeGoods then
		return 3
	elseif itemType == L.Consumable then
		return 4
	elseif isEquip then
		return 5
	else
		return 1
	end
end

local function GetContainerID(container,itemID)
	for index, name in pairs(SyncUI_Data["Bags"]["ContainerList"]) do
		if container == name then
			return index + #ItemCategories
		end
	end
	
	return GetDefaultContainerID(itemID)
end

local function GetContainerByItemID(itemID,isBank)
	local dataBase = SyncUI_Data
	local container, index
	
	if dataBase then
		container = dataBase["Bags"]["ItemList"][itemID]
	end
	
	if container then
		index = GetContainerID(container,itemID)
	else	
		index = GetDefaultContainerID(itemID)
	end
	
	if isBank then
		return _G["SyncUI_BankFrame_Container"..index]
	else
		return _G["SyncUI_BagFrame_Container"..index]
	end
end

local function GetContainerForItem(bagID,slot)
	local itemID = GetContainerItemID(bagID,slot)
	local isNewItem = C_NewItems.IsNewItem(bagID,slot)
	
	if isNewItem and SyncUI_Data and SyncUI_Data.Options.ShowNewItems then
		return SyncUI_BagFrame_Container2
	elseif itemID then
		return GetContainerByItemID(itemID,isBank(bagID))
	end
end

local function GetNumFreeSlots(isBank)
	totalFreeSlots = 0
	
	if isBank then
		for bagID = 5, 11 do
			totalFreeSlots = totalFreeSlots + GetContainerNumFreeSlots(bagID)
		end
		
		totalFreeSlots = totalFreeSlots + GetContainerNumFreeSlots(BANK_CONTAINER)
	else
		for bagID = 0, 4 do
			totalFreeSlots = totalFreeSlots + GetContainerNumFreeSlots(bagID)
		end
	end
	
	return totalFreeSlots
end

local function GetPrevContainer(self,containerID,isBank)
	local index = containerID - 1
	local container;
	
	if index < 1 then return end
	
	repeat
		if isBank then
			container = _G["SyncUI_BankFrame_Container"..index]
		else
			container = _G["SyncUI_BagFrame_Container"..index]
		end
		
		if container:IsShown() then
			return container:GetName()
		else
			index = index - 1
		end

	until index == 0
end

local function GetButton(bagID,slot)
	if bagID and slot then
		if isBank(bagID) then
			if bagID == -1 then
				return _G["SyncUI_BankFrame_Bag1_Slot"..slot]
			else
				return _G["SyncUI_BankFrame_Bag"..bagID-3 .."_Slot"..slot]
			end
		else
			return _G["SyncUI_BagFrame_Bag"..bagID+1 .."_Slot"..slot]
		end
	end
end


-- Update Functions
local function UpdateSlot(bagID,slot)
	local button = GetButton(bagID,slot)
	local container = GetContainerForItem(bagID,slot)
	
	if button then
		SyncUI_ItemSlot_ForceUpdate(button,bagID,slot)
	end
		
	if container then
		if button then
			local isEnabled = isEnabled(container,button)		
			
			if button.container and button.container ~= container then
				RemoveButtonFromContainer(button.container,button)
				SyncUI_ItemContainer_ForceUpdate(button.container)
			end
			
			if isEnabled then
				AddButtonToContainer(container,button)
				SyncUI_ItemContainer_ForceUpdate(container)
			end
		end
	elseif button and button.container then
		RemoveButtonFromContainer(button.container,button)
		SyncUI_ItemContainer_ForceUpdate(button.container)
	end
end

local function UpdateBag(bagID)
	local numSlots = GetContainerNumSlots(bagID)
	
	for slot = 1, numSlots do
		UpdateSlot(bagID,slot)
	end
end

local function UpdateAllBags(isBank)
	if isBank then
		if not SyncUI_BankFrame:IsShown() then
			return
		end
		
		for _, bagID in pairs({-1,5,6,7,8,9,10,11}) do
			UpdateBag(bagID)
		end
	else
		for bagID = 0, 4 do
			UpdateBag(bagID)
		end
	end
end

local function UpdateAllContainers(isBank)
	for i = 1, GetNumContainers() do
		local container;
		
		if isBank then
			container = _G["SyncUI_BankFrame_Container"..i]
		else
			container = _G["SyncUI_BagFrame_Container"..i]
		end
		
		SyncUI_ItemContainer_ForceUpdate(container)
	end
end

local function UpdateBankBagSlots()
	local numSlots, full = GetNumBankSlots()
	local cost = GetBankSlotCost(numSlots)
	local purchaseButton = SyncUI_BankFrame_BagBar_PurchaseButton
	
	for i = 1, NUM_BANKBAGSLOTS, 1 do
		local button = _G["SyncUI_BankFrame_BagBar_Bag"..i]

		local textureName = GetInventoryItemTexture("player",button:GetInventorySlot())
		local slotTextureName = select(2, GetInventorySlotInfo("Bag"..i))
		
		if textureName then
			button.icon:SetTexture(textureName)
			button.icon:Show()
			SetItemButtonCount(button,GetInventoryItemCount("player", button:GetInventorySlot()))
			button.hasItem = 1
		elseif slotTextureName and button.isBag then
			button.icon:SetTexture(slotTextureName)

			SetItemButtonCount(button,0)
			button.icon:Show()
		end
		
		if i <= numSlots then
			SetItemButtonTextureVertexColor(button, 1.0,1.0,1.0)
			button.tooltipText = BANK_BAG
		else
			SetItemButtonTextureVertexColor(button, 1.0,0.1,0.1)
			button.tooltipText = BANK_BAG_PURCHASE
		end
	end
	
	BankFrame.nextSlotCost = cost
	
	if full then
		purchaseButton:Hide()
		SyncUI_BankFrame_BagBar.border:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\Bags\\Bank_BagBar_Border2")
	else
		purchaseButton:Show()
	end
end

local function UpdateInventoryFreeSlot(self,isBank)
	local container = _G[self:GetName().."_Container1"]
	container.FreeSlot.Count:SetText(GetNumFreeSlots(isBank))	
end

local function UpdateSearchResults(frame)
	local bagID = frame:GetID()
	local numSlots = GetContainerNumSlots(bagID)
	
	for slot = 1, numSlots do
		local button = _G[frame:GetName().."_Slot"..slot]
		local isFiltered = select(8, GetContainerItemInfo(bagID, slot))
		if isFiltered then
			button.searchOverlay:Show()
		else
			button.searchOverlay:Hide()
		end
	end
end

local function UpdateFrameSize(self,height,isBank)
	local scrollFrame = self:GetParent():GetParent()
	local width, topEdge, bottomEdge, minHeight, maxHeight
	
	if isBank then
		width, topEdge, bottomEdge, minHeight, maxHeight = 464, 28, 50, 263,37*15+78
	else
		width, topEdge, bottomEdge, minHeight, maxHeight = 353, 28, 41, 254,37*15+69
	end
	
	height = height + topEdge + bottomEdge
	
	if height > maxHeight then
		height = maxHeight
	elseif height < minHeight then
		height = minHeight
	end
	
	scrollFrame:SetSize(width,height)
end

local function UpdateScrollChild(self,isBank)
	local scrollChild = self:GetParent()
	local container;
	local height = 0
	
	for i = 1, GetNumContainers() do
		if isBank then
			container = _G["SyncUI_BankFrame_Container"..i]
		else
			container = _G["SyncUI_BagFrame_Container"..i]
		end
		
		if container and container:IsShown() then
			height = height + select(2, container:GetSize())
		end
	end

	-- Result is always divisible by 37
	height = math.ceil(height)
	height = 37 * (math.floor(height / 37))

	scrollChild:SetHeight(height)
	UpdateFrameSize(scrollChild,height,isBank)
end

local function UpdateContainerAnchors(isBank)
	local container, containerID, containerAnchor;
	
	for i = 1, GetNumContainers() do
		if isBank then
			container = _G["SyncUI_BankFrame_Container"..i]
		else
			container = _G["SyncUI_BagFrame_Container"..i]
		end

		if container and container:IsShown() then
			containerID = container:GetID()
			
			if containerID ~= 1 then
				containerAnchor = GetPrevContainer(container,containerID,isBank)

				container:ClearAllPoints()
				container:SetPoint("TOPLEFT",containerAnchor,"BOTTOMLEFT")
			end
		end
	end
end


-- Global Functions
local rowMaxBag, rowMaxBank = 7, 10

function SyncUI_ItemSlot_SetStyle(self)
	local bagID = self:GetParent():GetID()
	local slot = self:GetID()

	self:SetSize(37,37)
	self:SetNormalTexture("")
	self.IconBorder:SetTexture("")
	
	if self.BattlepayItemTexture then
		self.BattlepayItemTexture:Hide()
	end

	self.icon:SetSize(33,33)
	self.icon:SetDrawLayer("BACKGROUND")
	self.icon:ClearAllPoints()
	self.icon:SetPoint("CENTER")
	self.icon:SetTexCoord(0.05,0.95,0.05,0.95)

	self.Count:SetFontObject(SyncUI_GameFontNormal_Medium)
	self.Count:ClearAllPoints()
	self.Count:SetPoint("BOTTOMRIGHT",-2,4)
	
	self.searchOverlay:SetSize(33,33)
	self.searchOverlay:SetDrawLayer("OVERLAY")
	self.searchOverlay:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\Bags\\ItemSlot_ItemSearch")
	self.searchOverlay:SetTexCoord(0,0.578125,0,0.578125)
	
	self.highlight = self:GetHighlightTexture()
	self.highlight:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\Bags\\ItemSlot_ItemHighlight")
	self.highlight:SetTexCoord(0,0.578125,0,0.578125)
	
	-- Create New Textures
	self.iconTex = self:CreateTexture(nil,"BORDER")
	self.iconTex:SetSize(37,37)
	self.iconTex:SetPoint("CENTER")
	self.iconTex:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\Bags\\ItemSlot_ItemBorder")
	self.iconTex:SetTexCoord(0,0.578125,0,0.578125)
	
	self.quality = self:CreateTexture(nil,"ARTWORK")
	self.quality:SetSize(37,37)
	self.quality:SetPoint("CENTER")
	self.quality:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\Bags\\ItemSlot_ItemQuality")
	self.quality:SetTexCoord(0,0.578125,0,0.578125)
	
	self.quest = self:CreateTexture(nil,"ARTWORK")
	self.quest:SetSize(37,37)
	self.quest:SetPoint("CENTER")
	self.quest:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\Bags\\ItemSlot_ItemQuest")
	self.quest:SetTexCoord(0,0.578125,0,0.578125)
	self.quest:Hide()
	
	if not self.cooldown then
		self.cooldown = _G[self:GetName().."Cooldown"]
		SyncUI_Cooldown_OnRegister(self.cooldown)
	end
	
	if bagID == BANK_CONTAINER then
		self.GetInventorySlot = ButtonInventorySlot
		self.UpdateTooltip = SyncUI_BankSlot_OnEnter
		self:SetScript("OnEnter",SyncUI_BankSlot_OnEnter)
	end
	
	self.container = GetContainerForItem(bagID,slot)
	self:HookScript("OnClick", function(self,button)
		if IsControlKeyDown() and IsAltKeyDown() then
			local itemID = GetContainerItemID(bagID,slot)

			if #SyncUI_Data["Bags"]["ContainerList"] > 0 then
				SyncUI_AssignItemDropDown_Open(self,itemID)
			end
		end
	end)
end

function SyncUI_ItemSlot_ForceUpdate(self,bagID,slot)
	local texture,itemCount,locked,rarity,readable = GetContainerItemInfo(bagID, slot)
	local itemLink = select(7, GetContainerItemInfo(bagID, slot))
	local noValue = select(9, GetContainerItemInfo(bagID, slot))
	local itemID = GetContainerItemID(bagID,slot)
	
	
	SetItemButtonTexture(self, texture)
	SetItemButtonCount(self, itemCount)
	SetItemButtonDesaturated(self, locked)
	
	if itemLink then
		do	-- update indicators
			local isQuest,questID,isActive = GetContainerItemQuestInfo(bagID, slot)
			
			if isQuest or questID then
				self.quality:SetVertexColor(0.8,0.6,0)
				self.quality:Show()			
			elseif not rarity or rarity <= 1 then
				self.quality:Hide()
			else
				local c = ITEM_QUALITY_COLORS[rarity]
				self.quality:SetVertexColor(c.r,c.g,c.b)
				self.quality:Show()
			end
			
			if questID and not isActive then
				self.quest:Show()
			else
				self.quest:Hide()
			end

			if self.JunkIcon then
				if rarity == LE_ITEM_QUALITY_POOR and not noValue then
					self.JunkIcon:Show()
				else
					self.JunkIcon:Hide()
				end
			end
		end
		do	-- update sorting variables
			local isEquip = isEquip(itemID)
			local itemLevel = select(4, GetItemInfo(itemID))
			
			if itemID == 110560 or itemID == 6948 then
				self.rarity = 10
			else
				self.rarity = rarity
			end
			
			if isEquip then
				local itemLevelUpgrade = SyncUI_GetItemUpgradeLevel(itemLink)
				
				if itemLevelUpgrade then
					itemLevel = itemLevelUpgrade
				end
				
				SetItemButtonCount(self, itemLevel)
				self.Count:SetVertexColor(1,0.8,0)
			else
				self.Count:SetVertexColor(1,1,1)
			end
			
			self.ilvl = itemLevel
			self.type = select(7, GetItemInfo(itemID))
			self.name = select(1, GetItemInfo(itemID))
			self.stack = itemCount
		end

		self.hasItem = true
		self.readable = readable
		self:Show()
	else
		self.hasItem = nil
		self:Hide()
	end
end
 
function SyncUI_ItemSlot_OnEnter(self)
	local bagID, slot = self:GetParent():GetID(),self:GetID()
	local x = self:GetRight()

	if x >= (GetScreenWidth() / 2) then
		GameTooltip:SetOwner(self, "ANCHOR_LEFT")
	else
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	end

	local showSell = nil
	local hasCooldown, repairCost, speciesID, level, breedQuality, maxHealth, power, speed, name = GameTooltip:SetBagItem(bagID,slot)
	
	-- Battle Pets
	if speciesID and speciesID > 0 then
		BattlePetToolTip_Show(speciesID, level, breedQuality, maxHealth, power, speed, name)
		return
	else
		if BattlePetTooltip then
			BattlePetTooltip:Hide()
		end
	end

	-- Repair Mode
	if InRepairMode() and (repairCost and repairCost > 0) then
		GameTooltip:AddLine(REPAIR_COST, nil, nil, nil, true)
		SetTooltipMoney(GameTooltip, repairCost)
		GameTooltip:Show()
	elseif MerchantFrame:IsShown() and MerchantFrame.selectedTab == 1 then
		showSell = 1
	end

	-- Modifier
	if IsControlKeyDown() and IsAltKeyDown() then
		SetCursor("INTERACT_CURSOR")
	elseif IsModifiedClick("DRESSUP") and self.hasItem then
		ShowInspectCursor()
	elseif showSell then
		ShowContainerSellCursor(bagID,slot)
	elseif self.readable then
		ShowInspectCursor()
	else
		ResetCursor()
	end
end

function SyncUI_ItemBag_OnLoad(self)
	for i = 1, self:GetNumChildren() do
		local button = _G[self:GetName().."_Slot"..i]
		
		SyncUI_ItemSlot_SetStyle(button)
	end
end

function SyncUI_ItemContainer_OnLoad(self)
	local buttonID = self:GetID()
	
	self.buttons = {}

	if buttonID ~= 1 then
		self:Hide()
	end
end

function SyncUI_ItemContainer_ForceUpdate(self)
	if not self then return end
	
	local isBank = self:GetName():find("Bank")
	local buttonSize, rowMax = 37, rowMaxBag
	local numSlots = #self.buttons
	
	table.sort(self.buttons,SortContainer)
	SortInventoryFreeSlot(self)
	
	if isBank then
		rowMax = rowMaxBank
	end
		
	for i, button in pairs(self.buttons) do
		local pos = i % rowMax
		if pos == 0 then pos = rowMax end
		local row = math.ceil(i / rowMax)
			
		button:ClearAllPoints()
		button:SetPoint("TOPLEFT",self,buttonSize*(pos-1),-buttonSize-(buttonSize*(row-1)))
	end

	if (numSlots == 0 or numSlots == nil) and self:GetID() ~= 1 then
		self:Hide()
	elseif numSlots > 0 then
		local width = self:GetParent():GetSize()
		local numRows = math.ceil(numSlots / rowMax)
		local height = (numRows + 1) * buttonSize
		
		self:SetSize(width,height)
		self:Show()
	end

	do	-- Update Header
		local buttonID = self:GetID()
		local adjust = #ItemCategories
		local text
		
		if buttonID <= adjust and buttonID >= 1 then
			text = ItemCategories[self:GetID()]
		else
			text = SyncUI_Data["Bags"]["ContainerList"][buttonID - adjust]
		end
		
		self.Header:SetText(text)
		
		local width = math.ceil(self.Header:GetStringWidth())
		local space;
		
		if width % 2 == 0 then
			space = 14
		else
			space = 15
		end
		
		self.Header:SetWidth(width+space)
	end
	
	UpdateContainerAnchors(isBank)
	UpdateScrollChild(self,isBank)
end

function SyncUI_ItemContainer_FreeSlot_OnReceiveItem(self)
	local container = self:GetParent():GetName()
	local containerID = self:GetParent():GetID()
	local isBank = (container:find("Bank") and container:find("Container1"))
		
	if isBank then
		for _, bagID in pairs({-1,5,6,7,8,9,10,11}) do
			for slot = 1, GetContainerNumSlots(bagID) do
				if not GetContainerItemInfo(bagID,slot) then
					return PickupContainerItem(bagID, slot)
				end
			end
		end	
	elseif containerID == REAGENTBANK_CONTAINER then
		for slot = 1, GetContainerNumSlots(containerID) do
			if not GetContainerItemInfo(containerID,slot) then
				return PickupContainerItem(containerID,slot)
			end
		end
	else
		for bagID = 0, 4 do
			for slot = 1, GetContainerNumSlots(bagID) do
				if not GetContainerItemInfo(bagID,slot) then
					return PickupContainerItem(bagID, slot)
				end
			end
		end
	end
end

function SyncUI_ItemContainer_ClearNewItems()
	C_NewItems.ClearAll()
	UpdateAllBags()
end


do	-- Check Bag AddOns
	local BagAddons = {"Bagnon","ArkInventory","cargBags_Nivaya","AdiBags","Inventorian","LiteBag"}

	for _, addon in pairs(BagAddons) do
		if IsAddOnLoaded(addon) then
			SyncUI_OptionPanel_Misc.AutoSellJunk:Disable()
			
			return
		end
	end
end


-- Bag Functions
function SyncUI_BagFrame_OnLoad(self)
	tinsert(UISpecialFrames,self:GetName())
	
	hooksecurefunc("ToggleAllBags",SyncUI_BagFrame_Toggle)

	self:SetScale(SyncUI_UIScale)
	self:SetMinResize(353,200)
	self:SetMaxResize(353,439)
	
	self:RegisterEvent("INVENTORY_SEARCH_UPDATE")
	self:RegisterEvent("MERCHANT_SHOW")
	self:RegisterEvent("MERCHANT_CLOSED")
	self:RegisterEvent("MAIL_SHOW")
	self:RegisterEvent("MAIL_CLOSED")
	self:RegisterEvent("QUEST_ACCEPTED")
	self:RegisterEvent("QUEST_REMOVED")
	self:RegisterEvent("BAG_UPDATE")
	self:RegisterEvent("BAG_NEW_ITEMS_UPDATED")
	self:RegisterEvent("ADDON_LOADED")
	
	for i = 1, 13 do
		SyncUI_DisableFrame("ContainerFrame"..i)
	end
	
	UpdateNewItemList = function() end
	ContainerFrameItemButton_OnEnter = function() end
end

function SyncUI_BagFrame_OnEvent(self,event,...)
	if event == "INVENTORY_SEARCH_UPDATE" then
		for i = 1, 5 do
			local bag = _G[self:GetName().."_Bag"..i]
			UpdateSearchResults(bag)
		end
		
		for i = 1, 8 do
			local bag = _G["SyncUI_BankFrame_Bag"..i]
			UpdateSearchResults(bag)
		end

		UpdateSearchResults(SyncUI_BankFrame_ReagentBank)
	end
	
	if event == "MERCHANT_SHOW" or event == "MAIL_SHOW" then
		SyncUI_BagFrame:Show()
		
		if event == "MERCHANT_SHOW" and SyncUI_Data.Options.AutoSellJunk then
			SellGreyItems()
		end
	end
	
	if event == "MERCHANT_CLOSED" or event == "MAIL_CLOSED" then
		SyncUI_BagFrame:Hide()
	end

	if event == "QUEST_ACCEPTED" or event == "QUEST_REMOVED" then
		UpdateAllBags()
	end
	
	if event == "ITEM_LOCKED" then
		local button = GetButton(...)
		if button then
			button.icon:SetDesaturated(true)
		end
	end

	if event == "ITEM_UNLOCKED" then
		local button = GetButton(...)
		if button then
			button.icon:SetDesaturated(false)
		end
	end	

	if event == "BAG_UPDATE" then
		local bagID = ...
		
		UpdateBag(bagID)

		if bagID < 0 or bagID > 4 then
			UpdateInventoryFreeSlot(SyncUI_BankFrame,isBank)
		else
			UpdateInventoryFreeSlot(self,false)
		end
		
		if MerchantFrame:IsShown() and SyncUI_Data.Options.AutoSellJunk then
			SellGreyItems()
		end
	end

	if event == "BAG_NEW_ITEMS_UPDATED" then
		UpdateAllBags()
	end
	
	if event == "ADDON_LOADED" and ... == "SyncUI" then
		CreateNewContainer()
	end
end

function SyncUI_BagFrame_OnShow(self)
	self:RegisterEvent("ITEM_LOCKED")
	self:RegisterEvent("ITEM_UNLOCKED")
	
	if not self.firstTime then
		UpdateAllBags()
		UpdateAllContainers()
		UpdateInventoryFreeSlot(self,false)
		
		self.firstTime = true
	end
end

function SyncUI_BagFrame_OnHide(self)
	local dropDownMenu = SyncUI_AssignItem_DropDown
	local categoryMenu = SyncUI_BagFrame_CategoryMenu
	
	dropDownMenu:Hide()
	categoryMenu:Hide()
	
	self:UnregisterEvent("ITEM_LOCKED")
	self:UnregisterEvent("ITEM_UNLOCKED")
	
	if SyncUI_Data.Options.ClearNewItems then
		SyncUI_ItemContainer_ClearNewItems()
	end
end

function SyncUI_BagFrame_Toggle()
	if SyncUI_BagFrame:IsShown() and not BankFrame:IsShown() then
		SyncUI_BagFrame:Hide()
	else
		SyncUI_BagFrame:Show()
	end
end


-- Info Leak Functions
local function filter(self,event,...)
	return true
end

local function ClearCharacterGoldInfo(serverName,realmlist)
	if realmlist then
		for _, realm in pairs(realmlist) do
			local dataBase = SyncUI_Data["MoneyBroker"][realm]
			
			if dataBase then
				for index, info in pairs(dataBase) do
					local guid, gold = unpack(info)
					local name = select(6, GetPlayerInfoByGUID(guid))
					local found;
					
					if realm ~= serverName then
						name = name.."-"..realm
					end

					for i = 1, GetNumIgnores() do
						local ignoreName = GetIgnoreName(i)
						
						if name == ignoreName then
							found = true
							break
						end
					end
					
					if name == select(1, UnitName("player")) then
						found = true
					end
					
					if not found then
						SyncUI_Data["MoneyBroker"][realm][index] = nil
					end
					
					DelIgnore(name)
				end
			end
		end
	end
	
	if not realmlist then
		local dataBase = SyncUI_Data["MoneyBroker"][serverName]
		
		for index, info in pairs(dataBase) do
			local guid, gold = unpack(info)
			local name = select(6, GetPlayerInfoByGUID(guid))
			local found;

			for i = 1, GetNumIgnores() do
				local ignoreName = GetIgnoreName(i)
				
				if name == ignoreName then
					found = true
					break
				end
			end
			
			if name == select(1, UnitName("player")) then
				found = true
			end
			
			if not found then
				SyncUI_Data["MoneyBroker"][realm][index] = nil
			end
			
			DelIgnore(name)
		end
	end
	
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", filter)
end

function SyncUI_BagFrame_MoneyFrame_OnClick(self,button)
	if not IsShiftKeyDown() then return end
	
	self:Disable()
	
	ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", filter)
	
	local serverName = GetRealmName()
	local realmlist = GetAutoCompleteRealms()
	
	if realmlist then
		for _, realm in pairs(realmlist) do
			local dataBase = SyncUI_Data["MoneyBroker"][realm]
			
			if dataBase then
				for index, info in pairs(dataBase) do
					local guid, gold = unpack(info)
					local name = select(6, GetPlayerInfoByGUID(guid))

					if realm ~= serverName then
						name = name.."-"..realm
					end
					
					AddIgnore(name)
				end
			end
		end	
	end

	if not realmlist then
		local dataBase = SyncUI_Data["MoneyBroker"][serverName]
		
		for index, info in pairs(dataBase) do
			local guid, gold = unpack(info)
			local name = select(6, GetPlayerInfoByGUID(guid))

			AddIgnore(name)
		end
	end
	
	C_Timer.After(0.5, function()
		ClearCharacterGoldInfo(serverName,realmlist)
	end)

	C_Timer.After(1.0, function()
		self:Enable()
	end)
end

function SyncUI_BagFrame_MoneyFrame_OnEvent(self,event,...)
	if event == "PLAYER_ENTERING_WORLD" then
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
		self.session = GetMoney()
	end
	
	self.value:SetText(GetCoinTextureString(GetMoney()))
	
	do	-- Update Gold Info
		local serverName, entry = GetRealmName(), nil
		local guid, gold = UnitGUID("player"), GetMoney()
		
		if not SyncUI_Data["MoneyBroker"] then
			SyncUI_Data["MoneyBroker"] = {}
		end
		if not SyncUI_Data["MoneyBroker"][serverName] then
			SyncUI_Data["MoneyBroker"][serverName] = {}
		end

		for index, info in pairs(SyncUI_Data["MoneyBroker"][serverName]) do
			if info and info[1] == guid then
				entry = index
				break
			end
		end
		
		if entry then
			SyncUI_Data["MoneyBroker"][serverName][entry][2] = gold
		else
			tinsert(SyncUI_Data["MoneyBroker"][serverName],{guid,gold})
		end
		
		table.sort(SyncUI_Data["MoneyBroker"][serverName], function(a,b) return a[2] > b[2] end)
	end
end

function SyncUI_BagFrame_MoneyFrame_OnEnter(self)
	local regions = {"US","KR","EU","TW","CN"}
	local serverName = GetRealmName()
	local difference = GetMoney() - self.session
	local total, numInfo = 0, 0
	local waitForInfo
	local realmlist = GetAutoCompleteRealms()
	
	GameTooltip:SetOwner(self:GetParent(), "ANCHOR_LEFT",1,-42)
	
	if realmlist then	-- Display all connected realms
		local header
		
		for _, realm in pairs(realmlist) do
			local dataBase = SyncUI_Data["MoneyBroker"][realm]

			if dataBase then
				if not header then
					GameTooltip:AddLine(serverName.." - "..regions[GetCurrentRegion()],1,1,1)
					GameTooltip:SetPrevLineJustify("CENTER")
					GameTooltip:AddDivider()
					header = true
				end
			
				for _, info in pairs(dataBase) do
					local guid, gold = unpack(info)
					local charName = select(6, GetPlayerInfoByGUID(guid))

					if charName then
						local class = select(2, GetPlayerInfoByGUID(guid))
						local color = SYNCUI_CLASS_COLORS[class]
						local r,g,b = color.r, color.g, color.b
						local path = "Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes"
						local left, right, top, bottom = unpack(CLASS_ICON_TCOORDS[class])
						local classIcon
						
						if left and right and top and bottom then
							local inset = 5
							
							left, right, top, bottom = (left*256+inset),(right*256-inset),(top*256+inset),(bottom*256-inset)
							classIcon = "|T"..path..":14:14:0:0:256:256:"..left..":"..right..":"..top..":"..bottom.."|t"
						end		
						
						if r and g and b then
							charName = SyncUI_GetColorizedText(charName,r,g,b)
						end
							
						if realm == serverName then
							GameTooltip:AddDoubleLine(classIcon.." "..charName, GetCoinTextureString(gold),1,1,1,1,1,1)
						else
							GameTooltip:AddDoubleLine(classIcon.." "..charName.." - "..realm, GetCoinTextureString(gold),1,1,1,1,1,1)
						end
					else
						GameTooltip:AddDoubleLine("Receive Info...", GetCoinTextureString(gold),1,1,1,1,1,1)
						
						waitForInfo = true
					end
					
					numInfo = numInfo + 1
					total = total + gold
				end
			end
		end
	end
	
	if not realmlist then	-- Display only the actual realm
		GameTooltip:SetOwner(self:GetParent(), "ANCHOR_LEFT",1,-42)
		GameTooltip:AddLine(serverName.." - "..regions[GetCurrentRegion()],1,1,1)
		GameTooltip:SetPrevLineJustify("CENTER")
		GameTooltip:AddDivider()
		
		for _, info in pairs(SyncUI_Data["MoneyBroker"][serverName]) do
			local guid, gold = unpack(info)
			local charName = select(6, GetPlayerInfoByGUID(guid))

			if charName then
				local class = select(2, GetPlayerInfoByGUID(guid))
				local color = SYNCUI_CLASS_COLORS[class]
				local r,g,b = color.r, color.g, color.b
				local path = "Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes"
				local left, right, top, bottom = unpack(CLASS_ICON_TCOORDS[class])
				local classIcon
				
				if left and right and top and bottom then
					local inset = 5
					
					left, right, top, bottom = (left*256+inset),(right*256-inset),(top*256+inset),(bottom*256-inset)
					classIcon = "|T"..path..":14:14:0:0:256:256:"..left..":"..right..":"..top..":"..bottom.."|t"
				end		
				
				if r and g and b then
					charName = SyncUI_GetColorizedText(charName,r,g,b)
				end
				
				GameTooltip:AddDoubleLine(classIcon.." "..charName, GetCoinTextureString(gold),1,1,1,1,1,1)
			else
				GameTooltip:AddDoubleLine("Receive Info...", GetCoinTextureString(gold),1,1,1,1,1,1)
				
				waitForInfo = true
			end
			
			numInfo = numInfo + 1
			total = total + gold
		end
	end
	
	if numInfo > 1 and total > 0 then
		GameTooltip:AddDivider()
		GameTooltip:AddDoubleLine("Total:",GetCoinTextureString(total),1,1,1,1,1,1)
	end
	
	if difference ~= 0 then
		local pre,r,g,b
		
		if difference < 0 then
			difference = difference - difference * 2
			
			pre,r,g,b = "- ", 1,0,0
		elseif difference > 0 then
			pre,r,g,b = "+ ", 0,1,0
		end
		
		GameTooltip:AddDivider()
		GameTooltip:AddDoubleLine("Session:",pre..GetCoinTextureString(difference),1,1,1,r,g,b)
	end
	
	GameTooltip:AddDivider()
	GameTooltip:AddLine("|cFF00FF99Shift-Click|r to delete unexisting characters",1,1,1)
	GameTooltip:Show()
	
	if waitForInfo then
		self:SetScript("OnUpdate", function()
			SyncUI_BagFrame_MoneyFrame_OnEnter(self)
		end)
	else
		self:SetScript("OnUpdate", nil)
	end
end

function SyncUI_BagFrame_CurrencyButton_OnEnter(self)
	local numEntries, spacer = 0
	
	GameTooltip:SetOwner(self:GetParent(), "ANCHOR_LEFT",1,-42)
	GameTooltip:AddLine("|cFF00FF99".."Currencies")
	GameTooltip:SetPrevLineJustify("CENTER")
	GameTooltip:AddDivider()
	
	-- watched list
	for index = 1, GetCurrencyListSize() do
		local name, isHeader, _, isUnused, isWatched, count, icon = GetCurrencyListInfo(index)
		
		if not isHeader and not isUnused then
			if spacer then
				GameTooltip:AddDivider()
				spacer = false
			end
			
			numEntries = numEntries + 1
			
			GameTooltip:AddDoubleLine(name, count.." |T"..icon..":14:14:0:0:64:64:5:59:5:59|t",1,1,1,1,1,1)
		end
	end
	
	if numEntries > 0 then
		GameTooltip:AddDivider()
	end
	
	GameTooltip:AddLine("|cFF00FF99Click|r to open currency window",1,1,1)
	GameTooltip:Show()
end


-- Bank Functions
function SyncUI_BankSlot_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetInventoryItem("player", self:GetInventorySlot())
	GameTooltip:Show()
end

function SyncUI_BankFrame_OnShow(self)
	BankFrame:Show()
	BankFrame:UnregisterAllEvents()
	BankFrame.selectedTab = 1
	SyncUI_BagFrame:Show()
	
	self.bank:Show()
	self.reagentBank:Hide()
	
	if not self.firstTime then
		UpdateAllContainers(isBank)
		UpdateAllBags(isBank)
		UpdateAllContainers(isBank)
		UpdateBankBagSlots()
		UpdateInventoryFreeSlot(self,isBank)
		
		self.firstTime = true
	end
end

function SyncUI_BankFrame_OnLoad(self)
	BankFrame:UnregisterAllEvents()
	BankFrame:SetPoint("TOPRIGHT",UIParent,"TOPLEFT")

	self:SetScale(SyncUI_UIScale)	
	self:RegisterEvent("BANKFRAME_OPENED")
	self:RegisterEvent("BANKFRAME_CLOSED")
	self:RegisterEvent("PLAYERBANKSLOTS_CHANGED")
	self:RegisterEvent("PLAYERBANKBAGSLOTS_CHANGED")

	UIPanelWindows[self:GetName()] = { area = "left", pushable = 0, width = 464 }
end

function SyncUI_BankFrame_OnEvent(self,event,...)
	if event == "BANKFRAME_OPENED" then		
		ShowUIPanel(self)
	end
	
	if event == "BANKFRAME_CLOSED" then
		HideUIPanel(self)
	end
	
	if event == "PLAYERBANKSLOTS_CHANGED" then
		UpdateAllBags(isBank)
		UpdateBankBagSlots()
		UpdateInventoryFreeSlot(self,isBank)
	end
	
	if event == "PLAYERBANKBAGSLOTS_CHANGED" then
		UpdateBankBagSlots()
		UpdateInventoryFreeSlot(self,isBank)
	end
end


-- Scroll Frame Fix
function SyncUI_ItemContainer_ScrollFrame_OnLoad(self)
	self.ScrollBar.scrollStep = 37

	ScrollFrame_OnLoad(self)
	ScrollFrame_OnScrollRangeChanged(self)
end

function SyncUI_ItemContainer_ScrollFrame_OnScrollRangeChanged(self,xRange,yRange)
	local scrollBar = self.ScrollBar or _G[self:GetName().."ScrollBar"]
	local scrollDown = _G[scrollBar:GetName().."ScrollDownButton"]
	local scrollUp = _G[scrollBar:GetName().."ScrollUpButton"]
	local scrollThumb = _G[scrollBar:GetName().."ThumbTexture"]
	local value = scrollBar:GetValue()
	local yRange;
	
	if not yRange then
		yRange = self:GetVerticalScrollRange()
	end

	yRange = yRange - 26.5	-- mysterious value to fix
	
	if yRange < 0 then
		yRange = 0
	end

	if value > yRange then
		value = yRange
	end
	
	scrollBar:SetMinMaxValues(0, yRange)
	scrollBar:SetValue(value)

	if floor(yRange) == 0 then
		scrollBar:Hide()
		scrollDown:Hide()
		scrollUp:Hide()
	else
		scrollBar:Show()
		scrollDown:Show()
		scrollUp:Show()
		scrollThumb:Show()

		if yRange - value > 0.005 then
			scrollDown:Enable()
		else
			scrollDown:Disable()
		end
	end
end


-- Assign Menu
local selectedItem;

function SyncUI_AssignItemDropDown_Open(self,itemID)
	local dataBase = SyncUI_Data["Bags"]["ContainerList"]
	local frame = SyncUI_AssignItem_DropDown
	local header = frame.Header.Text
	local width, height, adjust = 0, 0, 76
	local point, relativePoint, xPos, yPos
	
	do 	-- Adjust Header
		local name,_,quality = GetItemInfo(itemID)
		local minWidth = 138
		
		header:SetVertexColor(GetItemQualityColor(quality))
		header:SetText(name)

		width = math.ceil(header:GetStringWidth() + 30)

		if width < minWidth then
			width = minWidth
		end
		
		header:SetWidth(width)
	end
	
	for index = 1, 50 do
		local button = frame["Button"..index]
		
		if button then
			button:Hide()
		end
	end
	
	-- Setup category buttons for assign
	for index, containerName in pairs(dataBase) do
		local buttonHeight = 14
		local button;
		
		if not frame["Button"..index] then
			frame["Button"..index] = CreateFrame("Button",nil,frame,"SyncUI_AssignItem_DropDownButtonTemplate")
		end
		
		button = frame["Button"..index]
		button:SetText(containerName)
		button:SetPoint("TOP",frame["Button"..index-1] or frame.Inventory,"BOTTOM",0,0)
		button:SetID(index)
		button:Show()
		
		height = height + buttonHeight
	end

	-- Setup anchor points
	if (GetScreenWidth() - self:GetRight()) > width then
		point, relativePoint, xPos, yPos = "TOPLEFT", "TOPRIGHT", -2, 1
	else
		point, relativePoint, xPos, yPos = "TOPRIGHT", "TOPLEFT", 1, 1
	end
	
	frame:SetSize(width,ceil(height + adjust))
	frame:ClearAllPoints()
	frame:SetPoint(point,self,relativePoint,xPos,yPos)
	frame:Show()
	
	selectedItem = itemID
end

function SyncUI_AssignItemDropDownButton_OnClick(self,button)
	local dataBase = SyncUI_Data["Bags"]["ItemList"]
	local frame = SyncUI_AssignItem_DropDown
	local category = self:GetText()
	local itemID = selectedItem
	local buttonID = self:GetID()
	
	if itemID	then
		if buttonID > 0 then
			dataBase[itemID] = category
		else
			dataBase[itemID] = nil
		end
	end
	
	UpdateAllBags()
	UpdateAllBags(true)
	
	frame:Hide()
end


-- Category Menu
function SyncUI_BagFrame_CategoryMenu_OnShow(self)
	self.ScrollFrame.ScrollBar:SetValue(0)
	SyncUI_BagFrame_CategoryMenu_UpdateScrollFrame(self.ScrollFrame)
end

function SyncUI_BagFrame_CategoryMenu_AddCategory(self)
	local dataBase = SyncUI_Data["Bags"]["ContainerList"]
	local scrollFrame = self:GetParent().ScrollFrame
	local editBox = self:GetParent().Enter
	local text = editBox:GetText()

	if text == "Enter Name" or text == "" or text == nil or (text:gsub("%s","") == "") then
		return
	end
	
	editBox:SetText("")
	tinsert(dataBase,text)
	scrollFrame.ScrollBar:SetValue(0)
	SyncUI_BagFrame_CategoryMenu_UpdateScrollFrame(scrollFrame)
	SyncUI_AssignItem_DropDown:Hide()
	
	CreateNewContainer()
	
	UpdateAllBags()
	UpdateAllBags(true)
end

function SyncUI_BagFrame_CategoryMenu_UpdateScrollFrame(self)
	local dataBase = SyncUI_Data["Bags"]["ContainerList"]
	local offset = FauxScrollFrame_GetOffset(self)
	local maxLines = 7
	local size = #dataBase
	
	FauxScrollFrame_Update(self, size, maxLines, 20)
	
	for i = 1, maxLines do
		local line = SyncUI_BagFrame_CategoryMenu["Line"..i]
		local idx = offset + i

		line:SetID(idx)
		
		SyncUI_BagFrame_CategoryMenu_UpdateLine(line)
	end
end

function SyncUI_BagFrame_CategoryMenu_UpdateLine(self)
	local dataBase = SyncUI_Data["Bags"]["ContainerList"]
	local index = self:GetID()
	
	if not dataBase[index] then self:Hide() return end

	local name = dataBase[index]

	if name then
		self.Name:SetText(name)
	end
	
	if not self:IsShown() then self:Show() end
end

function SyncUI_BagFrame_CategoryMenu_DeleteButton_OnClick(self,button)
	local dataBase = SyncUI_Data["Bags"]["ContainerList"]
	local line = self:GetParent()
	local scrollFrame = line:GetParent().ScrollFrame
	local name = dataBase[line:GetID()]
	
	tremove(dataBase, line:GetID())
	scrollFrame.ScrollBar:SetValue(0)
	SyncUI_BagFrame_CategoryMenu_UpdateScrollFrame(scrollFrame)
	SyncUI_AssignItem_DropDown:Hide()
	
	UpdateAllBags()
	UpdateAllBags(true)
end

function SyncUI_BagFrame_CategoryLine_MoveUp(self)
	local dataBase = SyncUI_Data["Bags"]["ContainerList"]
	local lineID = self:GetID()
	local name = dataBase[lineID]
	
	for i = 2, #dataBase do
		if dataBase[i] == name then
			tremove(dataBase, i)
			tinsert(dataBase, i-1, name)
			break
		end
	end
	
	local frame = SyncUI_BagFrame_CategoryMenu.ScrollFrame
	SyncUI_BagFrame_CategoryMenu_UpdateScrollFrame(frame)
	
	UpdateAllBags()
	UpdateAllBags(true)
end

function SyncUI_BagFrame_CategoryLine_MoveDown(self)
	local dataBase = SyncUI_Data["Bags"]["ContainerList"]
	local lineID = self:GetID()
	local name = dataBase[lineID]
	
	for i = 1, #dataBase-1 do
		if dataBase[i] == name then
			tremove(dataBase, i)
			tinsert(dataBase, i+1, name)
			break
		end
	end
	
	local frame = SyncUI_BagFrame_CategoryMenu.ScrollFrame
	SyncUI_BagFrame_CategoryMenu_UpdateScrollFrame(frame)
	
	UpdateAllBags()
	UpdateAllBags(true)
end

function SyncUI_BagFrame_CategoryLine_ShowButtons(self,isLine)
	local frame = self:GetParent()
	local buttons = {"Delete","MoveUp","MoveDown"}
	
	if isLine then
		frame = self
	end
	
	for _, key in pairs(buttons) do
		local button = frame[key]
		
		button:SetAlpha(1)
	end
end

function SyncUI_BagFrame_CategoryLine_HideButtons(self,isLine)
	local frame = self:GetParent()

	if isLine then
		frame = self
	end
	
	if not frame:IsMouseOver() then
		local buttons = {"Delete","MoveUp","MoveDown"}
		
		for _, key in pairs(buttons) do
			local button = frame[key]
			
			button:SetAlpha(0)
		end
	end
end