
local version = GetAddOnMetadata("SyncUI", "Version")
local maxChangeLogLines = 5
local ChangeLogData = {
	"BugFix: incorrect player buff scaling.",
	"BugFix: disappeared drag frame for 'Raid Frames'.",
	"BugFix: support issues for 'XLoot'.",
	"BugFix: empty quest reward tooltip icon.",
	"BugFix: player status display within 'AFK & Flight Mode'.",
}

local function ModifiedString(string)
	local newString = string

	do	-- Color Header
		local count = string.find(string,":")

		if count then
			local prefix = string.sub(string,0,count)
			local suffix = string.sub(string,count+1)
			
			newString = tostring("|cFF00FF99"..prefix.."|r"..suffix)
		end
	end

	-- Color Highlights
	for pattern in gmatch(string,"('.*')") do
		newString = newString:gsub(pattern, "|cFFFFFF00"..pattern:gsub("'","").."|r")
	end
	
	return newString
end

local function GetChangeLogInfo(i)
	for line, info in pairs(ChangeLogData) do
		if line == i then
			return info
		end
	end
end


function SyncUI_ToggleChangeLog()
	PlaySound("igMainMenuOptionCheckBoxOn")
	local frame = SyncUI_ChangeLog
	
	if frame:IsShown() then
		frame:Hide()
	else
		SyncUI_SlideIn(frame)
	end
end

function SyncUI_ChangeLog_OnLoad(self)
	self:SetScale(SyncUI_UIScale)
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	
	self.Version:SetText("Sync|cFFFFFFFFUI v."..version)
	self.Version:SetTextColor(0,1,0.6)

	self.ScrollFrame:Show()
	self.ScrollFrame.ScrollBar:ClearAllPoints()
	self.ScrollFrame.ScrollBar:SetPoint("TOPLEFT",self.ScrollFrame,"TOPRIGHT",8,-24)
	self.ScrollFrame.ScrollBar:SetPoint("BOTTOMLEFT",self.ScrollFrame,"BOTTOMRIGHT",8,23)
	
	tinsert(UISpecialFrames,"SyncUI_ChangeLog")
end

function SyncUI_ChangeLog_OnCheckVersion(self)
	if not SyncUI_CharData.Installed then return end
	
	if not SyncUI_Data["Version"] or ( SyncUI_Data["Version"] and SyncUI_Data["Version"] ~= version) then
		SyncUI_SlideIn(self)
		SyncUI_Data["Version"] = version
	end
	
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
end

function SyncUI_ChangeLog_ScrollBar_ForceUpdate(self)
	local offset = FauxScrollFrame_GetOffset(self)
	local parent = self:GetParent()
	
	FauxScrollFrame_Update(self, #ChangeLogData, maxChangeLogLines, 30)
	
	for i = 1, maxChangeLogLines do
		local button = _G[parent:GetName().."Button"..i]
		local idx = offset + i
		
		if idx <= #ChangeLogData then
			local string = ModifiedString(GetChangeLogInfo(idx))
			
			button.Text:SetText(string)
			button.tipText = string
			
			if button:IsMouseOver() then
				button:GetScript("OnEnter")(button)
			end
			
			button:Show()
		else
			button:Hide()
		end
	end
end
