
local _G = _G
local dummy = function() end
local font, font2 = SyncUI_GameFontShadow_Medium, SyncUI_GameFontShadow_Small
local TexturesToDisable = {
	"TabLeft",
	"TabMiddle",
	"TabRight",
	"TabSelectedLeft",
	"TabSelectedMiddle",
	"TabSelectedRight",
	"TabHighlightLeft",
	"TabHighlightMiddle",
	"TabHighlightRight",
	"EditBoxLeft",
	"EditBoxMid",
	"EditBoxRight",
	"EditBoxFocusLeft",
	"EditBoxFocusMid",
	"EditBoxFocusRight",
}
local TexturesToHide = {
	"TopLeftTexture",
	"BottomLeftTexture",
	"TopRightTexture",
	"BottomRightTexture",
	"LeftTexture",
	"RightTexture",
	"BottomTexture",
	"TopTexture",
}
table.insert(CHAT_FONT_HEIGHTS, 1, 10)
table.remove(CHAT_FONT_HEIGHTS, 5)

local function OpenChat()
	if SyncUI_ChatFrame:IsShown() and not SyncUI_ChatFrame.chat:IsShown() then
		SyncUI_ChatFrame.ChatButton:Click()
	elseif not SyncUI_ChatFrame:IsShown() then
		if not SyncUI_ChatFrame.chat:IsShown() then
			SyncUI_ChatFrame.SlideInToChat:Play()
		else
			SyncUI_ToggleChatWindowButton:Click()
		end
	end
	
	ChatFrame1EditBox:Show()
	ChatFrame1EditBox:SetFocus(true)
end

local function ModifyChatWindow(window)
	if InCombatLockdown() then return end

	local tab = _G[window:GetName().."Tab"]
	local btnFrame = _G[window:GetName().."ButtonFrame"]
	
	tab:SetNormalFontObject(font)
	btnFrame.Show = dummy
	btnFrame:Hide()
	
	window.Background = _G[window:GetName().."Background"]

	if window.isDocked then
		window:SetParent(SyncUI_ChatFrame.chat)
		window.Background:Hide()
	else
		window:SetParent(SyncUI_UndockedChatFrame)
	end
	
	if window:GetID() == 1 then
		tab:RegisterForDrag()
		
		window:StartMoving()
			window:SetSize(412,149)
			window:ClearAllPoints()
			window:SetPoint("BOTTOMLEFT",SyncUI_ChatFrame,41,23)
			window:SetClampRectInsets(0, 0, 0, 0)
			window:SetMaxResize(412, 149)
			window:SetMinResize(412, 149)
		window:StopMovingOrSizing()

		window.editBox.header:SetFontObject(font)
		window.editBox.headerSuffix:SetFontObject(font)
		window.editBox:SetFontObject(font)
		window.editBox:SetParent(SyncUI_ChatFrame.chat)
		window.editBox:ClearAllPoints()
		window.editBox:SetPoint("TOPLEFT",window,"BOTTOMLEFT",-35,-21)
		window.editBox:SetPoint("RIGHT",window,35,0)
		window.editBox:HookScript("OnShow",function() SyncUI_ChatFrame.EditBox:Show() end)
		window.editBox:HookScript("OnHide",function() SyncUI_ChatFrame.EditBox:Hide() end)
	end

	do	-- Disable stuff
		BNToastFrame:SetClampedToScreen(true)
		BNToastFrame:SetClampRectInsets(-15,15,15,-15)	
		SyncUI_DisableFrame(FriendsMicroButton)
		SyncUI_DisableFrame(ChatFrameMenuButton)
		GeneralDockManager:SetParent(SyncUI_ChatFrame.chat)
		
		-- Hide Textures
		for _, texture in pairs(TexturesToDisable) do
			_G[window:GetName()..texture]:SetTexture(nil)
		end
		
		for _, texture in pairs(TexturesToHide) do
			_G[window:GetName()..texture]:Hide()
		end
	end
		
	SetCVar("chatStyle", "classic")
	SyncUI_DisableFrame(InterfaceOptionsSocialPanelChatStyle)
	SyncUI_DisableFrame(InterfaceOptionsSocialPanelWholeChatWindowClickable)
	InterfaceOptionsSocialPanelTimestamps:ClearAllPoints()
	InterfaceOptionsSocialPanelTimestamps:SetPoint("TOPLEFT",InterfaceOptionsSocialPanelProfanityFilter,"TOPRIGHT",260,-16)
end


function SyncUI_ChatFrame_OnLoad(self)
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("UI_SCALE_CHANGED")
	
	if GetScreenWidth() > 1920 then
		SyncUI_RegisterDragFrame(self,"Chat Console")
	end
	
	hooksecurefunc("FCF_OpenTemporaryWindow", function()
		for _, chatWindow in pairs(CHAT_FRAMES) do
			local window = _G[chatWindow]
			if window.isTemporary then
				ModifyChatWindow(window)
			end
		end
	end)
	
	hooksecurefunc("FCF_SetLocked", function(self)
		if self.isDocked then
			self:SetParent(SyncUI_ChatFrame.chat)
			
			if self.Background then
				self.Background:Hide()
			end
		else
			self:SetParent(SyncUI_UndockedChatFrame)
			
			if self.Background then
				self.Background:Show()
			end
		end
	end)

	hooksecurefunc("ChatFrame_OpenChat", OpenChat)	
end

function SyncUI_ChatFrame_OnEvent(self,event,...)
	if event == "PLAYER_ENTERING_WORLD" then
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
		
		SyncUI_SetUIFont()
		
		for i = 1, NUM_CHAT_WINDOWS do
			local window = _G["ChatFrame"..i]
			
			ModifyChatWindow(window)
		end
	end
	
	if event == "UI_SCALE_CHANGED" then
		ChatFrame1:SetSize(412,149)
		ChatFrame1:ClearAllPoints()
		ChatFrame1:SetPoint("BOTTOMLEFT",SyncUI_ChatFrame,41,23)
		ChatFrame1Tab:ClearAllPoints()
	end
end


------------------------------
-- Damage Meter Integration --
------------------------------
local function LoadSetupForSkada(self,option)
	do	-- check for Skada
		local isEnabled = select(4, GetAddOnInfo("Skada"))
		
		if isEnabled and not IsAddOnLoaded("Skada") then
			LoadAddOn("Skada")
		elseif not isEnabled then
			return
		end
		
		option:Show()
	end

	if not SyncUI_Data.Options.SkadaChat then
		return
	end

	self:GetParent().ChatButton:Show()
	self:GetParent().MeterButton:Show()
		
	hooksecurefunc(Skada, "CreateWindow", function(_,data)
		local window = _G["SkadaBarWindow"..data]

		if window then	
			window:SetParent(self)
			window:SetScript("OnSizeChanged",function(self)
				self:SetSize(297,109)
			end)
			window:SetSize(297,109)
			window:ClearAllPoints()
			window:SetPoint("TOPLEFT",SyncUI_ChatFrame,27.5,-13.5)
			window.Hide = dummy
			window:SetBackdrop(nil)
			window:Show()
			
			window.SetScale = dummy
			window.SetBackdrop = dummy
			window.resizebutton:Hide()
			window.resizebutton.Show = dummy
		end
	end)
	
	hooksecurefunc(Skada, "UpdateDisplay", function(self)
		for _, win in ipairs(self:GetWindows()) do
			local header = win.bargroup.button
			
			-- update title bar
			if not header.isStyled then
				header:EnableMouse(false)
				header:SetWidth(280)
				header:SetHeight(18)
				header:ClearAllPoints()
				header:SetPoint("TOPLEFT",7,19)
				header:SetNormalFontObject(font2)
				header:SetBackdrop(nil)
				
				header.SetWidth = dummy
				header.SetWidth = dummy
				header.ClearAllPoints = dummy
				header.SetPoint = dummy
				header.SetNormalFontObject = dummy
				header.SetBackdrop = dummy
				
				header.isStyled = true
			end
			
			-- update bars
			for _, bar in pairs(win.bargroup:GetBars()) do
				bar.label:SetFont(SyncUI_FontPath,8)
				bar.timerLabel:SetFont(SyncUI_FontPath,8)
				--bar.texture:Hide()
				--bar.bgtexture:Hide()
			end

			win.db.barwidth = 297
			win.db.background.height = 109
		end	
	end)
end

local function LoadSetupForRecount(self,option)
	do	-- check for Recount
		local isEnabled = select(4, GetAddOnInfo("Recount"))
		
		if isEnabled and not IsAddOnLoaded("Recount") then
			LoadAddOn("Recount")
		elseif not isEnabled then
			return
		end

		option:Show()
	end

	if not SyncUI_Data.Options.RecountChat then
		return
	end
	
	self:GetParent().ChatButton:Show()
	self:GetParent().MeterButton:Show()
	
	local window = Recount.MainWindow

	if window then
		do	-- Setup Window
			window.TitleClick.StartMoving = dummy
			window.StartMoving = dummy
			
			window:SetParent(self)
			window:SetUserPlaced(false)
			window:SetResizable(false)
			window:SetScale(SyncUI_UIScale)
			window:SetSize(420,185)
			window:ClearAllPoints()
			window:SetPoint("BOTTOMLEFT",SyncUI_ChatFrame,38,18)
			window:SetScript("OnSizeChanged", function(self)
				self:ClearAllPoints()
				self:SetPoint("BOTTOMLEFT",SyncUI_ChatFrame,38,18)
			end)
			window:SetBackdrop(nil)
			window:Show()

			window.SetParent = dummy
			window.SetScale = dummy
			window.SetSize = dummy
			--window.SetPoint = dummy
			window.SetUserPlaced = dummy
			window.SetBackdrop = dummy
			window.Hide = dummy	

			window.Title:SetFont(SyncUI_FontPath,12,"NONE")
			window.Title.SetFont = dummy
			

		end
		do	-- Setup Header
			window.Title:ClearAllPoints()
			window.Title:SetPoint("TOPLEFT",20,-11)
			window.CloseButton:ClearAllPoints()
			window.CloseButton:SetPoint("TOPRIGHT",-20,-8)
		end
		do	-- Setup Bars
			for _, bar in ipairs(window.Rows) do
				local left = bar.LeftText
				local right = bar.RightText

				left:SetFont(SyncUI_FontPath,12,"NONE")
				left.SetFont = dummy
				right:SetFont(SyncUI_FontPath,12,"NONE")
				right.SetFont = dummy
			end
		end
		do	-- kill resizers
			local sides = {"Left","Right"}
			for _, side in ipairs(sides) do
				local frame = _G["RecountResizeGrip"..side]
				frame:EnableMouse(false)
				frame:SetAlpha(0)
			end
		end
	end
	
	hooksecurefunc(Recount, "RefreshMainWindow", function()
		local window = Recount.MainWindow
		
		window:ClearAllPoints()
		window:SetPoint("BOTTOMLEFT",SyncUI_ChatFrame,38,18)
	end)

	if IsAddOnLoaded("RecountSkinByNth") then
		hooksecurefunc(RecountSkin, "CreateWindow", function()
			Recount_MainWindow_Background:Hide()
			Recount_MainWindow_Secondary:ClearAllPoints()
			Recount_MainWindow_Secondary:SetPoint("TOPLEFT",13,-10)
			window.Title:ClearAllPoints()
			window.Title:SetPoint("LEFT",Recount_MainWindow_Secondary,"RIGHT")
		end)
	end
end

function SyncUI_ChatFrame_DamageMeter_OnLoad()
	local parent = SyncUI_ChatFrame.meter
	local optionButton = SyncUI_OptionsMenu.SyncMenu.Meter
	
	LoadSetupForSkada(parent,optionButton)
	LoadSetupForRecount(parent,optionButton)
end



