
local function RegisterLayoutButton(self)
	self:RegisterForClicks('LeftButtonUp')
	self:SetFrameRef("Menu",SyncUI_GroupControl_LayoutMenu)
	self:SetAttribute("_onclick", [[
		local frame = self:GetFrameRef("Menu")
		
		if button == "LeftButton" then
			if frame:IsShown() then
				frame:Hide()
			else
				frame:Show()
			end
		end
	]])
end

local function RegisterLayoutMenu(self)
	for i = 1, 2 do
		local button = _G[self:GetName().."Button"..i]
		
		button:RegisterForClicks('LeftButtonUp')
		button:SetFrameRef("pNormal",SyncUI_PartyFrameContainer.Normal)
		button:SetFrameRef("pHeal",SyncUI_PartyFrameContainer.Heal)
		button:SetFrameRef("rNormal",SyncUI_RaidFrameContainer.Normal)
		button:SetFrameRef("rHeal",SyncUI_RaidFrameContainer.Heal)
		button:SetFrameRef("Menu",self)

		button:HookScript("OnClick", function()
			if i == 1 then	
				SyncUI_CharData.GroupControl.Layout = "Normal"
				self:GetParent().LayoutButton:SetText("Layout: |cFF00FF99".."Normal")
			end
			if i == 2 then	
				SyncUI_CharData.GroupControl.Layout = "Heal"
				self:GetParent().LayoutButton:SetText("Layout: |cFF00FF99".."Heal")
			end
		end)
		
		if i == 1 then
			button.Text:SetText("Normal")
			button:SetAttribute("_onclick", [[
				local menu = self:GetFrameRef("Menu")
				local pNormal = self:GetFrameRef("pNormal")
				local pHeal = self:GetFrameRef("pHeal")
				local rNormal = self:GetFrameRef("rNormal")
				local rHeal = self:GetFrameRef("rHeal")
				
				if button == "LeftButton" then
					menu:Hide()
					
					pHeal:Hide()
					rHeal:Hide()
					
					pNormal:Show()
					rNormal:Show()
				end
			]])

		end
		if i == 2 then
			button.Text:SetText("Heal")
			button:SetAttribute("_onclick", [[
				local menu = self:GetFrameRef("Menu")
				local pNormal = self:GetFrameRef("pNormal")
				local pHeal = self:GetFrameRef("pHeal")
				local rNormal = self:GetFrameRef("rNormal")
				local rHeal = self:GetFrameRef("rHeal")
				
				if button == "LeftButton" then
					menu:Hide()

					pNormal:Hide()
					rNormal:Hide()
					
					pHeal:Show()
					rHeal:Show()
				end
			]])
		end
	end
end

local function RegisterToggleButton(self)
	self:RegisterForClicks('LeftButtonUp')
	self:SetFrameRef("Container",SyncUI_RaidFrameContainer)
	self:SetAttribute("_onclick", [[
		local frame = self:GetFrameRef("Container")
		
		if button == "LeftButton" then
			if frame:IsShown() then
				frame:Hide()
			else
				frame:Show()
			end
		end
	]])
	self:HookScript("OnClick",function(self)
		local frame = SyncUI_RaidFrameContainer
		
		GameTooltip:ClearLines()
		
		if frame:IsShown() then
			self.Normal:SetTexCoord(0,0.328125,0,0.1640625)
			self.Pushed:SetTexCoord(0.5,0.828125,0,0.1640625)
			GameTooltip:AddLine("|cFF00FF99Click |r".."to minimize raid frames",1,1,1)
			
			SyncUI_CharData.GroupControl.Toggle = false
		else
			self.Normal:SetTexCoord(0,0.328125,0.25,0.4140625)
			self.Pushed:SetTexCoord(0.5,0.828125,0.25,0.4140625)
			GameTooltip:AddLine("|cFF00FF99Click |r".."to maximize raid frames",1,1,1)
			
			SyncUI_CharData.GroupControl.Toggle = true
		end
		
		GameTooltip:Show()
	end)

	RegisterStateDriver(self, "visibility", "[@raid1,exists] show; hide")	
end

function SyncUI_GroupControl_OnEvent(self,event,addOn)
	if event == "ADDON_LOADED" and addOn == "SyncUI" then 
		if SyncUI_Data.Options.DefaultRaid then
			UnregisterStateDriver(self, "visibility")
			
			self:Hide()
			
			SyncUI_RaidFrameContainer:Hide()
			SyncUI_UnregisterDragFrame(SyncUI_PartyFrameContainer.Heal)
			SyncUI_UnregisterDragFrame(SyncUI_RaidFrameContainer.Heal)
			
			return
		end

		RegisterToggleButton(self.ToggleButton)
		RegisterLayoutButton(self.LayoutButton)
		RegisterLayoutMenu(self.LayoutMenu)
	end
end


-- Raid Info Panel
local TokenTexCoords = {
	[1] = {0,0.390625,0,0.78125},
	[2] = {0.5,0.890625,0,0.78125},
}

local MethodStrings = {
	["freeforall"] = "FFA",
	["group"] = "Group",
	["needbeforegreed"] = "N > G",
	["roundrobin"] = "Round",
	["personalloot"] = "Personal"
}

function SyncUI_RaidInfoPanel_OnLoad(self)
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("GROUP_JOINED")
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RegisterEvent("GROUP_ROSTER_UPDATE")
	self:RegisterEvent("UPDATE_INSTANCE_INFO")

	RegisterStateDriver(self, "visibility", "[petbattle] hide; show")
	
	for index, info in pairs(TokenTexCoords) do
		local token = self["Token"..index]
		
		token.Icon:SetTexCoord(unpack(info))
	end
end

function SyncUI_RaidInfoPanel_OnEvent(self,event,...)
	local leader, looter;
	
	-- Get Raid Lead
	for i = 1,  GetNumGroupMembers() do
		local name, rank = GetRaidRosterInfo(i)
		local class = select(6, GetRaidRosterInfo(i))
		
		if name then
			name = name:match("[^%-]+")
		end
		
		if class then
			local color = SYNCUI_CLASS_COLORS[class]
			local r,g,b = color.r,color.g,color.b
			
			if r and g and b then
				name = SyncUI_GetColorizedText(name,r,g,b)
			end
		end
		
		if rank == 2 then
			leader = name
			break
		end
	end
	
	-- Get Master Looter
	for i = 1,  GetNumGroupMembers() do
		local name = GetRaidRosterInfo(i)
		local class = select(6, GetRaidRosterInfo(i))
		local isML = select(11, GetRaidRosterInfo(i))

		if name then
			name = name:match("[^%-]+")
		end
		
		if class then
			local color = SYNCUI_CLASS_COLORS[class]
			local r,g,b = color.r,color.g,color.b
			
			if r and g and b then
				name = SyncUI_GetColorizedText(name,r,g,b)
			end
		end
		
		if isML then
			looter = name
			break
		end
	end
	
	if not looter then
		looter = MethodStrings[GetLootMethod()]
	end

	self.Token1.Name:SetText(leader)
	self.Token2.Name:SetText(looter)
end

function SyncUI_RaidInfoToken_OnEnter(self)
	local index = self:GetID()
	
	GameTooltip:SetOwner(self, "ANCHOR_BOTTOM")
	
	if index == 1 then
		local numAssists = 0
		
		-- catch up num assistens
		for i = 1,  GetNumGroupMembers() do
			local rank = select(2, GetRaidRosterInfo(i))

			if rank == 1 then
				numAssists = numAssists + 1
			end
		end

		-- Add Assistants
		if numAssists > 0 then
			GameTooltip:AddLine("Assist",1,1,1)
			GameTooltip:SetPrevLineJustify("CENTER")
			GameTooltip:AddDivider()
			
			for i = 1,  GetNumGroupMembers() do
				local name, rank = GetRaidRosterInfo(i)
				local class = select(6, GetRaidRosterInfo(i))
				
				if name then
					name = name:match("[^%-]+")
				end
				
				if class then
					local color = SYNCUI_CLASS_COLORS[class]
					local r,g,b = color.r,color.g,color.b
					
					if r and g and b then
						name = SyncUI_GetColorizedText(name,r,g,b)
					end
				end
		
				if rank == 1 then
					GameTooltip:AddLine(name,1,1,1)
				end
			end			
		else
			GameTooltip:AddLine("Raidleader",1,1,1)
		end
	end
	
	if index == 2 then
		GameTooltip:AddLine("Lootmethod",1,1,1)
	end
	
	GameTooltip:Show()
end


-- World Markers
local WorldMarkers = {
	[1] = "Interface\\TargetingFrame\\UI-RaidTargetingIcon_6",
	[2] = "Interface\\TargetingFrame\\UI-RaidTargetingIcon_4",
	[3] = "Interface\\TargetingFrame\\UI-RaidTargetingIcon_3",
	[4] = "Interface\\TargetingFrame\\UI-RaidTargetingIcon_7",
	[5] = "Interface\\TargetingFrame\\UI-RaidTargetingIcon_1",
	[6] = "Interface\\TargetingFrame\\UI-RaidTargetingIcon_2",
	[7] = "Interface\\TargetingFrame\\UI-RaidTargetingIcon_5",
	[8] = "Interface\\TargetingFrame\\UI-RaidTargetingIcon_8",
}

function SyncUI_WorldMarkerButton_OnLoad(self)
	local index = self:GetID()
	local icon = WorldMarkers[index]
	
	self:RegisterForClicks("AnyUp")
	self:SetAttribute("type", "macro")
	self:SetAttribute("macrotext1", string.format("/wm %d",index))
	self:SetAttribute("shift-type*", "macro")
	self:SetAttribute("shift-macrotext1", string.format("/cwm %d",index))
	self.Icon:SetTexture(icon)
end

function SyncUI_ToggleWorldMarkerFrame()
	if InCombatLockdown() then return end
	
	local frame = SyncUI_WorldMarkerBar
	
	if frame:IsShown() then
		frame:Hide()
	else
		frame:Show()
	end
end
