
local _G = _G
local prefix = "InterfaceOptionsActionBarsPanel"
local buttons = {"BottomRight","Right","RightTwo","AlwaysShowActionBars"}
local bars = {"MultiBarBottomRight","MultiBarRight","MultiBarLeft"}

local PAGE_LAYOUT = {
	["DRUID"] = "[bonusbar:1,nostealth] 7; [bonusbar:1,stealth] 8; [bonusbar:2] 8; [bonusbar:3] 9; [bonusbar:4] 10;",
	["PRIEST"] = "[bonusbar:1] 7;",
	["ROGUE"] = "[bonusbar:1] 7;",
	["MONK"] = "[bonusbar:1] 7; [bonusbar:2] 8; [bonusbar:3] 9;",
	["DEFAULT"] = "[vehicleui] 12; [possessbar] 12; [shapeshift] 13; [overridebar] 14; [bar:2] 2; [bar:3] 3; [bar:4] 4; [bar:5] 5; [bar:6] 6;",
}

local function GetPageLayout()
	local class = select(2, UnitClass("player"))
	local condition = PAGE_LAYOUT["DEFAULT"]
	local page = PAGE_LAYOUT[class]

	if page then
		condition = condition.." "..page
	end

	condition = condition.." [form] 1; 1"

	return condition
end

local function StyleActionButton(self)
	self:SetSize(40,40)
	self:SetNormalTexture("")

	self.NormalTexture:SetAlpha(0)
	self:GetPushedTexture():SetAlpha(0)
	self.icon:SetTexCoord(0.05,0.95,0.05,0.95)

	self.border = self:CreateTexture(nil,"BORDER")
	self.border:SetSize(44,44)
	self.border:SetPoint("CENTER",0,0)
	self.border:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionButton")
	self.border:SetTexCoord(0,0.171875,0,0.6875)
	
	self.itemborder = _G[self:GetName().."Border"]
	self.itemborder:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionButton")
	self.itemborder:SetTexCoord(0.25,0.421875,0,0.6875)
	self.itemborder:SetSize(44,44)
	self.itemborder:SetPoint("CENTER",0,0)

	self.highlight = self:GetHighlightTexture()
	self.highlight:ClearAllPoints()
	self.highlight:SetPoint("CENTER")
	self.highlight:SetSize(44,44)
	self.highlight:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionButton")
	self.highlight:SetTexCoord(0.5,0.671875,0,0.6875)
	
	self.checked = self:GetCheckedTexture()
	self.checked:ClearAllPoints()
	self.checked:SetPoint("CENTER")
	self.checked:SetSize(44,44)
	self.checked:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionButton")
	self.checked:SetTexCoord(0.75,0.921875,0,0.6875)

	self.hotkey = _G[self:GetName().."HotKey"]
	self.hotkey:SetFontObject(SyncUI_GameFontNormal_Large)
	self.hotkey:ClearAllPoints()
	self.hotkey:SetPoint("TOPRIGHT",1,-3)
	self.hotkey:SetShadowColor(0,0,0)
	self.hotkey:SetShadowOffset(1,-1)
	
	self.count = _G[self:GetName().."Count"]
	self.count:SetFontObject(SyncUI_GameFontNormal_Large)
	self.count:ClearAllPoints()
	self.count:SetPoint("BOTTOMRIGHT",0,1)
	
	self.name = _G[self:GetName().."Name"]
	self.name:SetFontObject(SyncUI_GameFontNormal_Medium)
	self.name:SetSize(42,10)
	self.name:SetPoint("BOTTOM",0,1)
	self.name:SetShadowOffset(0,0)

	self.pushed = self:GetPushedTexture()
	self.pushed:SetSize(39,39)
	self.pushed:ClearAllPoints()
	self.pushed:SetPoint("CENTER")

	local floatingBG = _G[self:GetName().."FloatingBG"]
	if floatingBG then floatingBG:SetTexture("") end
	
	if self.cooldown then
		SyncUI_Cooldown_OnRegister(self.cooldown)
		
		self.cooldown:ClearAllPoints()
		self.cooldown:SetPoint("TOPLEFT",1,-1)
		self.cooldown:SetPoint("BOTTOMRIGHT",-1,1)
	end
end

local function StyleActionButton_SideBar(self)
	self:SetSize(33,33)
	self:SetNormalTexture("")
	self:GetNormalTexture():SetAlpha(0)
	self:GetPushedTexture():SetAlpha(0)
	self.icon:SetTexCoord(0.05,0.95,0.05,0.95)
	
	self.border = self:CreateTexture(nil,"BORDER")
	self.border:SetSize(37,37)
	self.border:SetPoint("CENTER",0,0)
	self.border:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionButton_Small")
	self.border:SetTexCoord(0,0.14453125,0,0.578125)

	self.itemborder = _G[self:GetName().."Border"]
	self.itemborder:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionButton_Small")
	self.itemborder:SetTexCoord(0.25,0.39453125,0,0.578125)
	self.itemborder:SetSize(37,37)
	self.itemborder:SetPoint("CENTER",0,0)

	self.highlight = self:GetHighlightTexture()
	self.highlight:ClearAllPoints()
	self.highlight:SetPoint("CENTER")
	self.highlight:SetSize(37,37)
	self.highlight:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionButton_Small")
	self.highlight:SetTexCoord(0.5,0.64453125,0,0.578125)
	
	self.checked = self:GetCheckedTexture()
	self.checked:ClearAllPoints()
	self.checked:SetPoint("CENTER")
	self.checked:SetSize(37,37)
	self.checked:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionButton_Small")
	self.checked:SetTexCoord(0.75,0.89453125,0,0.578125)

	self.hotkey = _G[self:GetName().."HotKey"]
	self.hotkey:SetFontObject(SyncUI_GameFontNormal_Medium)
	self.hotkey:ClearAllPoints()
	self.hotkey:SetPoint("TOPRIGHT",2,0)
	self.hotkey:SetShadowColor(0,0,0)
	self.hotkey:SetShadowOffset(1,-1)
	
	self.count = _G[self:GetName().."Count"]
	self.count:SetFontObject(SyncUI_GameFontNormal_Medium)
	self.count:ClearAllPoints()
	self.count:SetPoint("BOTTOMRIGHT",1,0)
	
	self.name = _G[self:GetName().."Name"]
	self.name:SetFontObject(SyncUI_GameFontNormal_Medium)
	self.name:SetSize(32,10)
	self.name:SetPoint("BOTTOM")

	local floatingBG = _G[self:GetName().."FloatingBG"]
	if floatingBG then floatingBG:SetTexture("") end
	
	if self.cooldown then
		SyncUI_Cooldown_OnRegister(self.cooldown)
		
		self.cooldown:ClearAllPoints()
		self.cooldown:SetPoint("TOPLEFT",1,-1)
		self.cooldown:SetPoint("BOTTOMRIGHT",-1,1)
	end
end

local function StyleActionButton_Vehicle(self,isPetAction)
	if not self.HotKey then self.HotKey = _G[self:GetName().."HotKey"] end
	if not self.icon then self.icon = self.Icon end

	if not self.border then
		self:SetSize(53,53)
		self:SetNormalTexture("")
		self:SetPushedTexture("")
		self.HotKey:SetSize(53,10)
		self.HotKey:SetFontObject(SyncUI_GameFontNormal_Large)
		self.HotKey:ClearAllPoints()
		self.HotKey:SetPoint("TOP",self,"BOTTOM",0,-2)
		self.HotKey:SetJustifyH("CENTER")

		self.icon:SetTexCoord(0.05,0.95,0.05,0.95)
		self.border = self:CreateTexture(nil,"BORDER")
		self.border:SetSize(57,57)
		self.border:SetPoint("CENTER",0,0)
		self.border:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionButton_Large")
		self.border:SetTexCoord(0,0.22265625,0,0.890625)
		
		self.highlight = self:GetHighlightTexture()
		self.highlight:ClearAllPoints()
		self.highlight:SetPoint("CENTER")
		self.highlight:SetSize(57,57)
		self.highlight:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionButton_Large")
		self.highlight:SetTexCoord(0.5,0.72265625,0,0.890625)

		if not isPetAction then
			self.checked = self:GetCheckedTexture()
			self.checked:ClearAllPoints()
			self.checked:SetPoint("CENTER")
			self.checked:SetSize(57,57)
			self.checked:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionButton_Large")
			self.checked:SetTexCoord(0.75,0.97265625,0,0.890625)
		end
	
		if self.SelectedHighlight then
			self.SelectedHighlight:SetSize(57,57)
			self.SelectedHighlight:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionButton_Large")
			self.SelectedHighlight:SetTexCoord(0.25,0.47265625,0,0.890625)
		end
	end
	
	if self.Cooldown then
		self.Cooldown:SetFontObject(SyncUI_GameFontNormal_Huge)
	end
	
	if self.CooldownFlash then
		self.CooldownFlash:Hide()
	end
end

local function StyleActionButton_Extra(self)
	self.noResize = true
	self:SetSize(36,36)
	self:ClearAllPoints()
	self:SetPoint("TOPLEFT",11,-11)
	self:SetNormalTexture("")
	self.style:SetAlpha(0)
	
	self.icon:SetSize(36,36)
	self.icon:SetTexCoord(0.05,0.95,0.05,0.95)
	
	self.border = self:CreateTexture(nil,"BORDER")
	self.border:SetSize(84,58)
	self.border:SetPoint("TOPLEFT",self:GetParent())
	self.border:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\SideButton")
	self.border:SetTexCoord(0.65625,0,0.75,0.9765625)

	self.highlight = self:GetHighlightTexture()
	self.highlight:ClearAllPoints()
	self.highlight:SetPoint("CENTER")
	self.highlight:SetSize(36,36)
	self.highlight:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\SideButton")
	self.highlight:SetTexCoord(0.71875,1,0,0.140625)
	
	self.checked = self:GetCheckedTexture()
	self.checked:ClearAllPoints()
	self.checked:SetPoint("CENTER")
	self.checked:SetSize(40,40)
	self.checked:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionButton_Small")
	self.checked:SetTexCoord(0.75,0.89453125,0,0.578125)
	
	self.hotkey = _G[self:GetName().."HotKey"]
	self.hotkey:SetFontObject(SyncUI_GameFontNormal_Large)
	self.hotkey:ClearAllPoints()
	self.hotkey:SetPoint("TOPRIGHT",self.icon,0,-2)
	self.hotkey:SetShadowColor(0,0,0)
	self.hotkey:SetShadowOffset(1,-1)
	
	self.count = _G[self:GetName().."Count"]
	self.count:SetFontObject(SyncUI_GameFontNormal_Large)
	self.count:ClearAllPoints()
	self.count:SetPoint("BOTTOMLEFT",34,13)
	
	if self.Cooldown then
		self.Cooldown:SetFontObject(SyncUI_GameFontNormal_Huge)
	end
	
	if self.cooldown then
		self.cooldown:ClearAllPoints()
		self.cooldown:SetPoint("TOPLEFT",1,-1)
		self.cooldown:SetPoint("BOTTOMRIGHT",-1,1)
	end
end

local function GetStanceIcon(index)
	local class = select(2, UnitClass("player"))
	local shapeshiftID, icon;
	
	if index and index ~= 0 then
		shapeshiftID = select(5, GetShapeshiftFormInfo(index))
		icon = select(3, GetSpellInfo(shapeshiftID))
	end
	
	-- special class treatments
	if class == "DRUID" and IsStealthed() then
		icon = select(3, GetSpellInfo(5215))
	end

	if class == "ROGUE" then
		if index == 1 then	-- Stealth
			icon = select(3, GetSpellInfo(1784))
		elseif index == 2 then	-- Vanish + Shadow Dance
			for i = 1, 40 do
				local spellID = select(11, UnitBuff("player",i))
				
				if spellID == 51713 or spellID == 11327 then
					icon = select(3, GetSpellInfo(spellID))
					break
				end
			end
		end
	end
	
	if class == "SHAMAN" and index == 1 then
		icon = select(3, GetSpellInfo(2645))
	end

	if class == "WARRIOR" and index == 1 then
		local active = GetSpecialization()
		local specID;
		
		if active then
			specID = GetSpecializationInfo(active)
		end
		
		if specID then
			local choice = select(2, GetTalentRowSelectionInfo(7))
			
			if choice == 21 then
				icon = select(3, GetSpellInfo(156291))
			end
		end
	end

	if class == "PRIEST" then
		local choice = select(2, GetTalentRowSelectionInfo(1))
		
		if choice == 19753 then
			for i = 1, 40 do
				local spellID = select(11, UnitBuff("player",i))
				
				if spellID == 119032 then
					icon = select(3, GetSpellInfo(spellID))
					break
				end
			end
		end
		--[[
		local active = GetSpecialization()
		local specID;
		
		if active then
			specID = GetSpecializationInfo(active)
		end
		
		if specID then
			local choice = select(2, GetTalentRowSelectionInfo(1))
			
			if choice == 2 then
				icon = select(3, GetSpellInfo(119032))
			end
		end
		--]]
	end
	
	return icon
end

local function RegisterStanceFlyout(self)
	self:RegisterEvent("PLAYER_LOGIN")
	self:RegisterEvent("PLAYER_TALENT_UPDATE")
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	
	self:SetAttribute("_onclick",[[
		local frame = self:GetFrameRef("Bar")
		
		if frame:IsShown() then
			frame:Hide()
		else
			frame:Show()
		end
	]])
	self:SetAttribute("_onstate-combat", "self:GetFrameRef('Bar'):Hide()")
	self:SetAttribute("_onstate-stance", "self:GetFrameRef('Bar'):Hide()")
	
	self:SetFrameRef("Bar", _G["SyncUI_StanceBar"])
	
	RegisterStateDriver(self, "combat", "[nocombat] 0; [combat] 1;")
	RegisterStateDriver(self, "stance", "[stance:1] 1; [stance:2] 2; [stance:3] 3; [stance:4] 4; [stance:5] 5;")
end

local function RegisterActionBarPageScrolling(self)
	self:SetAttribute("_onmousewheel", [[
		if delta > 0 then
			-- scroll page up
		else
			-- scroll page down
		end
		
		self:RunAttribute("action")
	]])
end

local function RegisterSideBarScrolling(self)
	do	-- Register MouseWheel
		self:EnableMouseWheel(true)
		self:SetAttribute("_onmousewheel", [[
			local bar1 = self:GetFrameRef("Bar1")
			local bar2 = self:GetFrameRef("Bar2")
			local bar3 = self:GetFrameRef("Bar3")
			local scrollUp = self:GetFrameRef("ScrollUp")
			local scrollDown = self:GetFrameRef("ScrollDown")
			
			if delta > 0 then
				if bar1:IsShown() then
					bar1:Hide()
					bar3:Hide()
					bar2:Show()
					
					scrollUp:Show()
					scrollDown:Show()
				elseif bar2:IsShown() then
					bar1:Hide()
					bar2:Hide()
					bar3:Show()
					
					scrollUp:Hide()
					scrollDown:Show()
				end			
			else
				if bar3:IsShown() then
					bar1:Hide()
					bar2:Show()
					bar3:Hide()
					scrollDown:Show()
					scrollUp:Show()
				elseif bar2:IsShown() then
					bar2:Hide()
					bar3:Hide()
					bar1:Show()
					scrollUp:Show()
					scrollDown:Hide()
				end
			end
		]])
		self:SetFrameRef("Bar1",self.Bar1)
		self:SetFrameRef("Bar2",self.Bar2)
		self:SetFrameRef("Bar3",self.Bar3)
		self:SetFrameRef("ScrollUp",SyncUI_SideBar.ScrollUp)
		self:SetFrameRef("ScrollDown",SyncUI_SideBar.ScrollDown)
	end
	do	-- Register ScrollUp Button
		self.ScrollUp:RegisterForClicks('LeftButtonUp')
		self.ScrollUp:SetAttribute("_onclick", [[
			local bar1 = self:GetFrameRef("Bar1")
			local bar2 = self:GetFrameRef("Bar2")
			local bar3 = self:GetFrameRef("Bar3")
			local scrollDown = self:GetFrameRef("ScrollDown")
			
			if button == "LeftButton" then
				if bar1:IsShown() then
					bar1:Hide()
					bar2:Show()
					bar3:Hide()
					self:Show()
					scrollDown:Show()
				elseif bar2:IsShown() then
					bar1:Hide()
					bar2:Hide()
					bar3:Show()
					self:Hide()
					scrollDown:Show()
				end
			end
		]])
		self.ScrollUp:SetFrameRef("Bar1",self.Bar1)
		self.ScrollUp:SetFrameRef("Bar2",self.Bar2)
		self.ScrollUp:SetFrameRef("Bar3",self.Bar3)
		self.ScrollUp:SetFrameRef("ScrollDown",self.ScrollDown)
	end
	do	-- Register ScrollDown Button
		self.ScrollDown:RegisterForClicks('LeftButtonUp')
		self.ScrollDown:SetAttribute("_onclick", [[
			local bar1 = self:GetFrameRef("Bar1")
			local bar2 = self:GetFrameRef("Bar2")
			local bar3 = self:GetFrameRef("Bar3")
			local scrollUp = self:GetFrameRef("ScrollUp")
			
			if button == "LeftButton" then
				if bar3:IsShown() then
					bar1:Hide()
					bar2:Show()
					bar3:Hide()
					scrollUp:Show()
					self:Show()
				elseif bar2:IsShown() then
					bar1:Show()
					bar2:Hide()
					bar3:Hide()
					scrollUp:Show()
					self:Hide()
				end
			end
		]])
		self.ScrollDown:SetFrameRef("Bar1",self.Bar1)
		self.ScrollDown:SetFrameRef("Bar2",self.Bar2)
		self.ScrollDown:SetFrameRef("Bar3",self.Bar3)
		self.ScrollDown:SetFrameRef("ScrollUp",self.ScrollUp)
	end
end

local function RegisterSideBarToggle(self)
	local frame = SyncUI_ActionBar.SideBarToggle
	
	frame:RegisterForClicks('LeftButtonUp')
	frame:SetAttribute("_onclick", [[
		local frame = self:GetFrameRef("SideBar")
		
		if button == "LeftButton" then
			if frame:IsShown() then
				frame:Hide()
			else
				frame:Show()
			end
		end
	]])
	frame:SetFrameRef("SideBar",self)
	frame:HookScript("OnClick", function()
		if SyncUI_SideBar:IsShown() then
			SyncUI_CharData.SideBarToggle = false
		else
			SyncUI_CharData.SideBarToggle = true
		end
	end)
end



-- Action Bars
function SyncUI_MainMenuBar_OnLoad(self)
	self:SetScale(SyncUI_UIScale)
	
	-- Kill Frames
	SyncUI_DisableFrame("MainMenuBar")
	SyncUI_DisableFrame("ActionBarDownButton")
	SyncUI_DisableFrame("ActionBarUpButton")	
	SyncUI_DisableFrame("MainMenuBarBackpackButton")
	SyncUI_DisableFrame("CharacterBag0Slot")
	SyncUI_DisableFrame("CharacterBag1Slot")
	SyncUI_DisableFrame("CharacterBag2Slot")
	SyncUI_DisableFrame("CharacterBag3Slot")
	SyncUI_DisableFrame("OverrideActionBarExpBar")
	SyncUI_DisableFrame("OverrideActionBarHealthBar")
	SyncUI_DisableFrame("OverrideActionBarPowerBar")
	SyncUI_DisableFrame("OverrideActionBarPitchFrame")
	SyncUI_DisableFrame("OverrideActionBarLeaveFrame")

	for _, frame in pairs(MICRO_BUTTONS) do
		SyncUI_DisableFrame(frame)
	end

	-- Kill Textures	
	MainMenuBarPageNumber:Hide()
	OverrideActionBarEndCapL:Hide()
	OverrideActionBarEndCapR:Hide()
	OverrideActionBarDivider2:Hide()
	OverrideActionBarBorder:Hide()
	OverrideActionBarBG:Hide()
	OverrideActionBarMicroBGL:Hide()
	OverrideActionBarMicroBGR:Hide()
	OverrideActionBarMicroBGMid:Hide()
	OverrideActionBarButtonBGL:Hide()
	OverrideActionBarButtonBGR:Hide()
	OverrideActionBarButtonBGMid:Hide()

	StanceBarLeft:SetTexture(nil)
	StanceBarMiddle:SetTexture(nil)
	StanceBarRight:SetTexture(nil)	
	PossessBackground1:SetTexture(nil)
	PossessBackground2:SetTexture(nil)	
	SlidingActionBarTexture0:SetTexture(nil)
	SlidingActionBarTexture1:SetTexture(nil)
	MainMenuBarTexture0:SetTexture(nil)
    MainMenuBarTexture1:SetTexture(nil)
    MainMenuBarTexture2:SetTexture(nil)
    MainMenuBarTexture3:SetTexture(nil)
    MainMenuBarLeftEndCap:SetTexture(nil)
    MainMenuBarRightEndCap:SetTexture(nil)
	
	UpdateMicroButtonsParent = function() end
	MoveMicroButtons = function() end
	
	UIPARENT_MANAGED_FRAME_POSITIONS["MultiCastActionBarFrame"] = nil
end

function SyncUI_ActionBar_OnLoad(self)
	MainMenuBarArtFrame:SetParent(self)
	MainMenuBarArtFrame:EnableMouse(false)

	for i = 1, 12 do
		local button = _G["ActionButton"..i]
		StyleActionButton(button)

		button:ClearAllPoints()
		button:SetPoint("BOTTOM",-253+(46*(i-1)),28)
	end
	
	-- check if bottomLeft was shown
	if not _G[prefix.."BottomLeft"]:GetChecked() then
		self.borderLeft:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionBar_Small")
		self.borderRight:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionBar_Small")	
	end
	
	if UnitOnTaxi("player") or UnitInVehicle("player") then
		self.ExitButton:Show()
		self.ExitButton:Enable()
		self:RegisterEvent("PLAYER_CONTROL_GAINED")		
	end
	
	self:RegisterEvent("TAXIMAP_OPENED")
	self:RegisterUnitEvent("UNIT_ENTERED_VEHICLE","player")
	self:RegisterUnitEvent("UNIT_EXITED_VEHICLE","player")
	
	--RegisterActionBarPageScrolling(self)
	RegisterStateDriver(self, "visibility", "[petbattle][overridebar][vehicleui][possessbar,@vehicle,exists] hide; show")
end

function SyncUI_ActionBar_OnEvent(self,event,...)
	if event == "UNIT_ENTERED_VEHICLE" or event == "PLAYER_CONTROL_LOST" then
		self.ExitButton:Show()
		self.ExitButton:Enable()

		if event == "PLAYER_CONTROL_LOST" then
			self:RegisterEvent("PLAYER_CONTROL_GAINED")
			self:UnregisterEvent("PLAYER_CONTROL_LOST")
		end
	end
	if event == "UNIT_EXITED_VEHICLE" or event == "PLAYER_CONTROL_GAINED" then
		self.ExitButton:Hide()
		
		if event == "PLAYER_CONTROL_GAINED" then
			self:UnregisterEvent("PLAYER_CONTROL_GAINED")
		end
	end
	if event == "TAXIMAP_OPENED" then
		self:RegisterEvent("PLAYER_CONTROL_LOST")
	end
end

function SyncUI_MultiBar_OnLoad(self)
	MultiBarBottomLeft:SetParent(self)
	MultiBarBottomLeft:EnableMouse(false)
	MultiBarBottomLeft:HookScript("OnShow",function()
		SyncUI_ActionBar.borderLeft:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionBar")
		SyncUI_ActionBar.borderRight:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionBar")
	end)
	MultiBarBottomLeft:HookScript("OnHide",function()
		SyncUI_ActionBar.borderLeft:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionBar_Small")
		SyncUI_ActionBar.borderRight:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionBar_Small")
	end)
	
	for i = 1, 12 do
		local button = _G["MultiBarBottomLeftButton"..i]
		StyleActionButton(button)
		
		button:ClearAllPoints()
		button:SetPoint("LEFT",self,0+(46*(i-1)),0)
	end

	RegisterStateDriver(self, "visibility", "[petbattle][overridebar][vehicleui][possessbar,@vehicle,exists] hide; show")
end

function SyncUI_SideBar_OnLoad(self)
	for _, bar in pairs(bars) do
		local Bar = _G[bar]
		local parent = _G["SyncUI_"..bar]
		Bar:SetParent(parent)
		Bar:EnableMouse(false)
		Bar:Show()
		
		for i = 1, 12 do
			local button = _G[bar.."Button"..i]
			button:ClearAllPoints()
			button:SetPoint("LEFT",self,12+(37*(i-1)),0)
			button:SetAttribute("flyoutDirection", "UP")
			StyleActionButton_SideBar(button)
		end
	end

	for _, button in pairs(buttons) do
		local btn = _G[prefix..button]
		btn:Hide()
	end

	_G[prefix.."LockActionBars"]:ClearAllPoints()
	_G[prefix.."LockActionBars"]:SetPoint("TOP",prefix.."BottomLeft","BOTTOM",0,-8)
	_G[prefix.."SecureAbilityToggle"]:ClearAllPoints()
	_G[prefix.."SecureAbilityToggle"]:SetPoint("TOP",prefix.."LockActionBars","BOTTOM",0,-55)

	RegisterSideBarScrolling(self)
	RegisterSideBarToggle(self)
end

function SyncUI_VehicleBar_OnLoad(self)
	OverrideActionBar:SetParent(self)
	OverrideActionBar:ClearAllPoints()
	OverrideActionBar:SetPoint("BOTTOM",0,27)
	OverrideActionBar:EnableMouse(false)
	OverrideActionBar:SetScript("OnShow", nil)
	
	local leaveButtonPlaced = false
	OverrideActionBar.LeaveButton:Hide()
	
	for i = 1, 6 do
		local button = _G["OverrideActionBarButton"..i]
		StyleActionButton_Vehicle(button)
		button:SetPoint("LEFT",self,90+(61*(i-1)),8)
	end
	
	SyncUI_RegisterDragFrame(VehicleSeatIndicator,"Vehicle",true)

	RegisterStateDriver(self, "visibility", "[petbattle] hide; [overridebar][vehicleui][possessbar,@vehicle,exists] show; hide")
	RegisterStateDriver(OverrideActionBar, "visibility", "[overridebar][vehicleui][possessbar,@vehicle,exists] show; hide")
end

function SyncUI_PetBar_OnLoad(self)
	PetActionBarFrame:SetParent(self)
	PetActionBarFrame:EnableMouse(false)

	self:SetScale(SyncUI_UIScale)
	
	for i = 1, NUM_PET_ACTION_SLOTS do
		local button = _G["PetActionButton"..i]
		StyleActionButton_SideBar(button)
		
		button:ClearAllPoints()
		button:SetPoint("TOPLEFT",self,11,-18-(37*(i-1)))
	end
	
	RegisterStateDriver(self, "visibility", "[petbattle][overridebar][vehicleui][possessbar,@vehicle,exists] hide; [@pet,exists] show; hide")
end

function SyncUI_PetBattleBar_OnLoad(self)
	self:RegisterEvent("PET_BATTLE_OPENING_START")
	self:RegisterEvent("PET_BATTLE_OPENING_DONE")
	self:RegisterEvent("PET_BATTLE_TURN_STARTED")
	self:RegisterEvent("PET_BATTLE_PET_CHANGED")
	
	RegisterStateDriver(self, "visibility", "[petbattle] show; hide")
	
	do	-- Hide Stuff
		local frame = PetBattleFrame
		local bottomFrame = frame.BottomFrame
		
		frame:SetFrameStrata("LOW")
		bottomFrame:SetParent(self)
		bottomFrame.Background:Hide()
		bottomFrame.RightEndCap:Hide()
		bottomFrame.LeftEndCap:Hide()
		bottomFrame.FlowFrame:Hide()
		bottomFrame.Delimiter:Hide()
		bottomFrame.TurnTimer:Hide()
		bottomFrame.TurnTimer:ClearAllPoints()
		bottomFrame.TurnTimer:SetPoint("BOTTOM",self,"TOP",0,-20)
		bottomFrame.MicroButtonFrame:Hide()
	end
end

function SyncUI_PetBattleBar_OnEvent(self)
	local frame = PetBattleFrame.BottomFrame
	local dataBase = frame.abilityButtons
	local dataBase2 = {"SwitchPetButton","CatchButton","ForfeitButton"}
	
	for index, value in pairs(dataBase) do
		local button = dataBase[index]

		button:ClearAllPoints()
		button:SetPoint("LEFT",self,90+(61*(index-1)),6.5)
		
		StyleActionButton_Vehicle(button,true)
	end

	for index, value in pairs(dataBase2) do
		local button = frame[value]
		local isPetAction
		
		if index > 1 then
			isPetAction = true
		end
		
		button:ClearAllPoints()
		button:SetPoint("LEFT",self,90+(61*(index+2)),6.5)

		StyleActionButton_Vehicle(button,isPetAction)
	end
	
	PetBattleFrameXPBar:Hide()
end

function SyncUI_StanceBar_OnLoad(self)
	RegisterStanceFlyout(SyncUI_StanceButton)
	
	for i = 1, 10 do
		local button = _G["StanceButton"..i]
		StyleActionButton_SideBar(button)
		
		button:SetParent(self)
		button:ClearAllPoints()
		button:SetPoint("BOTTOMLEFT",self,7,7+(37*(i-1)))
	end
end

function SyncUI_StanceBar_OnUpdateSize(self)
	local numBtn = 0
	
	for i = 1, 10 do
		local button = _G["StanceButton"..i]
		if button:IsShown() then
			numBtn = numBtn + 1
		end
	end
	
	self.backdrop:SetSize(47,(37*numBtn)+10)
end

function SyncUI_StanceButton_OnEvent(self,event,...)
	if InCombatLockdown() then return end
	
	if GetNumShapeshiftForms() > 0 then
		self:EnableMouse(true)
	else
		self:EnableMouse(false)
		SyncUI_StanceBar:Hide()
	end
end

function SyncUI_StanceButton_OnUpdate(self)
	local class = select(2, UnitClass("player"))
	local numForms = GetNumShapeshiftForms()
	
	if numForms > 0 then
		local form = GetShapeshiftForm()
		local icon = GetStanceIcon(form)
		
		if form > 0 then
			self.alert.icon:SetTexture(icon)
		else
			self.alert.icon:SetTexture(0.12,0.12,0.12)
		end
		
		self.alert:Show()
	elseif class == "SHAMAN" or class == "PRIEST" then
		local form = GetShapeshiftForm()
		local icon = GetStanceIcon(form)
		
		self.alert.icon:SetTexture(icon)
		self.alert:Show()
	else
		self.alert:Hide()
	end
end

function SyncUI_SocialButton_OnEnter(self)
	local found, bnetFound
	
	ShowFriends()
	GameTooltip:SetOwner(self, "ANCHOR_LEFT",0,0)
	GameTooltip:SetPadding(-3)
	
	-- BNet-Friend-List
	for i = 1, BNGetNumFriends() do
		local battleTag = select(2, BNGetFriendInfo(i))
		local client = select(7, BNGetFriendInfo(i))
		local isOnline = select(8, BNGetFriendInfo(i))
		local isAFK = select(10, BNGetFriendInfo(i))
		local isDND = select(11, BNGetFriendInfo(i))
		
		if isOnline then
			local BNetName = battleTag
			local info, status
			local clientIcon = " |T"..BNet_GetClientTexture(client)..":14:14:0:0:64:64:10:54:10:54|t "
			local numToons = BNGetNumFriendToons(i);

			if not bnetFound then
				GameTooltip:AddLine("|cFF2E9AFEBNet-Friend-List")
				GameTooltip:SetPrevLineJustify("CENTER")
				GameTooltip:AddDivider()
				
				bnetFound = true
			end

			do	-- Setup Status
				if isAFK then
					status = FRIENDS_TEXTURE_AFK
				elseif isDND then
					status = FRIENDS_TEXTURE_DND
				else
					status = FRIENDS_TEXTURE_ONLINE
				end

				status = "|T"..status..":14:14:|t "
			end
			
			do	-- Setup Info
				for index = 1, numToons do
					local charName = select(2, BNGetFriendToonInfo(i,index))
					local zoneName = select(10, BNGetFriendToonInfo(i,index))
					local gameText = select(12, BNGetFriendToonInfo(i,index))
					local classString = select(8, BNGetFriendToonInfo(i,index))
					local class;

					if classString then
						if classString == "Death Knight" then
							classString = "DeathKnight"
						end
						
						class = string.upper(classString)
					end
					
					if class then
						local color = SYNCUI_CLASS_COLORS[class]
						
						if color then
							local r,g,b = color.r,color.g,color.b
							
							charName = " ("..SyncUI_GetColorizedText(charName,r,g,b)..")"
							BNetName = battleTag..charName
						end
					end
					
					if client == "WoW" then
						if not info and zoneName and zoneName ~= "" then
							info = zoneName or ""
						end
					elseif client == "App" then
						info = "Battle.net"
					else
						info = gameText or ""
					end
				end

				if not info then
					info = ""
				end
			end

			GameTooltip:AddDoubleLine(status..BNetName,info..clientIcon,1,1,1,1,1,1)
		end
	end
	
	-- Realm-Friend-List
	for i = 1, GetNumFriends() do
		local name,level,class,area,isOnline,status = GetFriendInfo(i)
		
		if isOnline then
			local color = SYNCUI_CLASS_COLORS[string.upper(class)]
			local clientIcon = " |T"..BNet_GetClientTexture("WoW")..":14:14:0:0:64:64:10:54:10:54|t "
			local statusIcon
			
			-- Add Header
			if not found then
				if bnetFound then
					GameTooltip:AddLine(" ")
				end
				
				GameTooltip:AddLine("Realm-Friend-List")
				GameTooltip:SetPrevLineJustify("CENTER")
				GameTooltip:AddDivider()
				found = true
			end
			
			-- Set Status
			if status == "" then
				status = FRIENDS_TEXTURE_ONLINE
			elseif status == CHAT_FLAG_AFK then
				status = FRIENDS_TEXTURE_AFK
			elseif status == CHAT_FLAG_DND then
				status = FRIENDS_TEXTURE_DND
			end
			
			-- Set Status Icon
			statusIcon = "|T"..status..":14:14|t "
			
			-- Colorize Name
			if color then
				local r,g,b = color.r, color.g, color.b
				name = SyncUI_GetColorizedText(name,r,g,b)
			end
			
			-- Add Line
			GameTooltip:AddDoubleLine(statusIcon..name,area..clientIcon,1,1,1,1,1,1)
		end
	end

	-- no friends online
	if not found and not bnetFound then
		GameTooltip:AddLine("There are currently",1,1,1)
		GameTooltip:AddLine("no friends online!",1,1,1)
	end
	
	GameTooltip:Show()
end

function SyncUI_ExtraActionButton_OnLoad(self)
	ExtraActionBarFrame:SetParent(self)
	ExtraActionBarFrame:SetSize(84,58)
	ExtraActionBarFrame:ClearAllPoints()
	ExtraActionBarFrame:SetPoint("TOPLEFT",SyncUI_ExtraActionButton)
	ExtraActionBarFrame.ignoreFramePositionManager = true
	
	for i = 1, ExtraActionBarFrame:GetNumChildren() do
		local button = _G["ExtraActionButton"..i]
		
		if button then
			StyleActionButton_Extra(button)
		end
	end
end


-- Alternate Power Bar
function SyncUI_AlternatePowerBar_OnLoad(self)
	local unit = "player"
	
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterUnitEvent("UNIT_POWER_BAR_SHOW",unit)
	self:RegisterUnitEvent("UNIT_POWER_BAR_HIDE",unit)
	
	self.StatusBar.power = 0
	self.StatusBar.maxPower = 0
	self.Title = self.ArtFrame.Title
	
	PlayerPowerBarAlt:UnregisterAllEvents()
end

function SyncUI_AlternatePowerBar_OnEvent(self,event,...)
	local unit = "player"
	
	if event == "PLAYER_ENTERING_WORLD" then
		if UnitAlternatePowerInfo(unit) then
			self:RegisterUnitEvent("UNIT_POWER",unit)
			self:RegisterUnitEvent("UNIT_MAXPOWER",unit)
			self.StatusBar.power = UnitPower(unit, ALTERNATE_POWER_INDEX)
			self.StatusBar.maxPower = UnitPowerMax(unit, ALTERNATE_POWER_INDEX)
			self:Show()
		end
	end
	if event == "UNIT_POWER" or event == "UNIT_MAXPOWER" then
		self.StatusBar.power = UnitPower(unit, ALTERNATE_POWER_INDEX)
		self.StatusBar.maxPower = UnitPowerMax(unit, ALTERNATE_POWER_INDEX)
		self:Show()
	end
	if event == "UNIT_POWER_BAR_SHOW" then
		self:RegisterUnitEvent("UNIT_POWER",unit)
		self:RegisterUnitEvent("UNIT_POWER",unit)
		self:Show()
	end
	if event == "UNIT_POWER_BAR_HIDE" then
		self:UnregisterEvent("UNIT_POWER")
		self:UnregisterEvent("UNIT_MAXPOWER")
		self.StatusBar.power = 0
		self.StatusBar.maxPower = 0
		self:Hide()
	end
	
end

function SyncUI_AlternatePowerBar_OnUpdate(self)
	local power, maxPower = self.StatusBar.power, self.StatusBar.maxPower
	local _, r, g, b, a = UnitAlternatePowerTextureInfo("player",2,0)

	self.StatusBar:SetStatusBarColor(r,g,b,a)
	self.StatusBar:SetMinMaxValues(0,maxPower)
	SmoothBar(self.StatusBar,power)

	if power == 0 then
		self.Title:SetText("")
	else
		self.Title:SetText(math.ceil(power*100/maxPower).."%")
	end
end
