
local _G = _G
local MAX_TOKENS = 10
local lastUpdate, updateInterval = 0, 1.00
local bars = {"MultiBarBottomRight","MultiBarRight","MultiBarLeft"}
local MicroButtons = {
	["Character"] = {
		"Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\InfoBar\\MicroButton_Character_Normal",
		"Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\InfoBar\\MicroButton_Character_Pushed"
	},
	["Spellbook"] = {
		"Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\InfoBar\\MicroButton_SpellBook_Normal",
		"Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\InfoBar\\MicroButton_SpellBook_Pushed"
	},
	["Talent"] = {
		"Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\InfoBar\\MicroButton_Talent_Normal",
		"Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\InfoBar\\MicroButton_Talent_Pushed"
	},
	["Achievement"] = {
		"Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\InfoBar\\MicroButton_Achievement_Normal",
		"Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\InfoBar\\MicroButton_Achievement_Pushed"
	},
	["Guild"] = {
		"Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\InfoBar\\MicroButton_Guild_Normal",
		"Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\InfoBar\\MicroButton_Guild_Pushed"
	},
	["LFD"] = {
		"Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\InfoBar\\MicroButton_PvE_Normal",
		"Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\InfoBar\\MicroButton_PvE_Pushed"
	},
	["EJ"] = {
		"Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\InfoBar\\MicroButton_Browser_Normal",
		"Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\InfoBar\\MicroButton_Browser_Pushed"
	},
	["Collections"] = {
		"Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\InfoBar\\MicroButton_Mount_Normal",
		"Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\InfoBar\\MicroButton_Mount_Pushed"
	}
}
local slots = {
	[1] = "Head",
	[3] = "Shoulders",
	[5] = "Chest",
	[6] = "Waist",
	[7] = "Legs",
	[8] = "Feet",
	[9] = "Wrist",
	[10] = "Hands",
	[16] = "Mainhand",
	[17] = "Offhand",
}
local MemoryColor = {
	[1] = "007FFF",		-- Light Blue (0-100 KB)
	[2] = "00FF00",		-- Green  (100-500 KB)
	[3] = "7FFF00",		-- Light Green (500-1 MB)
	[4] = "FFFF00",		-- Yellow (1 - 2,5 MB) 
	[5] = "FF7F00",		-- Orange (2.5 - 5 MB)
	[6] = "FF0000",		-- Red (5+ MB)
	[7] = "FFFFFF",		-- White (ignored)
}
local AddOnMemory = {}

local function GetMemoryColorCode(value,ignore)
	local index;
	
	if ignore then
		index = 7
	elseif value <= 100 then
		index = 1
	elseif value <= 512 then
		index = 2
	elseif value <= 1024 then
		index = 3
	elseif value <= 2560 then
		index = 4
	elseif value <= 5120 then
		index = 5
	elseif value > 5120 then
		index = 6
	end
	
	return MemoryColor[index]
end

local function GetFormattedMemory(value,ignore)
	local memory, suffix;
	local colorcode = GetMemoryColorCode(value,ignore)
	
	if value < 1024 then
		suffix = "KB"
		memory = string.format("%.0f",value)
	else
		suffix = "MB"
		memory = string.format("%.2f",value / 1024)
	end
	
	return tostring("|cFF"..colorcode..memory.."|r "..suffix)
end

local function GetMicroButtonIndex(prefix)
	if prefix == "Character" then 	return 1 end
	if prefix == "Spellbook" then 	return 2 end
	if prefix == "Talent" then 		return 3 end
	if prefix == "Achievement" then return 4 end
	if prefix == "Guild" then 		return 5 end
	if prefix == "LFD" then 		return 6 end
	if prefix == "EJ" then 			return 7 end
	if prefix == "Collections" then	return 8 end
end

local function UpdateExpBar(self)
	if not IsXPUserDisabled() and UnitLevel("player") < MAX_PLAYER_LEVEL then
		local xp, maxXP = UnitXP("player"), UnitXPMax("player")
		local perc = xp*100/maxXP
		
		for i = 1, MAX_TOKENS do
			local token = self["Token"..i]
			
			token:SetValue(perc)
		end
		
		self.Text:SetText(string.format("%.1f",perc).."%")
		
		if not self:IsShown() then
			self:Show()
		end
	else
		if self:IsShown()
			then self:Hide()
		end
	end
end

local function UpdateBonusExpBar(self)
	local bonusXP = GetXPExhaustion()
	
	if bonusXP and not IsXPUserDisabled() and UnitLevel("player") < MAX_PLAYER_LEVEL then
		local xp, maxXP = UnitXP("player"), UnitXPMax("player")
		local perc = (xp+bonusXP)*100/maxXP
		
		for i = 1, MAX_TOKENS do
			local token = self["Token"..i]
			
			token:SetValue(perc)
		end

		if not self:IsShown() then
			self:Show()
		end
	else
		if self:IsShown()
			then self:Hide()
		end
	end
end

local function UpdateRepBar(self)
	if GetWatchedFactionInfo() then
		local _, _, minRep, maxRep, curRep = GetWatchedFactionInfo()
		local tokenDistanceValue = (maxRep - minRep) / MAX_TOKENS
		
		for i = 1, MAX_TOKENS do
			local token = self["Token"..i]
			local minValue = minRep+((i*tokenDistanceValue)-tokenDistanceValue)
			local maxValue = minRep+((i*tokenDistanceValue))
			local perc = (curRep-minRep)*100/(maxRep-minRep)
			
			token:SetMinMaxValues(minValue,maxValue)
			token:SetValue(curRep)
			
			if maxRep <= -3000 then
				token:SetStatusBarColor(0.8,0.2,0)
			elseif maxRep <= 0 then
				token:SetStatusBarColor(0.8,0.2,0)
			elseif maxRep <= 3000 then
				token:SetStatusBarColor(0.8,0.6,0)
			else
				token:SetStatusBarColor(0,0.7,0)
			end

			self.Text:SetText(string.format("%.1f",perc).."%")
		end

		if not self:IsShown() then
			self:Show()
		end
	else
		if self:IsShown() then
			self:Hide()
		end
	end
end

local function UpdateLatency(self)
	local _,_,latencyHome,latencyWorld = GetNetStats();				
	local home = tonumber(string.format("%3.0f",latencyHome))
	local world = tonumber(string.format("%3.0f",latencyWorld))

	if home >= 100 then
		home = tostring("|cffFF0000"..home.."|r")
	else
		home = tostring("|cff00FF00"..home.."|r")
	end
	
	if world >= 100 then
		world = tostring("|cffFF0000"..world.."|r")
	else
		world = tostring("|cff00FF00"..world.."|r")
	end

	self:SetText(home.." | "..world.." ms")
end

local function UpdateMemory(self)
	UpdateAddOnMemoryUsage()
	local memory = 0
	
	for i = 1, GetNumAddOns() do
		if IsAddOnLoaded(i) then
			memory = memory + GetAddOnMemoryUsage(i)
		end
	end
	
	self:SetText(GetFormattedMemory(memory,true))
end

local function UpdateFPS(self)
	local framerate = math.floor(GetFramerate())

	if framerate >= 50 then
		framerate = tostring("|cff00FF00"..framerate.."|r")
	elseif framerate >= 25 then
		framerate = tostring("|cffFFFF00"..framerate.."|r")
	else
		framerate = tostring("|cffFF0000"..framerate.."|r")
	end
	
	self:SetText(framerate.." FPS")
end

local function UpdateDurability(self)
	local durability, maxDurability = 0, 0
	for slot, type in pairs(slots) do
		local dur, maxDur = GetInventoryItemDurability(slot)
		if dur then
			durability = durability + dur
			maxDurability = maxDurability + maxDur
		end
	end
	
	local perc = math.floor(durability * 100 / maxDurability)
	
	if perc >= 70 then
		perc = tostring("|cff00FF00"..perc.."%|r")
	elseif perc >= 30 then
		perc = tostring("|cffFFFF00"..perc.."%|r")
	else
		perc = tostring("|cffFF0000"..perc.."%|r")
	end
	
	self:SetText(perc.." Durability")
end

local function UpdateLootButton(self)
	local specID, specName = GetLootSpecialization(), nil
	local color = RAID_CLASS_COLORS[select(2, UnitClass("player"))]
	
	if specID == 0 then
		local name = select(2, GetSpecializationInfo(GetSpecialization() or 0))
		
		if not name then
			specName = "No Spec"
		else
			specName = "Current"
		end
	else
		specName = select(2, GetSpecializationInfoByID(specID))
	end
	
	self:SetText("|cffFFFFFFLoot:|r "..specName)
	self.Text:SetVertexColor(color.r, color.g, color.b)
end

local function UpdateInfoButton(self)
	local selection = SyncUI_Data.Options.InfoSelection
	
	if selection == 1 then UpdateLatency(self) end
	if selection == 2 then UpdateMemory(self) end
	if selection == 3 then UpdateFPS(self) end
	if selection == 4 then UpdateDurability(self) end
end


function SyncUI_InfoBar_OnUpdate(self,elapsed)
	lastUpdate = lastUpdate + elapsed;
	
	-- Status Text
	if lastUpdate > updateInterval then
		UpdateRepBar(self.RepBar)
		UpdateExpBar(self.XPBar)
		UpdateBonusExpBar(self.BonusXPBar)
		
		UpdateLootButton(self.LootSpecButton)
		UpdateInfoButton(self.InfoToolButton)

		lastUpdate = 0.00
	end
end

function SyncUI_MicroMenuBar_OnLoad(self)
	for prefix, info in pairs(MicroButtons) do
		local button = _G[prefix.."MicroButton"]
		local index = GetMicroButtonIndex(prefix)
		local normal, pushed = info[1], info[2]
		
		button:SetParent(self)
		button:SetSize(32,32)
		button:SetHitRectInsets(3,3,3,3)
		button:ClearAllPoints()
		button:SetPoint("LEFT",self,0+(26*(index-1)),0)
		button:SetNormalTexture(normal)
		button:SetPushedTexture(pushed)
		button:SetHighlightTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\Options\\OptionsMenuButton")
		button:GetHighlightTexture():SetTexCoord(0.875,1,0.75,1)
		button:SetDisabledTexture(normal)
		
		if prefix == "Guild" then
			button:SetScript("OnEnter",SyncUI_GuildMicroButton_OnEnter)
		end
	end
	
	--[[ Info: MicroButton Updater is disabled in
		* File: MainMenuBar\Actionbars\ActionBars.lua
		* Line: 487
		* Function: SyncUI_MainMenuBar_OnLoad
	]]
	
	SyncUI_DisableFrame("TalentMicroButton")
	SyncUI_DisableFrame("QuestLogMicroButton")
	SyncUI_DisableFrame("StoreMicroButton")
	SyncUI_DisableFrame("HelpMicroButton")
	SyncUI_DisableFrame("MainMenuMicroButton")
	SyncUI_DisableFrame("TalentMicroButtonAlert")
	SyncUI_DisableFrame("GuildMicroButtonTabard")
end

function SyncUI_ToggleBagButton_OnLoad(self)
	self:RegisterForClicks("LeftButtonUp","RightButtonUp")
	
	for i = 0, 3 do
		local slot = _G["CharacterBag"..i.."Slot"]

		slot:SetParent(self)
		slot:SetSize(33,33)
		slot:ClearAllPoints()
		slot:SetPoint("BOTTOM",self,"TOP",0,47+(37*(i-1)))
		slot:SetNormalTexture("")
		slot.IconBorder:SetTexture("")
		slot.icon:SetTexCoord(0.05,0.95,0.05,0.95)
		slot:Hide()
		slot:HookScript("OnUpdate", function(self)
			PaperDollItemSlotButton_Update(self)
		end)	

		slot.tex = slot:CreateTexture(nil,"OVERLAY")
		slot.tex:SetSize(37,37)
		slot.tex:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\MainMenuBar\\ActionButton_Small")
		slot.tex:SetTexCoord(0,0.14453125,0,0.578125)
		slot.tex:SetPoint("CENTER")
	end
end

function SyncUI_ExperienceBar_OnEnter(self)
	local currXP, maxXP = UnitXP("player"), UnitXPMax("player")
	local _, name, multiply = GetRestState()
	local expState = format(EXHAUST_TOOLTIP1, name, multiply*100)
	local bonusXP = GetXPExhaustion()
	
	GameTooltip:SetOwner(self, "ANCHOR_TOP",0,0)
	GameTooltip:AddDoubleLine("Exp","|cFFFFFFFF"..currXP.." / "..maxXP)
	if bonusXP then
		--bonusXP = SyncUI_GetNumberFormat(bonusXP)
		GameTooltip:AddDoubleLine("Bonus","|cFFFFFFFF"..bonusXP)
	end
	GameTooltip:AddDivider()
	GameTooltip:AddLine(expState)
	GameTooltip:Show()
end

function SyncUI_ReputationBar_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_TOP",0,0)
	
	if GetWatchedFactionInfo() then
		local faction, level, minRep,maxRep,value = GetWatchedFactionInfo()
		local repLvL = getglobal("FACTION_STANDING_LABEL"..level)
		local curRep, realMaxRep = value - minRep, maxRep - minRep
		
		GameTooltip:AddLine("|cFF00FF99Click|r to open reputation list",1,1,1)
		GameTooltip:AddDivider()
		GameTooltip:AddLine(faction)

		if value >= 0 then
			GameTooltip:AddDoubleLine("|cFF00FF00"..repLvL, "|cFFFFFFFF"..curRep.." / "..realMaxRep)
		else
			GameTooltip:AddDoubleLine("|cFFFF0000"..repLvL, "|cFFFFFFFF"..curRep.." / "..realMaxRep)
		end
		
		GameTooltip:Show()
	end
end

function SyncUI_GuildMicroButton_OnEnter(self)
	if not IsInGuild() then
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT",0,0)
		GameTooltip:AddLine("Guild Finder",1,1,1)
		GameTooltip:Show()
		
		return
	end
	
	local guildName = GetGuildInfo("player")
	local numMembers,_,numOnline = GetNumGuildMembers()
	local MotD = GetGuildRosterMOTD()
	
	GuildRoster()

	GameTooltip:SetOwner(self, "ANCHOR_RIGHT",0,0)
	GameTooltip:AddDoubleLine("|cFF2E9AFEOnline: |r"..numOnline,guildName,1,1,1,1,1,1)
	GameTooltip:AddLine(" ")
	GameTooltip:AddLine("|cFF2E9AFE"..GUILD_MOTD_LABEL..":")
	GameTooltip:AddLine(MotD,1,1,1,true)
	GameTooltip:AddDivider()
	
	for index = 1, numOnline do
		local member,_,_,lvl,class,zone,_,_,online,status,_,_,_,isMobile = GetGuildRosterInfo(index)

		if member then
			-- Member without Server
			member = gsub(member,"-"..GetRealmName(),"")
						
			-- Class Color
			if class then
				class = gsub(class,"Death Knight","DeathKnight")
				
				local color = SYNCUI_CLASS_COLORS[string.upper(class)]
				
				if color then
					local r,g,b = color.r,color.g,color.b
					
					member = SyncUI_GetColorizedText(member,r,g,b)
				end
			end
			
			-- mobile fix
			if member:find(UnitName("player")) then
				isMobile = false
			end
			
			-- remote
			if isMobile then
				zone = "Remote Chat" 
				member = member.." |TInterface\\ChatFrame\\UI-ChatIcon-ArmoryChat:14:14:0:0:64:64:0:64:0:58|t"
			end
			
			-- status
			if status == 1 then
				status = FRIENDS_TEXTURE_AFK
			elseif status == 2 then
				status = FRIENDS_TEXTURE_DND
			else
				status = FRIENDS_TEXTURE_ONLINE
			end
			
			status = "|T"..status..":14:14|t "
			
			-- Add Line
			GameTooltip:AddDoubleLine(status..member,zone,1,1,1,1,1,1)
		end
	end

	GameTooltip:Show()
end


function SyncUI_RaidToolMenu_OnLoad(self,relativeTo)
	self:ClearAllPoints()
	self:SetPoint("BOTTOM",relativeTo,"TOP")
	
	for i = 1, 4 do
		local button = self["Button"..i]
		
		if i == 1 then
			button:SetText("Raid Compilation")
			button:SetScript("OnClick", function()
				PlaySound("igMainMenuOptionCheckBoxOn");
				ToggleFriendsFrame(4)
				SyncUI_SlideOut(self)
			end)
		end
		if i == 2 then
			button:SetText("Raid Lockouts")
			button:SetScript("OnClick", function()
				PlaySound("igMainMenuOptionCheckBoxOn")
				if not SyncUI_OptionsMenu:IsShown() then
					if RaidInfoFrame:IsShown() then
						RaidInfoFrame:Hide()
					else
						RaidInfoFrame:Show()
					end
				end
				SyncUI_SlideOut(self)
			end)
		end
		if i == 3 then
			button:SetText("World Marks")
			button:SetScript("OnClick", function()
				SyncUI_ToggleWorldMarkerFrame()
				PlaySound("igMainMenuOptionCheckBoxOn");
				SyncUI_SlideOut(self)
			end)
		end
		if i == 4 then
			button:SetText("Ready Check")
			button:SetScript("OnClick", function()
				PlaySound("igMainMenuOptionCheckBoxOn");
				DoReadyCheck()
				SyncUI_SlideOut(self)
			end)
		end
	end
	
	RaidInfoFrame:SetParent(self:GetParent())
	RaidInfoFrame:ClearAllPoints()
	RaidInfoFrame:SetPoint("CENTER",UIParent)
	tinsert(UISpecialFrames,"RaidInfoFrame");
	
	--FriendsFrameTab4:Hide()
end

function SyncUI_LootSpecMenu_OnLoad(self,relativeTo)
	self:ClearAllPoints()
	self:SetPoint("BOTTOM",relativeTo,"TOP")
	
	for i = 1, 4 do
		local button = self["Button"..i]
		local class = select(2, UnitClass("player"))
		local id, name = GetSpecializationInfo(i-1)
		
		if class == "DRUID" then
			id, name = GetSpecializationInfo(i)
		elseif i == 1 then
			id, name = 0, "Current Spec"
		end
		
		if id and name then
		button:SetText(name)
		button:SetScript("OnClick", function()
			PlaySound("igMainMenuOptionCheckBoxOn")
			SetLootSpecialization(id)
			SyncUI_SlideOut(self)
		end)
		end
	end
end

function SyncUI_InfoToolMenu_OnLoad(self,relativeTo)
	self:ClearAllPoints()
	self:SetPoint("BOTTOM",relativeTo,"TOP")
	
	for i = 1, 4 do
		local button = self["Button"..i]
		
		button:SetScript("OnClick", function()
			PlaySound("igMainMenuOptionCheckBoxOn")
			SyncUI_Data.Options.InfoSelection = i
			SyncUI_SlideOut(self)
		end)
		
		if i == 1 then
			button:SetText("Latency")
		end
		if i == 2 then
			button:SetText("Memory")
		end
		if i == 3 then
			button:SetText("FPS")
		end
		if i == 4 then
			button:SetText("Durability")
		end
	end
end

function SyncUI_InfoToolButton_OnEnter(self)
	local selection = SyncUI_Data.Options.InfoSelection
	
	if selection == 2 then	-- AddOn Memory
		local isDivider = false
		
		GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
		GameTooltip:AddLine("|cFF00FF99".."AddOn Memory")
		GameTooltip:SetPrevLineJustify("CENTER")
		GameTooltip:AddDivider()

		AddOnMemory = {}

		local maxMemory = gcinfo()
		local addOnMemory = 0
	
		for i = 1, GetNumAddOns(), 1 do
			if IsAddOnLoaded(i) then
				local memory = GetAddOnMemoryUsage(i)
				local addOn, name = GetAddOnInfo(i)

				addOnMemory = addOnMemory + memory
				
				tinsert(AddOnMemory,{name,memory})
			end
		end

		table.sort(AddOnMemory, function(a,b) return a[2] > b[2] end)

		for i = 1, #AddOnMemory do
			local name, memory = AddOnMemory[i][1],AddOnMemory[i][2]

			if memory < 1024 then
				if not isDivider then
					GameTooltip:AddDivider()
					isDivider = true
				end
			end

			GameTooltip:AddDoubleLine(name,GetFormattedMemory(memory),1,1,1,1,1,1)
		end

		GameTooltip:AddDivider()
		GameTooltip:AddDoubleLine("Blizzard UI",GetFormattedMemory(maxMemory-addOnMemory,true),1,1,1,1,1,1)
		GameTooltip:Show()
	end

	if selection == 4 then	-- Inventory Durability
		GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
		GameTooltip:AddLine("|cFF00FF99".."Durability")
		GameTooltip:SetPrevLineJustify("CENTER")
		GameTooltip:AddDivider()
		
		for slot, type in pairs(slots) do
			local dur, maxDur = GetInventoryItemDurability(slot)
			local perc;
			
			if dur then
				perc = math.floor(dur*100/maxDur)
			end
			
			if perc then
				if perc >= 70 then
					perc = tostring("|cff00FF00"..perc.."|r")
				elseif perc >= 30 then
					perc = tostring("|cffFFFF00"..perc.."|r")
				else
					perc = tostring("|cffFF0000"..perc.."|r")
				end
				
				GameTooltip:AddDoubleLine(type,perc.." %",1,1,1,1,1,1)
			end
		end
		
		GameTooltip:Show()
	end
end
