
local _G = _G
local tinsert = table.insert
local maxMinimapButtons = 5
local PreButtonList, ButtonList, IgnoreList = {}, {}, {
	"MiniMapWorldMapButton",
	"MiniMapRecordingButton",
	"MiniMapVoiceChatFrame",
	"MinimapZoomIn",
	"MinimapZoomOut",
	"TimeManagerClockButton",
	"GameTimeFrame",
}


-- Minimap
function SyncUI_Minimap_OnLoad(self)
	SyncUI_Minimap_OnStyling(self)
	SyncUI_RegisterDragFrame(self,"Minimap")
	
	-- Zoom Function
	Minimap:EnableMouseWheel(true)
	Minimap:SetScript("OnMouseWheel", function(_, zoom)
		if zoom > 0 then
			Minimap_ZoomIn()
		else
			Minimap_ZoomOut()
		end
	end)
	
	do	-- Disable Stuff
		SyncUI_DisableFrame(MiniMapInstanceDifficulty,ignore)
		SyncUI_DisableFrame(GuildInstanceDifficulty,ignore)
		SyncUI_DisableFrame(MiniMapChallengeMode,ignore)	
	end
	
	do	-- Garrison Minimap Alert
		local button = GarrisonLandingPageMinimapButton
		
		button.AlertBG:SetSize(200,50)
		button.AlertBG:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\AlertBG")
		button.AlertBG:ClearAllPoints()
		button.AlertBG:SetPoint("TOP",SyncUI_UIParent,0,-50)
		
		button.AlertText:ClearAllPoints()
		button.AlertText:SetPoint("CENTER",button.AlertBG)
		button.AlertText:SetFontObject(SyncUI_GameFontOutline_Small)
		button.AlertText:SetJustifyH("CENTER")
		
		if UnitFactionGroup("player") == "Horde" then
			button:GetNormalTexture():SetAtlas("GarrLanding-MinimapIcon-Horde-Up", true)
			button:GetPushedTexture():SetAtlas("GarrLanding-MinimapIcon-Horde-Down", true)
		else
			button:GetNormalTexture():SetAtlas("GarrLanding-MinimapIcon-Alliance-Up", true)
			button:GetPushedTexture():SetAtlas("GarrLanding-MinimapIcon-Alliance-Down", true)
		end
	end
		
	do	-- Garrison Mission Alert
		local frame = GarrisonMissionAlertFrame
		frame:ClearAllPoints()
		frame:SetPoint("TOP",SyncUI_UIParent,0,-50)
		frame.SetPoint = function() end
		
		frame.Name:ClearAllPoints()
		frame.Name:SetPoint("CENTER")
	end

	do	-- Help Ticket Button
		local frame = HelpOpenWebTicketButton
		frame:SetParent(self)
		frame:SetScale(0.8)
		frame:ClearAllPoints()
		frame:SetPoint("BOTTOMLEFT",-7,-4)
	end
	
	do	-- Ticket Status Frame
		local frame = TicketStatusFrame
		local button = TicketStatusFrameButton
		local text = TicketStatusTitleText
		button:SetBackdrop(SyncUI_Backdrop)
		text:SetFontObject(SyncUI_GameFontOutline_Medium)
		
		frame:ClearAllPoints()
		frame:SetPoint("TOP",self,"BOTTOM",-15,-5)
	end
end

function SyncUI_Minimap_OnStyling(self)
	-- Hide Stuff
	local frames = {
		"MiniMapVoiceChatFrame",
		"MiniMapWorldMapButton",
		"MinimapZoneTextButton",
		"MiniMapMailFrame",
		"MinimapNorthTag",
		"MinimapZoomOut",
		"MinimapZoomIn",
		"MinimapBackdrop",
		"GameTimeFrame",
		"MinimapBorderTop",
	}
	for _, frame in pairs(frames) do
		_G[frame]:Hide()
		_G[frame].Show = function() end
	end

	local frame = _G[self:GetName()]
	local scale = 0.8
	-- Minimap
    MinimapCluster:EnableMouse(false)
	Minimap:SetParent(frame)
	Minimap:SetScale(scale)
	Minimap:SetSize(113/scale,113/scale)
	Minimap:ClearAllPoints()
	Minimap:SetPoint("TOP",frame,0,-11/scale)
	Minimap:SetMaskTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\Minimap\\Minimap_Mask")
	Minimap:SetHitRectInsets(0,0,0,0)
	Minimap:SetArchBlobRingScalar(0);
	Minimap:SetQuestBlobRingScalar(0);
	
	UIParent:HookScript("OnShow",function()
		Minimap:Show()
	end)
	UIParent:HookScript("OnHide",function()
		Minimap:Hide()
	end)
end

function SyncUI_Minimap_SetTooltip(self,pvpType,factionName)
    local zoneName = GetZoneText()
    local subzoneName = GetSubZoneText()
	
    if subzoneName == zoneName then
		subzoneName = ""
    end
	
	GameTooltip:SetOwner(self, "ANCHOR_LEFT")	
    GameTooltip:AddLine( zoneName, 1.0, 1.0, 1.0 )
	
	if pvpType == "sanctuary" then
		GameTooltip:AddLine( subzoneName, 0.41, 0.8, 0.94 )
		GameTooltip:AddLine(SANCTUARY_TERRITORY, 0.41, 0.8, 0.94)
	elseif pvpType == "arena" then
		GameTooltip:AddLine( subzoneName, 1.0, 0.1, 0.1 )
		GameTooltip:AddLine(FREE_FOR_ALL_TERRITORY, 1.0, 0.1, 0.1)
	elseif pvpType == "friendly" then
		GameTooltip:AddLine( subzoneName, 0.1, 1.0, 0.1 )
		GameTooltip:AddLine(format(FACTION_CONTROLLED_TERRITORY, factionName), 0.1, 1.0, 0.1)
	elseif pvpType == "hostile" then
		GameTooltip:AddLine( subzoneName, 1.0, 0.1, 0.1 )
		GameTooltip:AddLine(format(FACTION_CONTROLLED_TERRITORY, factionName), 1.0, 0.1, 0.1)
	elseif pvpType == "contested" then
		GameTooltip:AddLine( subzoneName, 1.0, 0.7, 0.0 )
		GameTooltip:AddLine(CONTESTED_TERRITORY, 1.0, 0.7, 0.0)
	elseif pvpType == "combat" then
		GameTooltip:AddLine( subzoneName, 1.0, 0.1, 0.1 )
		GameTooltip:AddLine(COMBAT_ZONE, 1.0, 0.1, 0.1)
	else
		GameTooltip:AddLine( subzoneName, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b )
	end
	
    GameTooltip:Show()
end

function SyncUI_Minimap_InstanceType_Update(self,event,...)
	if IsInInstance() then
		local _, instance, type = GetInstanceInfo()
		local text
		
		if type == 0 then
			text = ""
		elseif type == 1 then	-- Dungeon
			text = "5x"
		elseif type == 2 then	-- Dungeon HC
			text = "5x HC"
		elseif type == 3 then	-- 10x Raid
			text = "10x"
		elseif type == 4 then	-- 25x Raid
			text = "25x"
		elseif type == 5 then	-- 10x Raid HC
			text = "10x HC"
		elseif type == 6 then	-- 25x Raid HC
			text = "25x HC"
		elseif (type == 7 or type == 17) then	-- LFR
			text = "LFR"
		elseif type == 8 then	-- Challenge Mode
			text = "CM"
		elseif type == 9 then	-- 40x Raid
			text = "40x"
		elseif type == 11 then	-- Heroic Scenario
			text = "3x HC"
		elseif type == 12 then	-- Scenario
			text = "3x"
		elseif type == 14 then	-- Normal
			text = "Normal"
		elseif type == 15 then	-- Heroic
			text = "Heroic"
		elseif type == 16 then	-- Mythic Raid
			text = "20 Myth"
		elseif type == 23 then	-- Mythic Dungeon
			text = "5 Myth"
		elseif type == 24 then	-- timewalk
			text = "Timewalk"
		end

		if GuildInstanceDifficulty:IsShown() then
			text = text.." (G)"
		end

		if instance == "arena" then
			if IsArenaSkirmish then
				text = "Skirmish"
			else
				text = "Arena"
			end
		end
		
		self.Text:SetText(text)
	else
		self.Text:SetText("")
	end
end

function SyncUI_Minimap_ClockButton_OnClick(self,button)
	if button == "LeftButton" then
		PlaySound("igMainMenuOptionCheckBoxOn")
		
		if IsShiftKeyDown() then
			TimeManager_ToggleLocalTime()
			SyncUI_Minimap_ClockButton_OnEnter(self)
		else
			TimeManager_ToggleTimeFormat()
		end
	end
	if button == "RightButton" then
		PlaySound("igMainMenuQuit")
		Stopwatch_Toggle()
	end
end

function SyncUI_Minimap_ClockButton_OnEnter(self)
	local string;
	
	if GetCVarBool("timeMgrUseLocalTime") then
		string = "Local Time"
	else
		string = "Server Time"
	end
	
	GameTooltip:SetOwner(self, "ANCHOR_BOTTOM",0,-5)
	GameTooltip:AddLine("|cFF00FF99Click|r to toggle 24 hour mode",1,1,1)
	GameTooltip:AddLine("|cFF00FF99Shift-Click|r to toggle display",1,1,1)
	GameTooltip:AddLine("|cFF00FF99Right-Click|r to toggle Stopwatch",1,1,1)
	GameTooltip:AddDivider()
	GameTooltip:AddDoubleLine("|cFF00FF99Display:|r",string,1,1,1,1,1,1)								
	GameTooltip:Show()
end


-- Minimap Button Frame
local function SkinButton(index,button)
	local parent = SyncUI_Minimap.ButtonFrame["Button"..index]

	if not parent then
		local numClipBoardButtons = SyncUI_Minimap.ButtonFrame.ClipBoard:GetNumChildren()
		local numButtons = maxMinimapButtons + numClipBoardButtons
		
		parent = CreateFrame("Frame",nil,SyncUI_Minimap.ButtonFrame.ClipBoard,"SyncUI_MinimapButtonTemplate")
		parent:SetPoint("TOP",SyncUI_Minimap.ButtonFrame.Button5,"BOTTOM",0,-15-((numClipBoardButtons)*20))
		
		SyncUI_Minimap.ButtonFrame["Button"..numButtons+1] = parent
	end
	
	if parent then
		local icon = button.icon or button.texture or _G[button:GetName().."Icon"] or _G[button:GetName().."_Icon"] or button:GetNormalTexture()
		
		-- Battleground Targets Fix
		if button == BattlegroundTargets_MinimapButton then
			local pushed = button:GetPushedTexture()
			pushed:SetSize(15,15)
			pushed:SetTexCoord(0,1,0,1)
			pushed:ClearAllPoints()
			pushed:SetPoint("CENTER")
		end
		
		parent:SetAlpha(1)
		
		button:SetParent(parent)
		button:SetToplevel(false)
		button:SetSize(19,19)
		button:ClearAllPoints()
		button:SetPoint("CENTER",parent,0,0)
		button:SetHitRectInsets(0,0,0,0)
		button:RegisterForDrag()
		button:SetHighlightTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\Minimap\\Minimap")
		button:GetHighlightTexture():SetTexCoord(0.875,0.95703125,0.125,0.20703125)

		if icon then
			button:HookScript("OnMouseDown", function()
				icon:SetTexCoord(0,1,0,1)
			end)
			button:HookScript("OnMouseUp", function()
				icon:SetTexCoord(0.05,0.95,0.05,0.95)
			end)

			icon:SetSize(15,15)
			icon:SetTexCoord(0.05,0.95,0.05,0.95)
			icon:ClearAllPoints()
			icon:SetPoint("CENTER")
			icon.ClearAllPoints = function() end
			icon.SetPoint = function() end
		end
				
		for _, region in pairs({button:GetRegions()}) do
			if region:GetObjectType() == "Texture" then
				local file = region:GetTexture()
				if file ~= nil and (file:find("Border") or file:find("Background") or file:find("AlphaMask")) then
					region:SetTexture("")
				end
			end
		end
	end
end

local function RegisterButton(button)
	local ignore = {"HandyNotesPin", "GatherNote", "GatherMatePin"}
	local name = button:GetName()
	local type = button:GetObjectType()

	if name then
		if type ~= "Button" or SyncUI_GetTableEntry(PreButtonList,button) or SyncUI_GetTableEntry(IgnoreList,name) then
			return
		end
		
		for _, ref in pairs(ignore) do
			if name:find(ref) then
				return
			end
		end
	end
	
	if not name or not type then
		return
	end
	
	if not button.isHooked then
		button:SetScript("OnShow", function() SyncUI_Minimap.ButtonFrame.forceUpdate = true end)
		button:SetScript("OnHide", function() SyncUI_Minimap.ButtonFrame.forceUpdate = true end)
		button.isHooked = true
	end

	tinsert(PreButtonList,button)
end

local function GetButtons()
	do	-- Garrison Minimap Button
		local button = GarrisonLandingPageMinimapButton
		
		RegisterButton(button)
		PreButtonList[1] = button
	end
	
	for _, button in pairs({Minimap:GetChildren()}) do
		RegisterButton(button)
	end
end

local function UpdateButtons()
	local numClipBoardButtons = SyncUI_Minimap.ButtonFrame.ClipBoard:GetNumChildren()
	
	for i = 1, maxMinimapButtons + numClipBoardButtons do
		SyncUI_Minimap.ButtonFrame["Button"..i]:SetAlpha(0)
	end

	wipe(ButtonList)
	
	for _, button in ipairs(PreButtonList) do
		if button:IsShown() then
			tinsert(ButtonList,button)
		end
	end

	table.sort(ButtonList, function(a,b)
		local button = GarrisonLandingPageMinimapButton
		
		if a and b ~= button then
			return a:GetName() < b:GetName()
		end
	end)

	for index, button in ipairs(ButtonList) do
		SkinButton(index,button)
		
		--[[
		if index <= maxMinimapButtons then
			SkinButton(index,button)
		else
			local i = index - (maxMinimapButtons + 1)
			button:ClearAllPoints()
			button:SetPoint("TOP",SyncUI_Minimap.ButtonFrame,"BOTTOM",-2,0-(i*22))
		end
		--]]
	end
end


function SyncUI_MinimapButtonFrame_OnEvent(self,event,...)
	if event == "PLAYER_ENTERING_WORLD" then
		self.numButtons = Minimap:GetNumChildren()
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")	
	end
	
	if event == "GARRISON_SHOW_LANDING_PAGE" then
		self:UnregisterEvent("GARRISON_SHOW_LANDING_PAGE")
		GarrisonLandingPageMinimapButton:Show()
	end
	
	GetButtons()
	UpdateButtons()
end

function SyncUI_MinimapButtonFrame_OnUpdate(self)
	local numChildren = Minimap:GetNumChildren()

	if self.numButtons ~= numChildren then
		self.numButtons = Minimap:GetNumChildren()

		GetButtons()
		UpdateButtons()
	end

	if self.forceUpdate then
		self.forceUpdate = false
		
		UpdateButtons()
	end
end
