﻿
local _G = _G
local maxPlateDebuffs = 3
local GUIDList = {}
local spellDurations = {}

local function GetHealthBarColor(nameplate,healthBar)
	local r,g,b = healthBar:GetStatusBarColor()
	local class;

	if nameplate.GUID then
		class = select(2, GetPlayerInfoByGUID(nameplate.GUID))
	end
	
	if class then
		local classColor = SYNCUI_CLASS_COLORS[class]
		r,g,b = classColor.r, classColor.g, classColor.b
	else
		if r == 0 and g == 0 and b >= 0.8 then
			r,g,b = 0.2,0.5,1.0
		elseif r >= 0.8 and g == 0 and b == 0 then
			r,g,b = 1.0,0.1,0.1
		end
	end
	
	return r,g,b
end

local function RepositionNamePlates()
	SyncUI_PlateFrame:Hide()
	
	for nameplate, plate in pairs(SyncUI_PlateFrame.list) do
		plate:Hide()
		if nameplate:IsShown() then
			plate:SetPoint("CENTER", nameplate,0,-7)
			plate:SetAlpha(nameplate:GetAlpha())
			plate:Show()
		end
	end
	
	SyncUI_PlateFrame:Show()
end

local function UpdateGUID(nameplate)
	local isTarget, isMouseover
	
	if UnitGUID("target") and UnitExists("target") and not UnitIsUnit("target","player") and not UnitIsDead("target") then
		isTarget = true
	end
	
	if UnitGUID("mouseover") and UnitExists("mouseover") and not UnitIsUnit("mouseover","player") and not UnitIsDead("mouseover") then
		isMouseover = true
	end

	if isTarget and nameplate:GetAlpha() == 1 then
		nameplate.GUID = UnitGUID("target")
		nameplate.unit = "target"
	elseif isMouseover and nameplate.highlight:IsShown() then
		nameplate.GUID = UnitGUID("mouseover")
		nameplate.unit = "mouseover"
	else
		nameplate.GUID = nil
		nameplate.unit = nil
	end

	-- Update Group GUIDs
	if IsInGroup() then
		local name = nameplate.unitName

		nameplate.ignoreAuras = false
			
		for i = 1, GetNumGroupMembers() do
			local unitID, unitName

			if IsInRaid() then
				unitID = "raid"..i
			elseif UnitExists("party1") then
				unitID = "party"..i
			end
			
			if unitID then
				unitName = UnitName(unitID)
			end
			
			if unitName and unitName == name then
				nameplate.GUID = UnitGUID(unitID)
				nameplate.unit = unitID
				nameplate.ignoreAuras = true
			end
		end
	end

	-- Update Arena GUIDs
	if IsActiveBattlefieldArena() or IsArenaSkirmish() then
		local name = nameplate.unitName
	
		for i = 1, GetNumArenaOpponents() do
			local unitID = "arena"..i
			local unitName = UnitName(unitID)
			
			if unitName and unitName == name then
				nameplate.GUID = UnitGUID(unitID)
				nameplate.unit = unitID
			end
		end
	end

	-- Create info table for each new GUID
	if nameplate.GUID and not GUIDList[nameplate.GUID] then
		GUIDList[nameplate.GUID] = {}
		
		for index = 1, maxPlateDebuffs do
			GUIDList[nameplate.GUID]["Debuff"..index] = {}
			GUIDList[nameplate.GUID]["Debuff"..index]["SpellID"] = nil
			GUIDList[nameplate.GUID]["Debuff"..index]["Expiration"] = 0
			GUIDList[nameplate.GUID]["Debuff"..index]["Icon"] = ""
		end
	end	
end


local function PlateAura_OnCacheInfo(self,nameplate)
	if nameplate.unit then
		for index = 1, maxPlateDebuffs do
			local debuff = self["Debuff"..index]
			local name,_,icon,_,_,duration,expirationTime,unitCaster,_,_,spellID = UnitDebuff(nameplate.unit,index,"PLAYER")
			
			if spellID then
				spellDurations[spellID] = duration
			end

			if name and (unitCaster == "player" or unitCaster == "pet") then
				if not GUIDList[nameplate.GUID] then
					GUIDList[nameplate.GUID] = {}
				end
				if not GUIDList[nameplate.GUID]["Debuff"..index] then
					GUIDList[nameplate.GUID]["Debuff"..index] = {}
				end
				
				GUIDList[nameplate.GUID]["Debuff"..index]["SpellID"] = spellID
				GUIDList[nameplate.GUID]["Debuff"..index]["Expiration"] = expirationTime
				GUIDList[nameplate.GUID]["Debuff"..index]["Icon"] = icon
			elseif not nameplate.ignoreAuras then
				if not GUIDList[nameplate.GUID] then
					GUIDList[nameplate.GUID] = {}
				end
				if not GUIDList[nameplate.GUID]["Debuff"..index] then
					GUIDList[nameplate.GUID]["Debuff"..index] = {}
				end
				
				GUIDList[nameplate.GUID]["Debuff"..index]["SpellID"] = nil
				GUIDList[nameplate.GUID]["Debuff"..index]["Expiration"] = 0
				GUIDList[nameplate.GUID]["Debuff"..index]["Icon"] = ""
			end
		end
	end
end

local function PlateAura_OnUpdate(self,nameplate)
	if not nameplate.GUID then
		for index = 1, maxPlateDebuffs do
			local debuff = self["Debuff"..index]
		
			if debuff then
				debuff:Hide()
			end
		end
	end

	if nameplate.GUID and not nameplate.ignoreAuras then
		for guid in pairs(GUIDList) do
			if nameplate.GUID == guid then
				for index = 1, maxPlateDebuffs do
					local debuff = self["Debuff"..index]
					local spellID = GUIDList[nameplate.GUID]["Debuff"..index]["SpellID"]

					if spellID then
						local icon = GUIDList[nameplate.GUID]["Debuff"..index]["Icon"]
						local expire = GUIDList[nameplate.GUID]["Debuff"..index]["Expiration"]
						local timeLeft = expire - GetTime()

						if timeLeft and timeLeft > 0 or expire == 0 then
							if timeLeft >=60 then
								debuff.Timer:SetText(math.floor(timeLeft/60).."m")
							elseif timeLeft >=10 then
								debuff.Timer:SetText(math.floor(timeLeft))
							elseif timeLeft >= 0.05 then
								debuff.Timer:SetText(string.format("%.1f",timeLeft))
							else
								debuff.Timer:SetText()
							end
														
							debuff.Icon:SetTexture(icon)
							debuff:Show()
						else
							debuff:Hide()
						end
					else
						debuff:Hide()
					end
				end
			end
		end
	end
end


local function NamePlate_OnUpdate(self,nameplate)
	local healthBar = nameplate.healthbar
	local castBar = nameplate.castbar
	local shield = castBar.shield
	local highlight = nameplate.highlight
	local maxHealth = select(2, healthBar:GetMinMaxValues())
	local health = healthBar:GetValue()
	local perc = math.ceil(health*100/maxHealth)
	
	-- healthBar
	self.HealthBar:SetStatusBarColor(GetHealthBarColor(nameplate,healthBar))
	self.HealthBar:SetMinMaxValues(0,maxHealth)
	self.Health:SetText(perc.."%")
	SmoothBar(self.HealthBar,health)
	
	-- alpha fix
	if nameplate:GetAlpha() < 1 then
		self.HealthBar:SetAlpha(0.35)
		self.CastingBar.StatusBar:SetAlpha(0.35)
	else
		self.HealthBar:SetAlpha(1)
		self.CastingBar.StatusBar:SetAlpha(1)
	end

	-- 	castingBar
	if castBar:IsShown() then
		self.CastingBar:Show()
		self.CastingBar.Icon:SetTexture(castBar.icon:GetTexture())
		self.CastingBar.Name:SetText(castBar.name:GetText())
		self.CastingBar.StatusBar:SetMinMaxValues(castBar:GetMinMaxValues())
		self.CastingBar.StatusBar:SetValue(castBar:GetValue())
		
		if shield:IsShown() then
			self.CastingBar.StatusBar:SetStatusBarColor(1,0,0)
			self.CastingBar.Shield:Show()
		else
			self.CastingBar.StatusBar:SetStatusBarColor(castBar:GetStatusBarColor())
			self.CastingBar.Shield:Hide()
		end
	else
		self.CastingBar:Hide()
	end

	-- highlight
	if highlight:IsShown() then
		self.Highlight:Show()
	else
		self.Highlight:Hide()
	end
	
	UpdateGUID(nameplate)
	PlateAura_OnCacheInfo(self,nameplate)
	PlateAura_OnUpdate(self,nameplate)
end

local function NamePlate_OnShow(self,nameplate)
	local name = nameplate.name:GetText()
	local level = nameplate.level:GetText()
	local healthBar = nameplate.healthbar
	local r,g,b = GetHealthBarColor(nameplate,healthBar)
	local width = nameplate:GetWidth()
	local scale = nameplate:GetScale()
	
	self.Name:SetText(name)
	self.HealthBar:SetStatusBarColor(r,g,b)

	if name then
		nameplate.unitName = gsub(name, " %(%*%)", "")
	end
	
	-- update size out of combat
	if width > 100 then
		if not InCombatLockdown() then
			nameplate:SetScale(1)
		end
		self:SetScale(1)
	else
		if not InCombatLockdown() then
			nameplate:SetScale(2)
		end
		self:SetScale(0.8)
	end
	
	if nameplate.boss:IsShown() then 
		self.Level:SetText("")
		self.Boss:Show()
		self.Elite:Hide()
	else
		self.Level:SetText(level)
		self.Level:SetVertexColor(SyncUI_GetEffectiveLevelColor(level))
		self.Boss:Hide()

		if nameplate.elite:IsShown() then
			self.Elite:Show()
		else
			self.Elite:Hide()
		end
	end
end

local function NamePlate_OnHide(self,nameplate)
	nameplate.GUID = nil
	
--[[
	if GUIDList[nameplate.GUID] then
		GUIDList[nameplate.GUID] = nil
	end
--]]

	for index = 1, maxPlateDebuffs do
		self["Debuff"..index]:Hide()
	end
end

local function NamePlate_OnRegister(nameplate)
	do	-- Register ParentKeys
		nameplate.unit, nameplate.GUID, nameplate.hasStyled = nil, nil, true	
		nameplate.name = nameplate.NameContainer.NameText
		nameplate.highlight = nameplate.ArtContainer.Highlight
		nameplate.level = nameplate.ArtContainer.LevelText
		nameplate.raid = nameplate.ArtContainer.RaidTargetIcon
		nameplate.elite = nameplate.ArtContainer.EliteIcon
		nameplate.boss = nameplate.ArtContainer.HighLevelIcon	
		nameplate.healthbar = nameplate.ArtContainer.HealthBar
		nameplate.absorbbar = nameplate.ArtContainer.AbsorbBar
		nameplate.castbar = nameplate.ArtContainer.CastBar
		nameplate.castbar.icon = nameplate.ArtContainer.CastBarSpellIcon
		nameplate.castbar.name = nameplate.ArtContainer.CastBarText
		nameplate.castbar.shield = nameplate.ArtContainer.CastBarFrameShield
		nameplate.ArtContainer:SetAlpha(0)
		nameplate.NameContainer:SetAlpha(0)
	end
	
	SyncUI_PlateFrame.list[nameplate] = CreateFrame("Frame","New"..nameplate:GetName(),SyncUI_PlateFrame,"SyncUI_NamePlateTemplate")
	local plate = SyncUI_PlateFrame.list[nameplate]
	plate:SetFrameStrata("BACKGROUND")
	plate:SetFrameLevel(nameplate:GetFrameLevel())
	plate:SetScript("OnUpdate",function() NamePlate_OnUpdate(plate,nameplate) end)
	
	nameplate.raid:SetParent(plate.ArtFrame)
	nameplate.raid:SetSize(15,15)
	nameplate.raid:SetDrawLayer("ARTWORK",1)
	nameplate.raid:ClearAllPoints()
	nameplate.raid:SetPoint("LEFT",plate,-8,0)

	nameplate:HookScript("OnShow",function() NamePlate_OnShow(plate,nameplate) end)
	nameplate:HookScript("OnHide",function() NamePlate_OnHide(plate,nameplate) end)
	
	NamePlate_OnShow(plate,nameplate)
end


local function PlateFrame_OnLoad(self)
	local update, interval, support = 0, 1.0, {"Kui_Nameplates","TidyPlates","InfinityPlates"}

	for _, addon in ipairs(support) do
		if IsAddOnLoaded(addon) then
			return
		end
	end

	WorldFrame:HookScript("OnUpdate", function(self,elapsed)
		RepositionNamePlates()
		
		update = update + elapsed
		
		if update > interval then
			for _, frame in pairs({self:GetChildren()}) do
				--local name = nameplate:GetName()
				--if name and name:find("NamePlate") and not nameplate.isInit then
				if frame.ArtContainer and not frame.hasStyled then
					NamePlate_OnRegister(frame)
				end
			end
			
			update = 0
		end	
	end)

end

local function PlateFrame_OnCombatLogEvent(self,event,...)
	local subevent = select(2, ...) 
	local caster, casterName = select(4, ...)

	if caster == UnitGUID("player") then
		local unit, unitName = select(8, ...)
		
		if subevent == "SPELL_AURA_APPLIED" or subevent == "SPELL_AURA_REFRESH" or subevent == "SPELL_AURA_APPLIED_DOSE" or subevent == "SPELL_AURA_REMOVED_DOSE" then
			local spellID, spellName = select(12, ...)
			local duration = spellDurations[spellID] or 0
			local expirationTime = duration + GetTime()
			local icon = GetSpellTexture(spellID)

			local index = 1

			if not GUIDList[unit] then
				GUIDList[unit] = {}
			end
			
			for i = 1, maxPlateDebuffs do
				if not GUIDList[unit]["Debuff"..i] then
					GUIDList[unit]["Debuff"..i] = {}
					GUIDList[unit]["Debuff"..i]["SpellID"] = nil
					GUIDList[unit]["Debuff"..i]["Expiration"] = 0
					GUIDList[unit]["Debuff"..i]["Icon"] = ""
				end
				
				local spell = GUIDList[unit]["Debuff"..i]["SpellID"]
				
				if not spell or spellID == spell then
					break
				elseif index < maxPlateDebuffs then
					index = index + 1
				end					
			end
			
			GUIDList[unit]["Debuff"..index]["SpellID"] = spellID
			GUIDList[unit]["Debuff"..index]["Expiration"] = expirationTime
			GUIDList[unit]["Debuff"..index]["Icon"] = icon
		end
		
		if subevent == "SPELL_AURA_REMOVED" or subevent == "SPELL_AURA_BROKEN" or subevent == "SPELL_AURA_BROKEN_SPELL" then
			local spellID, spellName = select(12, ...)
			local index = 1
			
			if not GUIDList[unit] then
				GUIDList[unit] = {}
			end
			
			for i = 1, maxPlateDebuffs do
				if not GUIDList[unit]["Debuff"..i] then
					return
				end
				
				local spell = GUIDList[unit]["Debuff"..i]["SpellID"]
				
				if spellID == spell then
					index = i
				end				
			end

			GUIDList[unit]["Debuff"..index] = {}
		end
	end
end

function SyncUI_PlateFrame_OnEvent(self,event,...)
	if event == "PLAYER_LOGIN" then
		PlateFrame_OnLoad(self)
		--self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end
	
	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		PlateFrame_OnCombatLogEvent(self,event,...)
	end
end

--[[	NAMEPLATE_CHANGES_6.2.2
	nameplate.name = nameplate.NameContainer.NameText
	nameplate.border = nameplate.ArtContainer.Border
	nameplate.highlight = nameplate.ArtContainer.Highlight
	nameplate.level = nameplate.ArtContainer.LevelText
	nameplate.raidIcon = nameplate.ArtContainer.RaidTargetIcon
	nameplate.eliteIcon = nameplate.ArtContainer.EliteIcon
	nameplate.threat = nameplate.ArtContainer.AggroWarningTexture
	nameplate.bossIcon = nameplate.ArtContainer.HighLevelIcon

	nameplate.healthBar = nameplate.ArtContainer.HealthBar
	nameplate.healthBar.texture = nameplate.healthBar:GetRegions() --No parentKey, yet?
	--Available child element (texture): nameplate.healthBar.OverAbsorb

	nameplate.absorbBar = nameplate.ArtContainer.AbsorbBar
	nameplate.absorbBar.region1 = nameplate.absorbBar:GetRegions() --No parentKey, yet? Not sure what region1 is, probably texture like the others.
	nameplate.absorbBar.overlay = nameplate.absorbBar.Overlay

	nameplate.castBar = nameplate.ArtContainer.CastBar
	nameplate.castBar.texture = nameplate.castBar:GetRegions() --No parentKey, yet?
	nameplate.castBar.border = nameplate.ArtContainer.CastBarBorder
	nameplate.castBar.icon = nameplate.ArtContainer.CastBarSpellIcon
	nameplate.castBar.shield = nameplate.ArtContainer.CastBarFrameShield
	nameplate.castBar.name = nameplate.ArtContainer.CastBarText
	nameplate.castBar.shadow = nameplate.ArtContainer.CastBarTextBG
--]]
