
local floor = floor
local lineHeight = 12
local maxAVCriteria, maxQuestCriteria, maxBonusObjCriteria, maxScenarioCriteria = 6, 8, 6, 10
local maxQuests, maxBonusObj, maxAchievements = 15, 8, 10
local poiTable, POIByQuestID = {}, {}
local QuestHubTabs = {"QuestTab", "BonusObjTab", "AchievementTab", "ScenarioTab"}
-----------------------
-- General Functions --
-----------------------
local function DisableBlizzardModules()
	local modules = {
		--QUEST_TRACKER_MODULE,
		--AUTO_QUEST_POPUP_TRACKER_MODULE,
		--ACHIEVEMENT_TRACKER_MODULE,
		--BONUS_OBJECTIVE_TRACKER_MODULE,
	}
	
	for _, module in ipairs(modules) do
		module.updateReasonModule = nil
		module.updateReasonEvents = nil
	end
end

local function AdjustScrollBars(self)
	for _, tab in pairs(QuestHubTabs) do
		local scrollFrame = self[tab]["ScrollFrame"]
		scrollFrame.ScrollBar:SetPoint("TOPLEFT",scrollFrame,"TOPRIGHT",10,-43)
	end
end

local function SetBonusTabToScenario()
	local frame = SyncUI_QuestHub
	local quest, bonus = frame.QuestButton, frame.BonusButton
	local inInstance, instanceType = IsInInstance()
	local flags = select(4,  C_Scenario.GetInfo())
	local inChallengeMode = bit.band(flags, SCENARIO_FLAG_CHALLENGE_MODE) == SCENARIO_FLAG_CHALLENGE_MODE
	local inProvingGrounds = bit.band(flags, SCENARIO_FLAG_PROVING_GROUNDS) == SCENARIO_FLAG_PROVING_GROUNDS or select(8, GetInstanceInfo()) == 1148
	
	if C_Scenario.IsInScenario() or inInstance then
		if inChallengeMode then
			bonus.tab = frame.ScenarioTab
			bonus.title = "Challenge Mode"
		elseif inProvingGrounds then
			bonus.tab = frame.ScenarioTab
			bonus.title = "Proving Grounds"
		else
			bonus.tab = frame.ScenarioTab
			bonus.title = "Scenario"
		end
		
		bonus:Click()
	else
		bonus.tab = frame.BonusObjTab
		bonus.title = "Objectives"
		quest:Click()		
	end
end

local function UpdateEntrySize(entry,size,adjust,titleSize)
	if not titleSize then
		titleSize = lineHeight
	end
	
	if size and size > 0 then
		entry:SetHeight(titleSize + (lineHeight * size) - (adjust*2))
		entry:SetHitRectInsets(0,0,0,lineHeight * size - (adjust*2))
		entry.Content.size = size
	else
		size = 1
		
		entry:SetHeight(titleSize + (lineHeight * size))
		entry:SetHitRectInsets(0,0,0,0)
		entry.Content.size = size
	end
end

function SyncUI_QuestHub_OnLoad(self)
	DisableBlizzardModules()
	AdjustScrollBars(self)

	setglobal("BINDING_NAME_CLICK SyncUI_ObjectiveHub_QuestTab_QuickItem:LeftButton", "Use Active Quest Item")

	self:SetScale(SyncUI_UIScale)
	SyncUI_RegisterDragFrame(self,"QuestHub")
end

function SyncUI_QuestHub_TabButton_OnClick(self,button)
	local parent = self:GetParent()
	local buttons = {"QuestButton", "BonusButton", "AchievementButton"}
	
	
	for _, tab in pairs(QuestHubTabs) do
		parent[tab]:Hide()
	end
	
	for _, suffix in pairs(buttons) do
		local button = parent[suffix]
		button:SetChecked(false)
	end
	
	self:SetChecked(true)
	self.tab:Show()
end

function SyncUI_QuestHub_ModuleEntry_OnClick(self,button)
	if self.module == "quest" then
		local questID = select(1, GetQuestWatchInfo(self.index))
		local questLogID = select(3, GetQuestWatchInfo(self.index))
		local isComplete = select(6, GetQuestWatchInfo(self.index))
		local isAutoComplete = select(8, GetQuestWatchInfo(self.index))
		
		if IsModifiedClick("CHATLINK") and ChatEdit_GetActiveWindow() then
			local link = GetQuestLink(questLogID)
			
			if link then
				ChatEdit_InsertLink(link)
			end
		elseif IsShiftKeyDown() then
			local tracked = GetSuperTrackedQuestID()

			if questID == tracked then
				SetSuperTrackedQuestID(0)
			end
			
			RemoveQuestWatch(questLogID)
		elseif isAutoComplete and isComplete == 1 then
			ShowQuestComplete(questLogID)
		else
			QuestLogPopupDetailFrame_Show(questLogID)
		end
	end
	
	if self.module == "achievement" then
		if not AchievementFrame then
			AchievementFrame_LoadUI()
		end
		if IsModifiedClick("CHATLINK") and ChatEdit_GetActiveWindow() then
			local link = GetAchievementLink(self.index)
			
			if link then
				ChatEdit_InsertLink(link)
			end
		elseif IsShiftKeyDown() or IsModifiedClick("QUESTWATCHTOGGLE") then
			AchievementObjectiveTracker_UntrackAchievement(_, self.index)
		elseif not AchievementFrame:IsShown() then
			AchievementFrame_ToggleAchievementFrame()
			AchievementFrame_SelectAchievement(self.index)
		else
			if AchievementFrameAchievements.selection ~= self.index then
				AchievementFrame_SelectAchievement(self.index)
			else
				AchievementFrame_ToggleAchievementFrame()
			end
		end
	end
end

function SyncUI_QuestHub_ModuleEntry_OnEnter(self)
	if self.module == "bonus" then
		local questID = self.index
		local count = 0
		
		GameTooltip:SetOwner(SyncUI_QuestHub, "BOTTOMLEFT",0,-45)
		GameTooltip:SetText(REWARDS, 1, 0.831, 0.380)
		
		if not HaveQuestData(questID) then
			GameTooltip:AddLine(RETRIEVING_DATA,1,0,0)
		else
			
			GameTooltip:AddLine(BONUS_OBJECTIVE_TOOLTIP_DESCRIPTION, 1, 1, 1, 1)
			GameTooltip:AddLine(" ")
			
			do	-- xp
				local xp = GetQuestLogRewardXP(questID)
				
				if xp > 0 then
					GameTooltip:AddLine(string.format(BONUS_OBJECTIVE_EXPERIENCE_FORMAT, xp), 1, 1, 1)
					count = count + 1
				end
			end
			
			do	-- currency
				local numQuestCurrencies = GetNumQuestLogRewardCurrencies(questID)
				
				for i = 1, numQuestCurrencies do
					local name, texture, numItems = GetQuestLogRewardCurrencyInfo(i, questID)
					local text = string.format(BONUS_OBJECTIVE_REWARD_WITH_COUNT_FORMAT, texture, numItems, name)
					GameTooltip:AddLine(text,1,1,1)
					count = count + 1
				end
			end
			
			do	-- items
				local numQuestRewards = GetNumQuestLogRewards(questID)
				
				for i = 1, numQuestRewards do
					local name, texture, numItems, quality, isUsable = GetQuestLogRewardInfo(i, questID)
					local text
					
					if numItems > 1 then
						text = string.format(BONUS_OBJECTIVE_REWARD_WITH_COUNT_FORMAT, texture, numItems, name);
					elseif( texture and name ) then
						text = string.format(BONUS_OBJECTIVE_REWARD_FORMAT, texture, name);    
					end
					
					if text then
						local color = ITEM_QUALITY_COLORS[quality]
						GameTooltip:AddLine(text, color.r, color.g, color.b)
						count = count + 1
					end
				end
			end
			
			do	-- money
				local money = GetQuestLogRewardMoney(questID)
				if money > 0 then
					SetTooltipMoney(GameTooltip, money, nil)
					count = count + 1
				end
			end
		end
		
		if count > 0 then
			GameTooltip:Show()
		else
			GameTooltip_Hide()
		end
	end
	
	if self.module == "scenario" then
		local _,stage,numStages,flags,_,_,_,xp,money = C_Scenario.GetInfo()
		local name, description = C_Scenario.GetStepInfo()
		
		GameTooltip:SetOwner(SyncUI_QuestHub, "BOTTOMLEFT",0,-45)
		
		if name and bit.band(flags, SCENARIO_FLAG_SUPRESS_STAGE_TEXT) == SCENARIO_FLAG_SUPRESS_STAGE_TEXT then
			GameTooltip:SetText(name, 1, 0.914, 0.682, 1)
			GameTooltip:AddLine(description, 1, 1, 1, true)
			
			if (xp > 0 and UnitLevel("player") < MAX_PLAYER_LEVEL) or (money > 0) then
				GameTooltip:AddLine(" ")
			end
			
			if xp > 0 and UnitLevel("player") < MAX_PLAYER_LEVEL then
				GameTooltip:AddLine(string.format(BONUS_OBJECTIVE_EXPERIENCE_FORMAT, xp), 1, 1, 1)
			end
			
			if money > 0 then
				SetTooltipMoney(GameTooltip, money, nil)
			end
			
			GameTooltip:Show()
		elseif stage <= numStages then
			GameTooltip:SetText(string.format(SCENARIO_STAGE_STATUS, stage, numStages), 1, 0.914, 0.682, 1)
			GameTooltip:AddLine(name, 1, 0.831, 0.380, true)
			GameTooltip:AddLine(" ")
			GameTooltip:AddLine(description, 1, 1, 1, true)
			GameTooltip:Show()
		end
		
		return
	end
	
	if self.module == "quest" or self.module == "achievement" then
		GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
		GameTooltip:AddLine("|cFF00FF99Click|r to toggle details",1,1,1)
		GameTooltip:AddLine("|cFF00FF99Shift-Click|r to untrack "..self.module,1,1,1)
		GameTooltip:Show()
	end
	
	if self.isRepeatable then
		self.Text:SetVertexColor(0,0.6,1)
	else
		self.Text:SetVertexColor(1,0.8,0)
	end
	
	for i = 1, self.Content.size do
		local line = self.Content["Line"..i]
		
		if line then
			if self.isRed then
				line.Text:SetVertexColor(1,0,0)
			else
				line.Text:SetVertexColor(1,1,1)
			end
		end
	end
end

function SyncUI_QuestHub_ModuleEntry_OnLeave(self)
	GameTooltip_Hide()
	
	if self.module == "scenario" then
		return
	end
	
	if self.isRepeatable then
		self.Text:SetVertexColor(0,0.4,0.8)
	else
		self.Text:SetVertexColor(0.7,0.5,0)
	end
	
	for i = 1, self.Content.size do
		local line = self.Content["Line"..i]
		
		if line then
			if self.isRed then
				line.Text:SetVertexColor(0.7,0,0)
			else
				line.Text:SetVertexColor(0.65,0.65,0.65)
			end
		end
	end
end

---------------------
-- Quest Functions --
---------------------
local function GetQuestTypeString(type)
	local string
	
	if type then
		if type == LE_QUEST_FREQUENCY_DAILY then
			string = "daily"
		end
		if type == LE_QUEST_FREQUENCY_WEEKLY then
			string = "weekly"
		end
	end
	
	if string then
		string = " |cFF00FF99("..string..")|r"
	else
		string = ""
	end
	
	return string
end

local function UpdateQuickItemButton(i)
	local button = SyncUI_ObjectiveHub_QuestTab_QuickItem
	local questID = select(1, GetQuestWatchInfo(i)) or 0
	local questLogID = select(3, GetQuestWatchInfo(i)) or 0
	local isComplete = select(6, GetQuestWatchInfo(i))
	local link, item, charges, showItemWhenComplete = GetQuestLogSpecialItemInfo(questLogID)
	local isQuestComplete;
	
	if isComplete == 1 then
		isQuestComplete = true
	end	
					
	if item and (not isQuestComplete or showItemWhenComplete) then
		button:SetID(questLogID)
		button.charges = charges
		button.rangeTimer = -1
		button:Show()
		
		SetItemButtonTexture(button, item)
		SetItemButtonCount(button, charges)

		SyncUI_QuestHub.QuestTab.Counter:Hide()
	else
		button:SetID(0)
		button:Hide()
		
		SyncUI_QuestHub.QuestTab.Counter:Show()
	end
end

local function UpdatePOIs()
	local numPOIs = QuestMapUpdateAllQuests()

	if numPOIs > 0 and GetCVarBool("questPOI") then
		GetQuestPOIs(poiTable)
	end
	
	for index, questID in pairs(poiTable) do
		POIByQuestID[questID] = index
	end
end

local function UpdateQuestPOI(entry,isComplete)
	local questID = select(1, GetQuestWatchInfo(entry.index))
	local hasLocalPOI = select(15, GetQuestWatchInfo(entry.index))
	local tracked = GetSuperTrackedQuestID()
	local count = POIByQuestID[questID] or ""
	
	-- Set Checked
	if tracked == questID then
		entry.poiButton:SetChecked(true)
	else
		entry.poiButton:SetChecked(false)
	end
	
	-- Set Count
	if isComplete then
		entry.poiButton.Count:SetText("?")
	else
		entry.poiButton.Count:SetText(count)
	end
	
	-- Show / Hide
	if hasLocalPOI or isComplete then
		entry.poiButton:Show()
	else
		entry.poiButton:Hide()
	end
end

local function UpdateQuestItem(entry,isComplete)
	local questLogID = select(3, GetQuestWatchInfo(entry.index)) or 0
	local link, item, charges, showItemWhenComplete = GetQuestLogSpecialItemInfo(questLogID)
	
	if item and (not isComplete or showItemWhenComplete) then
		entry.Item:SetID(questLogID)
		entry.Item.charges = charges
		entry.Item.rangeTimer = -1
		
		SetItemButtonTexture(entry.Item, item)
		SetItemButtonCount(entry.Item, charges)

		entry.Item:Show()
	else
		entry.Item:Hide()
	end
end

local function UpdateQuestCriteria(entry,isComplete)
	local index = entry.index
	local questID, title, questLogID, numCriteria, reqMoney = GetQuestWatchInfo(index)

	if questID then
		local isRepeatable = select(7, GetQuestLogTitle(questLogID))
		local moneyString, r, g, b
				
		if isRepeatable == 1 then
			r,g,b = 0.7,0.5,0
			entry.isRepeatable = false
		else
			r,g,b = 0,0.4,0.8
			entry.isRepeatable = true
		end

		entry:SetText(title)
		entry.Text:SetVertexColor(r,g,b)

		-- Check for money requirement
		if reqMoney and reqMoney > 0 and GetMoney() < reqMoney then
			moneyString = GetCoinTextureString(GetMoney()).." / "..GetCoinTextureString(reqMoney)
			numCriteria = numCriteria + 1
		else
			reqMoney = false
		end
		
		-- Pre Reset
		for i = 1, maxQuestCriteria do
			local line = entry.Content["Line"..i]
			
			line:SetHeight(lineHeight)
			line:Hide()
			line.Mark:Hide()
			line.ProgressBar:Hide()
		end
		
		-- Update Complete & Auto Complete
		if isComplete then
			local isAutoComplete = select(8, GetQuestWatchInfo(entry.index))
			local line = entry.Content["Line1"]

			if isAutoComplete then
				local line2 = entry.Content["Line2"]
				line.Text:SetText("|cFF00FF99"..QUEST_WATCH_QUEST_COMPLETE.."|r")
				line2.Text:SetText(QUEST_WATCH_CLICK_TO_COMPLETE)
				line2:Show()
				numCriteria = 2
			else
				line.Text:SetText("|cFF00FF99"..QUEST_WATCH_QUEST_READY.."|r")
				numCriteria = 1
			end
			
			line.Mark:SetAtlas("Objective-Nub")
			line.Mark:Show()
			line:Show()
		elseif entry.isRed then
			local line = entry.Content["Line1"]
		
			line.Text:SetText("Failed")
			line.Text:SetVertexColor(0.7,0,0)
			line.Mark:SetAtlas("Objective-Nub")
			line.Mark:Show()
			line:Show()
		elseif numCriteria then
			for index = 1, numCriteria do
				local text, type, finished = GetQuestLogLeaderBoard(index,questLogID)
				local line = entry.Content["Line"..index]
				
				if line then
					if not text and reqMoney then
						text = moneyString
					elseif not text then
						text = select(2, GetQuestLogQuestText())
					end
					
					if finished then
						line.Mark:SetAtlas("Tracker-Check")
						
						if not line.complete then
							line.Glow.Anim:Play()
							line.Sheen.Anim:Play()
						end
						
						line.complete = true
					else
						line.Mark:SetAtlas("Objective-Nub")
						line.complete = false
					end
					
					line.Text:SetVertexColor(0.65,0.65,0.65)
					line.Text:SetText(text)
					line.Mark:Show()
					line:Show()
				end
			end
		end
		
		if numCriteria > maxQuestCriteria then
			numCriteria = maxQuestCriteria
		end
		
		UpdateEntrySize(entry,numCriteria,0)
	end
end

local function UpdateQuestEntries(self)
	for i = 1, maxQuests do
		local entry = self["Entry"..i]
		
		if entry then
			entry:Hide()
			entry.index = nil
		end
	end
	
	for i = 1, GetNumQuestWatches() do
		local entry = self["Entry"..i]
		
		if entry then
			local _,_,_,numCriteria,reqMoney,isComplete,startEvent = GetQuestWatchInfo(i)
			
			if isComplete == -1 then
				entry.isRed = true
			else	
				entry.isRed = false
			end
			
			if isComplete and isComplete < 0 then
				isComplete = false
			elseif numCriteria == 0 and GetMoney() >= reqMoney and not startEvent then
				isComplete = true
			end

			entry.index = i
			entry:Show()
			
			UpdateQuestPOI(entry,isComplete)
			UpdateQuestItem(entry,isComplete)
			UpdateQuestCriteria(entry,isComplete)
		end
	end

	SyncUI_QuestHub.QuestTab.Counter:SetText(GetNumQuestWatches().."/"..maxQuests)
end

function SyncUI_QuestHub_QuestContainer_OnLoad(self)
	-- Update Quest Item Hotkey
	self:RegisterEvent("PLAYER_LOGIN")
	self:RegisterEvent("PLAYER_MONEY")
	self:RegisterEvent("UPDATE_BINDINGS")

	-- Update Quests
	self:RegisterEvent("WORLD_MAP_UPDATE")
	self:RegisterEvent("QUEST_LOG_UPDATE")
	self:RegisterEvent("QUEST_POI_UPDATE")
	self:RegisterEvent("QUEST_WATCH_LIST_CHANGED")
	self:RegisterEvent("SUPER_TRACKED_QUEST_CHANGED")
	
	-- Hooks
	hooksecurefunc("AddQuestWatch", function()
		UpdateQuestEntries(self)
	end)
	hooksecurefunc("RemoveQuestWatch", function()
		UpdateQuestEntries(self)
	end)
	hooksecurefunc("AddAutoQuestPopUp", function()
		UpdateQuestEntries(self)
	end)
end

function SyncUI_QuestHub_QuestContainer_OnEvent(self,event,arg1,...)
	UpdatePOIs()
	
	if event == "UPDATE_BINDINGS" or event == "PLAYER_LOGIN" then
		local quickButton = SyncUI_ObjectiveHub_QuestTab_QuickItem
		local key1 = GetBindingKey("CLICK "..SyncUI_ObjectiveHub_QuestTab_QuickItem:GetName()..":LeftButton")
		local text = GetBindingText(key1, 1)
		
		if key1 then
			quickButton.Hotkey:SetText(text)
		else
			quickButton.Hotkey:SetText("")
		end
	elseif event == "WORLD_MAP_UPDATE" then
		local mapID = GetCurrentMapAreaID()
		if mapID ~= self.mapID then
			self.mapID = mapID
			UpdateQuestEntries(self)
		end
	else
		UpdateQuestEntries(self)
	end
	
	if event == "QUEST_POI_UPDATE" then
		QuestSuperTracking_OnPOIUpdate()
	end
	
	if event == "SUPER_TRACKED_QUEST_CHANGED" then
		for i = 1, GetNumQuestWatches() do
			local questID = GetQuestWatchInfo(i)
			
			if questID == arg1 then
				UpdateQuickItemButton(i)
				break
			end
		end
	end
end

function SyncUI_QuestHub_QuestPOIButton_OnClick(self,button)
	local frame = self:GetParent():GetParent()
	local index = self:GetParent().index
	local questID = select(1, GetQuestWatchInfo(index))
	
	SetSuperTrackedQuestID(questID)
	UpdateQuickItemButton(index)
	UpdateQuestEntries(self)
end

---------------------------
-- Achievement Functions --
---------------------------
local function UpdateAchievementCriteria(entry,achieveID)
	local _,title,_,completed,_,_,_,description = GetAchievementInfo(achieveID)
	local numCriteria = GetAchievementNumCriteria(achieveID)
	local numShown, adjust, adjustFactor = 0, 0, 0
	local forceProgress

	entry.isRed = not IsAchievementEligible(achieveID)
	
	if not entry.info then
		entry.info = {}
	else
		wipe(entry.info)
	end
	
	for _, line in pairs({entry.Content:GetChildren()}) do
		line:SetHeight(lineHeight)
		line:Hide()
		line.Mark:Hide()
		line.ProgressBar:Hide()
	end

	if numCriteria > 0 then
		for index = 1, numCriteria do
			local text,_,completed,refCur,refMax,_,flags,assetID,progress = GetAchievementCriteriaInfo(achieveID, index)
			local isCopy;
			
			if description and bit.band(flags, EVALUATION_TREE_FLAG_PROGRESS_BAR) == EVALUATION_TREE_FLAG_PROGRESS_BAR then
				if string.find(strlower(progress), "interface\\moneyframe") then
					text = progress.."\n"..description;
				else
					text = description
					--text = string.gsub(progress, " / ", "/").." "..description
				end
				
				forceProgress = true
			end
			
			for _, info in pairs(entry.info) do
				if info == text then
					isCopy = true
				end
			end

			if not isCopy then
				if text and not completed then
					tinsert(entry.info,text)
					numShown = numShown + 1
				end
			end
			
			if forceProgress then
				tinsert(entry.info,{refCur, refMax})
				numShown = numShown + 1
			end
		end
	elseif description then
		numShown = 1
		tinsert(entry.info,description)
	end
	
	for index, text in pairs(entry.info) do
		local line = entry.Content["Line"..index]

		if line then
			if index == maxAVCriteria and numShown > maxAVCriteria then
				line.Text:SetText("...")
			elseif type(text) == "table" and forceProgress then
				local value, maxValue = text[1], text[2]
				line.Text:SetText("")
				line.ProgressBar:Show()
				line.ProgressBar:SetMinMaxValues(0,maxValue)
				line.ProgressBar:SetValue(value)
				line.ProgressBar.Text:SetText(value.." / "..maxValue)
			else
				line.Text:SetText(text)
				line.Mark:Show()
			end
			
			do	-- adjust string height
				local fontWidth = line.Text:GetStringWidth()
				local fontHeight = select(2, line.Text:GetFont())
				local factor = math.ceil(fontWidth / 274)
				
				if factor > 2 then
					factor = factor - 1
				end
				
				if fontWidth > 274 then
					line:SetHeight(fontHeight * factor + 2)
					
					numShown = numShown + (factor - 1)
					
					adjust = adjust + 1
				elseif forceProgress and line.ProgressBar:IsShown() then
					line:SetHeight(20)
					
					adjustFactor = adjustFactor - 4
				else
					line:SetHeight(lineHeight)
				end
			end
			
			if entry.isRed then
				line.Text:SetVertexColor(0.7,0,0)
			else
				line.Text:SetVertexColor(0.65,0.65,0.65)
			end
			
			line:Show()
		end
	end	
	
	adjustFactor = adjustFactor + adjust
	
	UpdateEntrySize(entry,min(numShown, maxAVCriteria + adjust),adjustFactor)
end

local function UpdateAchievementEntries(self)
	local listedAVs = {GetTrackedAchievements()}

	for i = 1, maxAchievements do
		local entry = self["Entry"..i]
		
		if entry then
			entry:Hide()
			entry.index = nil
		end
	end
	
	for entryID = 1, GetNumTrackedAchievements() do
		local entry = self["Entry"..entryID]

		if entry then
			local achieveID = listedAVs[entryID]
			local title = select(2,  GetAchievementInfo(achieveID))
			
			UpdateAchievementCriteria(entry,achieveID)
			
			entry.index = achieveID
			entry:SetText(title)
			entry:Show()
		end
	end

	SyncUI_QuestHub.AchievementTab.Counter:SetText(#listedAVs.."/"..maxAchievements)
end

function SyncUI_QuestHub_AchievementContainer_OnLoad(self)
	self:RegisterEvent("TRACKED_ACHIEVEMENT_UPDATE")
	self:RegisterEvent("ACHIEVEMENT_EARNED")
	self:RegisterEvent("PLAYER_LOGIN")
	
	hooksecurefunc("AddTrackedAchievement", function(achieveID)
		UpdateAchievementEntries(self)
	end)
	hooksecurefunc("RemoveTrackedAchievement", function(achieveID)
		UpdateAchievementEntries(self)
	end)
	
	UpdateAchievementEntries(self)
end

function SyncUI_QuestHub_AchievementContainer_OnEvent(self,event,...)
	if event == "TRACKED_ACHIEVEMENT_UPDATE" then
		local achieveID = ...
		
		for _, entry in pairs({self:GetChildren()}) do
			if entry.index == achieveID then
				UpdateAchievementCriteria(entry,achieveID)
			end
		end
	end
	
	if event == "ACHIEVEMENT_EARNED" or event == "PLAYER_LOGIN" then
		UpdateAchievementEntries(self)
	end
end

----------------------
-- Bonus Objectives --
----------------------
local function GetExistingTable(self, questID)
	if not questID then
		return
	end
	
	for i = 1, maxQuests do
		local entry = self["Entry"..i]
		
		if entry and entry.index == questID then
			return true
		end
	end
	
	return
end

local function AddRewards(questID,xp,money)
	local rewardFrame = SyncUI_BonusObjRewardFrame
	
	rewardFrame.data = {}
	
	-- Add Exp Reward
	if not xp then
		xp = GetQuestLogRewardXP(questID)
	end
	if xp and xp > 0 and UnitLevel("player") < MAX_PLAYER_LEVEL then
		local t = {
			["Title"] = xp,
			["Texture"] = "Interface\\Icons\\XP_Icon",
			["Count"] = 0
		}
		table.insert(rewardFrame.data,t)
	end

	-- Add Currency Reward
	local numQuestCurrencies = GetNumQuestLogRewardCurrencies(questID)
	for i = 1, numQuestCurrencies do
		local name, texture, numItems = GetQuestLogRewardCurrencyInfo(i, questID)
		local t = {
			["Title"] = name,
			["Texture"] = texture,
			["Count"] = numItems,
		}
		table.insert(rewardFrame.data,t)
	end

	-- Add Item Reward
	local numQuestRewards = GetNumQuestLogRewards(questID)
	for i = 1, numQuestRewards do
		local name, texture, numItems = GetQuestLogRewardInfo(i, questID)
		local t = {
			["Title"] = name,
			["Texture"] = texture,
			["Count"] = numItems,
		}
		table.insert(rewardFrame.data,at)
	end

	-- Add Money Reward
	if not money then
		money = GetQuestLogRewardMoney(questID)
	end
	if money > 0 then
		local t = {
			["Title"] = GetMoneyString(money),
			["Texture"] = "Interface\\Icons\\inv_misc_coin_01",
			["Count"] = 0
		}
		table.insert(rewardFrame.data,t)
	end
	
	-- Spawn Reward Animation
	if #rewardFrame.data > 0 then
		rewardFrame.Spawn:Play()
	end
end

local function UpdateBonusQuestCriteria(entry,questID)
	local title = C_TaskQuest.GetQuestTitleByQuestID(questID)
	local isInArea, isOnMap, numCriteria = GetTaskInfo(questID)
	local size, adjustFactor = 0, 0
	
	entry:Show()
	entry:SetText(title)
	entry.index = questID
	
	if isInArea then
		entry.isRepeatable = true
		entry.Text:SetVertexColor(0,0.4,0.8)
	else
		entry.isRepeatable = false
		entry.Text:SetVertexColor(0.7,0.5,0)
	end
	
	for i = 1, maxBonusObjCriteria do
		local line = entry.Content["Line"..i]
		
		line:SetHeight(lineHeight)
		line:Hide()
		line.Mark:Hide()
		line.ProgressBar:Hide()
	end

	for i = 1, numCriteria do
		local line = entry.Content["Line"..i]
		local text, type, finished = GetQuestObjectiveInfo(questID, i, false)
		local forceProgress
		
		if line then
			line.Text:SetText(text)
			line.Mark:SetAtlas("Objective-Nub")
			line.Mark:Show()
			line:Show()
			
			if type == "progressbar" then
				local line = entry.Content["Line"..i+1]
				local perc = GetQuestProgressBarPercent(questID)
				local percString = string.format("%.0f",perc).."%"
				
				line.ProgressBar:Show()
				line.ProgressBar:SetMinMaxValues(0,100)
				line.ProgressBar:SetValue(perc)
				line.ProgressBar.Text:SetText(percString)
				line:SetHeight(20)
				line.Text:SetText("")
				line:Show()
				
				adjustFactor = adjustFactor - 4
				forceProgress = true
			end
		end
		
		if forceProgress then
			size = size + 2
		else
			size = size + 1
		end
	end
	
	UpdateEntrySize(entry,size,adjustFactor)
end

local function UpdateBonusQuestEntries(self)
	local listedBonusObj = 0

	for i = 1, maxBonusObj do
		local entry = self["Entry"..i]
		
		if entry then
			entry:Hide()
			entry.index = nil
		end
	end
	
	local index = 1
	
	for _, questID in ipairs(GetTasksTable()) do
		local isInArea, isOnMap = GetTaskInfo(questID)

		if isOnMap then
			local entry = self["Entry"..index]
			
			if entry then
				UpdateBonusQuestCriteria(entry,questID)
			end
			
			listedBonusObj = listedBonusObj + 1
			index = index + 1
		end
	end

	if listedBonusObj > maxBonusObj then
		listedBonusObj = maxBonusObj
	end
	
	SyncUI_QuestHub.BonusObjTab.Counter:SetText(listedBonusObj.."/"..maxBonusObj)
end

local function UpdateBonusScenarioCriteria(entry,step)
	local numCriteria = select(3, C_Scenario.GetStepInfo(step))
	local title = GetInstanceInfo()
	local entryID = entry:GetID()
	local headerHeight
	
	entry:Show()
	entry:SetText("Bonus Objectives")
	entry.index = C_Scenario.GetBonusStepRewardQuestID(step)
	
	for i = 1, maxBonusObjCriteria do
		local line = entry.Content["Line"..i]
		
		line:Hide()
		line.Mark:Hide()
		line.ProgressBar:Hide()
	end

	for i = 1, numCriteria do
		local line = entry.Content["Line"..i]
		local text,_,completed,value,maxValue,flags,assetID,valueString,criteriaID = C_Scenario.GetCriteriaInfoByStep(step,i)
		
		if line then
			line.Text:SetText(text)
			
			line.Mark:Show()
			line:Show()
			
			if completed then
				line.Mark:SetAtlas("Tracker-Check")
				
				if not line.complete then
					line.Glow.Anim:Play()
					line.Sheen.Anim:Play()
				end
				line.complete = true
			else
				line.Mark:SetAtlas("Objective-Nub")
				line.complete = false
			end
		end
	end

	if entryID > 0 then
		headerHeight = 0
		entry.Text:Hide()
		entry.Content:SetPoint("TOPLEFT",entry,10,0)
	end
	
	UpdateEntrySize(entry,numCriteria,0,headerHeight)
	
	if entryID > 0 then
		entry:SetHitRectInsets(0,0,0,0)
	end
end

local function UpdateBonusScenarioEntries(self)
	local steps = C_Scenario.GetBonusSteps()
	local numSteps = 0
	
	for i = 2, maxBonusObj do
		local entry = self["Entry"..i]
		
		if entry then
			entry:Hide()
			entry.index = nil
		end
	end
	
	for i, step in pairs(steps) do
		local entry = self["Entry"..i+2]
		
		if entry then
			UpdateBonusScenarioCriteria(entry,step)
		end
		
		if step then
			numSteps = numSteps + 1
		end
	end
	
	if numSteps > 0 then
		local entry = self["Entry2"]
		
		if entry then
			entry:Show()
			entry.Content:Hide()
			entry:SetText("Bonus Objectives")
			UpdateEntrySize(entry,1,0,0)
		end
	end
end

function SyncUI_QuestHub_BonusObjContainer_OnLoad(self)
	self:RegisterEvent("QUEST_ACCEPTED")
	self:RegisterEvent("QUEST_TURNED_IN")
	self:RegisterEvent("QUEST_LOG_UPDATE")
	
	self:RegisterEvent("ZONE_CHANGED")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")

	self:RegisterEvent("CRITERIA_EARNED")
	self:RegisterEvent("CRITERIA_COMPLETE")
end

function SyncUI_QuestHub_BonusObjContainer_OnEvent(self,event,...)
	if event == "QUEST_ACCEPTED" then
		local questLogID, questID = ...
		
		if IsQuestTask(questID) then
			TopBannerManager_Show(ObjectiveTrackerBonusBannerFrame, questID)
		end
	end

	if event == "QUEST_TURNED_IN" then
		local questID,xp,money = ...

		if IsQuestTask(questID) then
			AddRewards(questID,xp,money)
		end
	end

	if event == "QUEST_LOG_UPDATE" then
		UpdateBonusQuestEntries(self)
		
		if C_Scenario.IsInScenario() then
			local content = SyncUI_QuestHub.ScenarioTab.ScrollFrame.Content
			UpdateBonusScenarioEntries(content)
		end
	end

	if event == "ZONE_CHANGED" or event == "ZONE_CHANGED_NEW_AREA" then
		if not C_Scenario.IsInScenario() then
			local mapID = GetCurrentMapAreaID()
			
			if mapID ~= self.mapID then
				self.mapID = mapID
				UpdateBonusQuestEntries(self)
			end
		end
	end
	
	if event == "CRITERIA_EARNED" or event == "CRITERIA_COMPLETE" or event == "ZONE_CHANGED_NEW_AREA" then
		if C_Scenario.IsInScenario() then
			UpdateBonusScenarioEntries(self)
			
			if event == "CRITERIA_COMPLETE" then
				local id = ...
				
				if id > 0 then
					local steps = C_Scenario.GetBonusSteps()
					
					for i, step in pairs(steps) do
						local numCriteria = select(3, C_Scenario.GetStepInfo(step))

						for i = 1, numCriteria do
							local criteriaID = select(9, C_Scenario.GetCriteriaInfoByStep(step,i))
							
							if id == criteriaID then
								local questID = C_Scenario.GetBonusStepRewardQuestID(step)
								if questID ~= 0 then
									AddRewards(questID)
								end
							end
						end
					end
				end
			end
		end
	end

	if event == "ZONE_CHANGED_NEW_AREA" and IsInInstance() then
		SetBonusTabToScenario()
	end
end

function SyncUI_BonusObjRewardFrame_AnimateRewards(self)
	local numRewards = #self.data
	
	for i = 1, numRewards do
		local reward = self.Rewards[i]
		local data = self.data[i]
	
		if not reward then
			reward = CreateFrame("Frame",nil,self,"SyncUI_BonusObjRewardTemplate")
			reward:SetPoint("TOPLEFT",self.Rewards[i-1],"BOTTOMLEFT",0,0)
		end
		
		if data.Count > 1 then
			reward.Count:Show()
			reward.Count:SetText(data.Count)
		else
			reward.Count:Hide()
		end
		
		if reward.Spawn:IsPlaying() then
			return
		end
		
		reward.Title:SetText(data.Title)
		reward.Icon:SetTexture(data.Texture)
		reward.Spawn:Play()
	end
end

--------------------------------------
-- Challenge Mode & Proving Grounds --
--------------------------------------
local function StopTimer(self,timerID)
	local entry = self.Entry1
	local timerBar = entry.Content.TimerBar
	local statusBar = timerBar.StatusBar
	local updateFrame = SyncUI_QuestHub_ScenarioTimerFrame

	if self.timerID == timerID then
		self.timerID = nil
		self:RegisterEvent("SCENARIO_UPDATE")
		self:RegisterEvent("SCENARIO_CRITERIA_UPDATE")
	end
	
	timerBar.module = nil
	timerBar.info = nil
	timerBar.wave = nil
	timerBar.score = nil
	timerBar.baseTime = nil
	timerBar.timeSinceBase = nil
	timerBar.MedalFrame:Hide()

	statusBar:SetMinMaxValues(0,1)
	statusBar:SetValue(1)
	statusBar.Text:SetText("")
	
	updateFrame:SetScript("OnUpdate", nil)
end

local function CheckTimers(self,...)
	local entry = self.Entry1
	local timerBar = entry.Content.TimerBar
	local updateFrame = SyncUI_QuestHub_ScenarioTimerFrame
	
	for i = 1, select("#", ...) do
		local timerID = select(i, ...)
		local _, elapsedTime, type = GetWorldElapsedTime(timerID)
		
		
		if type == LE_WORLD_ELAPSED_TIMER_TYPE_CHALLENGE_MODE then
			local mapID = select(8, GetInstanceInfo())
			
			if mapID then
				local bronzeTime, silverTime, goldTime = GetChallengeModeMapTimes(mapID)
				local info = {timerID, bronzeTime, silverTime, goldTime}

				timerBar.MedalFrame:Show()
				timerBar.module = "ChallengeMode"
				timerBar.info = info
				timerBar.baseTime = elapsedTime
				timerBar.timeSinceBase = 0
				
				self.timerID = timerID
				
				updateFrame:SetScript("OnUpdate", function(_,elapsed)
					SyncUI_QuestHub_ScenarioTimerFrame_OnUpdate(timerBar,elapsed)
				end)
				
				return
			end
		elseif type == LE_WORLD_ELAPSED_TIMER_TYPE_PROVING_GROUND then
			local diffID, currWave, maxWave, duration = C_Scenario.GetProvingGroundsInfo()
			
			if duration > 0 then
				local info = {timerID, duration, diffID, currWave, maxWave}

				timerBar.MedalFrame:Show()
				timerBar.module = "ProvingGrounds"
				timerBar.info = info
				timerBar.baseTime = elapsedTime
				timerBar.timeSinceBase = 0
			
				self.timerID = timerID
				self:UnregisterEvent("SCENARIO_UPDATE")
				self:UnregisterEvent("SCENARIO_CRITERIA_UPDATE")
				
				entry:SetText(C_Scenario.GetInfo())
				
				updateFrame:SetScript("OnUpdate", function(_,elapsed)
					SyncUI_QuestHub_ScenarioTimerFrame_OnUpdate(timerBar,elapsed)
				end)
		
				return
			end
		end
	end

	StopTimer(self)
end

--------------------------
-- Dungeons & Scenarios --
--------------------------
local function AddScenarioRewards(xp,money)
	local rewardFrame = SyncUI_BonusObjRewardFrame
	
	rewardFrame.data = {}
	
	-- Add Exp Reward
	if xp and xp > 0 and UnitLevel("player") < MAX_PLAYER_LEVEL then
		local t = {
			["Title"] = xp,
			["Texture"] = "Interface\\Icons\\XP_Icon",
			["Count"] = 0
		}
		table.insert(rewardFrame.data,t)
	end

	-- Add Money Reward
	if money > 0 then
		local t = {
			["Title"] = GetMoneyString(money),
			["Texture"] = "Interface\\Icons\\inv_misc_coin_01",
			["Count"] = 0
		}
		table.insert(rewardFrame.data,t)
	end
	
	-- Spawn Reward Animation
	if #rewardFrame.data > 0 then
		rewardFrame.Spawn:Play()
	end
end

local function UpdateScenarioCriteria(self)
	local entry = self["Entry1"]
	local timerBar = entry.Content.TimerBar
	local line1 = entry.Content["Line1"]
	local title,stage,numStages,flags,_,_,_,xp,money = C_Scenario.GetInfo()
	
	if numStages > 0 then
		local stageName, stageDescription, numCriteria = C_Scenario.GetStepInfo()
		local inChallengeMode = bit.band(flags, SCENARIO_FLAG_CHALLENGE_MODE) == SCENARIO_FLAG_CHALLENGE_MODE
		local inProvingGrounds = bit.band(flags, SCENARIO_FLAG_PROVING_GROUNDS) == SCENARIO_FLAG_PROVING_GROUNDS or select(8, GetInstanceInfo()) == 1148
		local inDungeon = bit.band(flags, SCENARIO_FLAG_USE_DUNGEON_DISPLAY) == SCENARIO_FLAG_USE_DUNGEON_DISPLAY
		local fullCompleted = stage > numStages

		for i = 1, maxScenarioCriteria do
			local line = entry.Content["Line"..i]

			line:SetHeight(lineHeight)
			line:Hide()
			line.Mark:Hide()
			line.ProgressBar:Hide()
		end

		if fullCompleted then
			if inDungeon then
				title = DUNGEON_COMPLETED
			else
				title = SCENARIO_COMPLETED_GENERIC
			end		
		else
			if not stageName then
				stageName = ""
			end
		
			if inDungeon or inChallengeMode then
				--do nothing
			elseif bit.band(flags, SCENARIO_FLAG_SUPRESS_STAGE_TEXT) == SCENARIO_FLAG_SUPRESS_STAGE_TEXT then
				title = stageName
			else
				if stage == numStages then
					title = SCENARIO_STAGE_FINAL
				else
					title = string.format(SCENARIO_STAGE, stage)
				end
				
				title = title..": "..stageName
			end

			for i = 1, numCriteria do
				local text, type, isCompleted, quantity, totalQuantity, flags, assetID, quantityString, criteriaID, duration, elapsed = C_Scenario.GetCriteriaInfo(i)
				local line = entry.Content["Line"..i]
				
				if text then
					text = string.format("%d/%d %s", quantity, totalQuantity, text)
				end
				
				if line then
					if isCompleted then
						line.Mark:SetAtlas("Tracker-Check")
						
						if not line.complete then
							line.Glow.Anim:Play()
							line.Sheen.Anim:Play()
						end
						line.complete = true
					else
						line.Mark:SetAtlas("Objective-Nub")
						line.complete = false
					end

					line.Text:SetVertexColor(0.65,0.65,0.65)
					line.Text:SetText(text)
					line.Mark:Show()
					line:Show()
				end
			end
		end

		if inChallengeMode or inProvingGrounds then
			timerBar:Show()
			line1:ClearAllPoints()
			line1:SetPoint("TOPLEFT",timerBar,"BOTTOMLEFT")
		else
			timerBar:Hide()
			line1:ClearAllPoints()
			line1:SetPoint("TOPLEFT")
		end
		
		entry:SetText(title)
		entry:Show()
		
		UpdateEntrySize(entry,numCriteria,0,20)
	else
		entry:Hide()
		entry.index = nil	
	end
end

function SyncUI_QuestHub_ScenarioContainer_OnLoad(self)
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("SCENARIO_UPDATE")
	self:RegisterEvent("SCENARIO_CRITERIA_UPDATE")
	self:RegisterEvent("SCENARIO_COMPLETED")
	
	self:RegisterEvent("PROVING_GROUNDS_SCORE_UPDATE")
	self:RegisterEvent("WORLD_STATE_TIMER_START")
	self:RegisterEvent("WORLD_STATE_TIMER_STOP")
end

function SyncUI_QuestHub_ScenarioContainer_OnEvent(self,event,...)
	if event == "SCENARIO_COMPLETED" then
		local xp, money = ...
		if( ( xp > 0 and UnitLevel("player") < MAX_PLAYER_LEVEL ) or money > 0 ) then
			AddScenarioRewards(xp,money)
		end
	end
	
	if event == "SCENARIO_UPDATE" or event == "SCENARIO_CRITERIA_UPDATE" or event == "PLAYER_ENTERING_WORLD" then
		UpdateScenarioCriteria(self)
		
		if event == "PLAYER_ENTERING_WORLD" then
			SetBonusTabToScenario(event)
			CheckTimers(self,GetWorldElapsedTimers())
		end
	end

	if event == "PROVING_GROUNDS_SCORE_UPDATE" then
		local score = ...
		local entry = self["Entry1"]
		local timerBar = entry.Content.TimerBar
		
		timerBar.score = score
	end
	
	if event == "WORLD_STATE_TIMER_START" then
		CheckTimers(self,...)
	end
	
	if event == "WORLD_STATE_TIMER_STOP" then
		StopTimer(self,...)
	end
end

function SyncUI_QuestHub_ScenarioTimerFrame_OnUpdate(self,elapsed)
	self.timeSinceBase = self.timeSinceBase + elapsed
	
	local content = self:GetParent()
	local statusBar = self.StatusBar
	local medal = self.MedalFrame
	local timerID, arg1, arg2, arg3, arg4 = unpack(self.info)
	local elapsedTime = floor(self.baseTime + self.timeSinceBase)
	
	if self.module == "ChallengeMode" then
		local bronzeTime, silverTime, goldTime = arg1, arg2, arg3
		local medalTime, prevMedalTime, timeLeft;
		
		if elapsedTime <= goldTime then
			medalTime = goldTime
			medal:SetNormalTexture("Interface\\Challenges\\Challenges-Gold")
		elseif elapsedTime <= silverTime then
			medalTime, prevMedalTime = silverTime, goldTime
			medal:SetNormalTexture("Interface\\Challenges\\Challenges-Silver")
		else
			medalTime, prevMedalTime = bronzeTime, silverTime
			medal:SetNormalTexture("Interface\\Challenges\\Challenges-Bronze")
		end
		
		if medalTime then
			timeLeft = medalTime - elapsedTime
		end
		
		if prevMedalTime then
			medalTime = medalTime - prevMedalTime
		end
		
		statusBar:SetMinMaxValues(0, medalTime)
		statusBar.Text:SetText(GetTimeStringFromSeconds(timeLeft))
		SmoothBar(statusBar,timeLeft)	
	end
	
	if self.module == "ProvingGrounds" then
		local duration, diffID, currWave, maxWave = arg1, arg2, arg3, arg4
		local timeLeft = duration - elapsedTime

		if diffID == 1 then
			medal:SetNormalTexture("Interface\\Challenges\\Challenges-Bronze")
		elseif diffID == 2 then
			medal:SetNormalTexture("Interface\\Challenges\\Challenges-Silver")
		elseif diffID == 3 then
			medal:SetNormalTexture("Interface\\Challenges\\Challenges-Gold")
		elseif diffID == 4 then
			medal:SetNormalTexture("Interface\\Challenges\\Challenges-plat")
		end

		if self.wave ~= currWave then
			local line = content["Line1"]
			local waveString
			
			if not currWave or currWave <= 0 then
				waveString = ""
			elseif maxWave > 0 then
				waveString = tostring("Wave: "..currWave.."/"..maxWave)
			else
				waveString = tostring("Wave: "..currWave)
			end
			
			line.Text:SetText(waveString)
			line.Mark:Show()
			line:Show()
			
			self.wave = currWave
		end
		
		if self.score then
			local line = content["Line2"]
			
			line.Text:SetText("Score: "..self.score)
			line.Mark:Show()
			line:Show()
		end

		statusBar:SetMinMaxValues(0, duration)
		statusBar.Text:SetText(GetTimeStringFromSeconds(timeLeft))
		SmoothBar(statusBar,timeLeft)
	end
end

--------------------------
-- Auto Quest Functions --
--------------------------
local function SetStyle(self)
	local font = SyncUI_GameFontShadow_Medium
	local tracker = ObjectiveTrackerFrame
	local anim = ScenarioProvingGroundsBlockAnim
	local reward = ObjectiveTrackerScenarioRewardsFrame

	anim:SetParent(tracker)
	reward:SetParent(tracker)
	tracker:SetParent(self)
	tracker:ClearAllPoints()
	tracker:SetPoint("TOPRIGHT",0,0)
	tracker:SetHeight(self:GetHeight())
	tracker:SetMovable(true)
	tracker:SetUserPlaced(true)

	if SyncUI_Data.Options.Quest.DisableTracker then
		SyncUI_DisableFrame(tracker,true)
	end
	
	-- hook update
	hooksecurefunc("ObjectiveTracker_Update", function()
		if tracker.MODULES and #tracker.MODULES > 0 then
			for i = 1, #tracker.MODULES do
				local module = tracker.MODULES[i]

				module.Header.Text:SetFontObject(font)

				for _, block in pairs(module.usedBlocks) do
					if block.HeaderText then
						block.HeaderText:SetFontObject(font)
					end
				
					for _, line in pairs(block.lines) do
						line.Text:SetFontObject(font)
					end
				end
			end
		end
	end)
end

local function QuestSelect(event)
	if GetNumGossipOptions() > 0 then return end
	
	if event == "GOSSIP_SHOW" then
		if SyncUI_Data.Options.Quest.Accept then
			for i = 1, GetNumGossipAvailableQuests() do
				local isLow = select(i*6-3, GetGossipAvailableQuests())
				local type = select(i*6-2, GetGossipAvailableQuests())
				local isRepeatable = select(i*6-1, GetGossipAvailableQuests())

				if (not isRepeatable and not isLow) and (type == 1 or (type ~= 1 and SyncUI_Data.Options.Quest.Daily)) then
					SelectGossipAvailableQuest(i)
				end
			end
		end
		if SyncUI_Data.Options.Quest.TurnIn then
			for i = 1, GetNumGossipActiveQuests() do
				local isComplete = select(i*5-1, GetGossipActiveQuests())

				if isComplete then
					SelectGossipActiveQuest(i)
				end
			end
		end
	end
	
	if event == "QUEST_GREETING" then
		if SyncUI_Data.Options.Quest.Accept then
			for i = 1, GetNumAvailableQuests() do
				local isLow = select(i*6-3, GetGossipAvailableQuests())
				local type = select(i*6-2, GetGossipAvailableQuests())
				local isRepeatable = select(i*6-1, GetGossipAvailableQuests())

				if (not isRepeatable and not isLow) and (type == 1 or (type ~= 1 and SyncUI_Data.Options.Quest.Daily)) then
					SelectAvailableQuest(i)
				end
			end
		end
		if SyncUI_Data.Options.Quest.TurnIn then
			for i = 1, GetNumActiveQuests() do
				local isComplete = select(i*5-1, GetGossipActiveQuests())
				
				if isComplete then
					SelectActiveQuest(i)
				end
			end
		end
	end
end

local function QuestAccept()
	if not SyncUI_Data.Options.Quest.Accept then return end
	--[[
	if GetRewardXP() == 0 and not self.db.char.acceptQuestsWorthNoExperience then
		return
	end
	--]]

	--[[
	if not QuestGetAutoAccept() then
		local questID = GetQuestID()
		local isDaily = select(3, GetNumAvailableQuests(questID))

		print(questID)
		
		if isDaily and not SyncUI_Data.Options.Quest.Daily then
			return		
		else
			--AcceptQuest()
		end
	end
	--]]
	if not QuestGetAutoAccept() then
		local isRepeatable;
		
		if QuestIsDaily() or QuestIsWeekly() then
			isRepeatable = true
		end

		if not isRepeatable or (isRepeatable and SyncUI_Data.Options.Quest.Daily) then
			AcceptQuest()	
		end		
	end
end

local function QuestShare(questID)
	if not SyncUI_Data.Options.Quest.Share then return end
	
	if GetNumGroupMembers() > 0 then
		SelectQuestLogEntry(questID)
		if GetQuestLogPushable() then
			QuestLogPushQuest()
		end
	end
end

local function QuestTurnIn()
	if not SyncUI_Data.Options.Quest.TurnIn then return end
	
	if IsQuestCompletable() then
		CompleteQuest()
	end
end

local function QuestReward()
	if not SyncUI_Data.Options.Quest.TurnIn then return end
	
	if GetNumQuestChoices() <= 1 then
		QuestFrameCompleteQuestButton:Click()
	end
--[[
	local numChoices = GetNumQuestChoices()
	if numChoices == 0 then
		GetQuestReward(nil)
	elseif numChoices == 1 then
		GetQuestReward(1)	
	end
	
	return
	
	--]]
end

function SyncUI_ObjectiveTracker_OnLoad(self)
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("GOSSIP_SHOW")
	self:RegisterEvent("QUEST_GREETING")
	self:RegisterEvent("QUEST_DETAIL")
	self:RegisterEvent("QUEST_ACCEPTED")
	self:RegisterEvent("QUEST_PROGRESS")
	self:RegisterEvent("QUEST_COMPLETE")
end

function SyncUI_ObjectiveTracker_OnEvent(self,event,...)
	if event == "PLAYER_ENTERING_WORLD" then
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
		SetStyle(self)
	end
	if event == "GOSSIP_SHOW" or event == "QUEST_GREETING" then
		QuestSelect(event)
	end
	if event == "QUEST_DETAIL" then
		QuestAccept()
	end
	if event == "QUEST_ACCEPTED" then
		QuestShare(...)
	end
	if event == "QUEST_PROGRESS" then
		QuestTurnIn()
	end
	if event == "QUEST_COMPLETE" then
		QuestReward()
	end
end
