
local fontSmall = SyncUI_GameFontShadow_Small
local fontMedium = SyncUI_GameFontShadow_Medium
local fontLarge = SyncUI_GameFontShadow_Large
local backdrop = {
	edgeFile = "Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\Tooltip\\Tooltip_Edge",
	edgeSize = 16
}
local fakedrop = {
	bgFile = "Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\Tooltip\\Tooltip_Background",
	edgeFile = "Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\Tooltip\\Tooltip_Edge",
	edgeSize = 16, tileSize = 16, insets = {left = 4, right = 4, top = 4, bottom = 4}
}
local tooltips = {
	GameTooltip,
	FriendsTooltip,
	ItemRefTooltip,
	ItemRefShoppingTooltip1,
	ItemRefShoppingTooltip2,
	ItemRefShoppingTooltip3,
	ShoppingTooltip1,
	ShoppingTooltip2,
	ShoppingTooltip3,
	WorldMapTooltip,
	WorldMapCompareTooltip1,
	WorldMapCompareTooltip2,
	WorldMapCompareTooltip3,
	
	-- Garrison Specific Tooltips
	GarrisonBonusAreaTooltip,
	GarrisonFollowerTooltip,
	GarrisonFollowerAbilityTooltip,
	GarrisonMissionMechanicTooltip,
	GarrisonMissionMechanicFollowerCounterTooltip,
	GarrisonShipyardMapMissionTooltip,
	GarrisonShipyardFollowerTooltip,
	FloatingGarrisonShipyardFollowerTooltip,
}


-- Tooltip API
local function SetUnitIcon(self,unitID)
	local class = select(2, UnitClass(unitID))

	self.tex:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes")
	self.tex:SetTexCoord(unpack(CLASS_ICON_TCOORDS[class]))
	self.icon:Show()
end

local function SetIcon(self,spellID,isItem)
	local icon;
	
	if isItem then
		icon = select(10, GetItemInfo(spellID))
	else
		icon = select(3, GetSpellInfo(spellID))
	end

	if icon then
		self.icon:Show()
		self.tex:SetTexture(icon)
		self.tex:SetTexCoord(0.05,0.95,0.05,0.95)
	end
end

local function OverwriteBackdrop(self)
	if not self.bgFile then
		self.bgFile = self:CreateTexture(nil,"BACKGROUND")
		self.bgFile:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\Tooltip\\Tooltip_Background")
		self.bgFile:SetPoint("TOPLEFT",4,-4)
		self.bgFile:SetPoint("BOTTOMRIGHT",-4,4)
		
		self:SetBackdrop(backdrop)
		self.SetBackdrop = function() end
		self:SetBackdropColor(0,0,0,0)
		self.SetBackdropColor = function() end
		
		-- use a fake reference to inherit addon based tooltips
		self.GetBackdrop = function() return fakedrop end
		self.GetBackdropColor = function() return 1,1,1,1 end
	end

	if self.BorderTop then
		self.BorderTopLeft:Hide()
		self.BorderTopRight:Hide()
		self.BorderBottomLeft:Hide()
		self.BorderBottomRight:Hide()
		self.BorderTop:Hide()
		self.BorderLeft:Hide()
		self.BorderRight:Hide()
		self.BorderBottom:Hide()
		self.Background:Hide()
	end
	
	-- Specific Hooks
	if self == FriendsTooltip then
		local fonts = {"Header","OtherToons","NoteText","BroadcastText","LastOnline","ToonMany","Toon1Name","Toon1Info","Toon2Name","Toon2Info","Toon3Name","Toon3Info","Toon4Name","Toon4Info","Toon5Name","Toon5Info"}
		
		for _, font in pairs(fonts) do
			_G[self:GetName()..font]:SetFontObject(fontMedium)
		end
	end

	if self == GameTooltip then
		for line, justify in pairs(self.justify) do
			local width = self:GetWidth()-20
			
			line:SetWidth(width)
			line:SetJustifyH(justify)
		end	
	end
end

local function CreateDivider(self,line)
	self.divider[line] = CreateFrame("Frame",nil,self)
	
	local divider = self.divider[line]
	divider:SetHeight(7)
	divider.tex = divider:CreateTexture(nil,"BACKGROUND")
	divider.tex:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\Tooltip\\Tooltip_Divider")
	divider.tex:SetAllPoints(true)

	return divider
end

local function AddDivider(self)
	self:AddLine(" ")
	
	local line = self:NumLines()
	local relativeTo = _G[self:GetName().."TextLeft"..line]
	local divider = self.divider[line] or self:CreateDivider(line)

	divider:ClearAllPoints()
	divider:SetPoint("RIGHT",-12,0)
	divider:SetPoint("TOPLEFT",relativeTo,0,-1)
	divider:Show()
end

local function SetPrevLineJustify(self,justify)
	local index = self:NumLines()
	local line = _G[self:GetName().."TextLeft"..index]

	self.justify[line] = justify
end


function SyncUI_Tooltip_OnLoad()
	for _, tooltip in pairs(tooltips) do
		tooltip.justify = {}
		tooltip.divider = {}
		
		tooltip.SetUnitIcon = SetUnitIcon
		tooltip.SetIcon = SetIcon
		tooltip.OverwriteBackdrop = OverwriteBackdrop
		tooltip.CreateDivider = CreateDivider
		tooltip.AddDivider = AddDivider
		tooltip.SetPrevLineJustify = SetPrevLineJustify
		
		tooltip:HookScript("OnShow", function()
			tooltip:OverwriteBackdrop()
		end)
	end
	
	do	-- Modify default GameTooltip
		GameTooltip.icon = CreateFrame("Frame",nil,GameTooltip)
		GameTooltip.icon:SetSize(40,40)
		GameTooltip.icon:SetPoint("TOPRIGHT",GameTooltip,"TOPLEFT",4,0)
		GameTooltip.icon:SetBackdrop(backdrop)
		GameTooltip.icon:SetBackdropColor(1,1,1,0)
		GameTooltip.icon:Hide()
		
		GameTooltip.tex = GameTooltip.icon:CreateTexture(nil,"BACKGROUND")
		GameTooltip.tex:SetTexCoord(0.05,0.95,0.05,0.95)
		GameTooltip.tex:SetSize(31,31)
		GameTooltip.tex:SetPoint("CENTER")
		
		GameTooltipStatusBar:ClearAllPoints()
		GameTooltipStatusBar:SetPoint("TOPLEFT",15,12)
		GameTooltipStatusBar:SetPoint("TOPRIGHT",-15,12)
		GameTooltipStatusBar:SetHeight(10)
		GameTooltipStatusBar:SetStatusBarTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\Elements\\Statusbar\\Statusbar_Fill2")

		GameTooltipStatusBar.leftBorder = GameTooltipStatusBar:CreateTexture(nil,"OVERLAY")
		GameTooltipStatusBar.leftBorder:SetSize(20,40)
		GameTooltipStatusBar.leftBorder:SetPoint("LEFT",GameTooltipStatusBar,-5,-1)
		GameTooltipStatusBar.leftBorder:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\Tooltip\\Tooltip_StatusBar_Edge")
		GameTooltipStatusBar.leftBorder:SetTexCoord(0,0.5,0,1)
		
		GameTooltipStatusBar.rightBorder = GameTooltipStatusBar:CreateTexture(nil,"OVERLAY")
		GameTooltipStatusBar.rightBorder:SetSize(20,40)
		GameTooltipStatusBar.rightBorder:SetPoint("RIGHT",GameTooltipStatusBar,6,-1)
		GameTooltipStatusBar.rightBorder:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\Tooltip\\Tooltip_StatusBar_Edge")
		GameTooltipStatusBar.rightBorder:SetTexCoord(0.5,1,0,1)
		
		GameTooltipStatusBar.border = GameTooltipStatusBar:CreateTexture(nil,"OVERLAY")
		GameTooltipStatusBar.border:SetPoint("TOPLEFT",GameTooltipStatusBar.leftBorder,20,0)
		GameTooltipStatusBar.border:SetPoint("BOTTOMRIGHT",GameTooltipStatusBar.rightBorder,-20,0)
		GameTooltipStatusBar.border:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\Tooltip\\Tooltip_StatusBar_Edge")
		GameTooltipStatusBar.border:SetTexCoord(0.4,0.6,0,1)
		
		GameTooltipStatusBar.bg = GameTooltipStatusBar:CreateTexture(nil,"BACKGROUND")
		GameTooltipStatusBar.bg:SetAllPoints(GameTooltipStatusBar)
		GameTooltipStatusBar.bg:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\Elements\\Statusbar\\Statusbar_Background")

		GameTooltip.arrow = GameTooltip:CreateTexture(nil,"BACKGROUND")
		GameTooltip.arrow:SetDrawLayer("BACKGROUND", 2)
		GameTooltip.arrow:SetSize(48,24)
		GameTooltip.arrow:SetPoint("TOP",GameTooltip,"BOTTOM",0,6)
		GameTooltip.arrow:SetTexture("Interface\\AddOns\\SyncUI\\Media\\Textures\\UI-Mods\\Tooltip\\Tooltip_Arrow")
		GameTooltip.arrow:Hide()
	end
	
	do	-- Change Font
		GameTooltipHeaderText:SetFontObject(fontMedium)
		GameTooltipText:SetFontObject(fontMedium)
		Tooltip_Small:SetFontObject(fontSmall)
		SmallTextTooltipText:SetFontObject(fontMedium)
	end

	do	-- Tooltip Hooks
		GameTooltip:HookScript("OnTooltipSetSpell", function(self)
			local spellID = select(3, self:GetSpell())
			
			if spellID then
				local owner = self:GetOwner()
				if not owner.isTalent then
					self:SetIcon(spellID)
				end
			end
		end)
		GameTooltip:HookScript("OnTooltipSetItem", function(self)
			local name, link = self:GetItem()
			
			if name and link then
				self:SetIcon(link,true)
			end
		end)
		GameTooltip:HookScript("OnTooltipSetUnit", function(self,unit)
			local name, unitID = self:GetUnit()
			
			if not unitID then
				unitID = GetMouseFocus() and GetMouseFocus():GetAttribute("unit")
			end
			
			if not unitID and UnitExists("mouseover") then
				unitID = "mouseover"
			end
			
			if not unitID then
				return
			end
			
			if UnitIsUnit(unitID,"mouseover") then
				unit = "mouseover"
			end
			
			if unitID and UnitIsPlayer(unitID) then
				local guid = UnitGUID(unitID)
				local class = select(2, GetPlayerInfoByGUID(guid))
				
				if class then
					local color = SYNCUI_CLASS_COLORS[class]
					local r,g,b = color.r, color.g, color.b
					local left, right, top, bottom = unpack(CLASS_ICON_TCOORDS[class])
					local adjust = 0.02
					
					self.tex:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes")
					self.tex:SetTexCoord(left+adjust,right-adjust,top+adjust,bottom-adjust)
					self.icon:Show()
					
					do	-- Color Name 
						local name, realm = UnitName(unitID)
						local titledName = UnitPVPName(unitID)
						local string = SyncUI_GetColorizedText(titledName or name,r,g,b)
						
						GameTooltipTextLeft1:SetText(string)
					end
					do	-- Color Guild
						local guildName = GetGuildInfo(unitID)
						
						if guildName then	
							GameTooltipTextLeft2:SetText(SyncUI_GetColorizedText(guildName,0.18,0.6,1))
						end
					end
					do	-- Color Character Info
						local guildName = GetGuildInfo(unitID)
						local level = UnitLevel(unitID)
						local race = UnitRace(unitID)
						local class = UnitClass(unitID)
						local classString = SyncUI_GetColorizedText(class,r,g,b)
						
						if level == -1 then
							level = "??"
						end
						
						local string = format(TOOLTIP_UNIT_LEVEL_RACE_CLASS,level,race,classString)
						
						if guildName then
							GameTooltipTextLeft3:SetText(string)
						else
							GameTooltipTextLeft2:SetText(string)
						end
					end
					
					GameTooltipStatusBar:SetStatusBarColor(r,g,b)				
				else
					self.icon:Hide()
				end
			else
				self.icon:Hide()
			end
			
			if unitID then
				if unitID ~= "player" then
					local target = unitID.."target"
					local name, realm = UnitName(target)


					if target and name then
						if UnitIsUnit("player",target) then
							GameTooltip:AddLine(TARGET..": "..YOU,1,1,1)
						else
							if UnitIsPlayer(target) then
								local class = select(2, UnitClass(target))
								local r,g,b
								
								if class then
									local color = SYNCUI_CLASS_COLORS[class]
									
									if color then
										r,g,b = color.r, color.g, color.b
									end
								end
								
								if r and g and b then
									name = SyncUI_GetColorizedText(name,r,g,b)
								end
							elseif UnitIsEnemy("player", target) then
								name = SyncUI_GetColorizedText(name,1,0,0)
							elseif UnitIsFriend("player", target) then
								name = SyncUI_GetColorizedText(name,0,1,0)
							end

							GameTooltip:AddLine(TARGET..": "..(name or "unknown"),1,1,1)
						end
					end
				end
			end
		end)

		hooksecurefunc(GameTooltip,"SetOwner", function(self)
			if self.icon then
				self.icon:Hide()
			end
		end)
		hooksecurefunc("GameTooltip_SetDefaultAnchor", function(self, parent)
			if cursor and GetMouseFocus() == WorldFrame then
				self:SetOwner(parent, "ANCHOR_CURSOR")
			else
				self:SetOwner(parent, "ANCHOR_NONE")
				self:SetPoint("BOTTOMRIGHT","UIParent","BOTTOMRIGHT",-10,110)
				self.default = 1
			end
		end)
		hooksecurefunc("GameTooltip_OnHide", function(self)
			if self.icon then
				self.icon:Hide()
			end
			if self.arrow then
				self.arrow:Hide()
			end
			if self.justify then
				for line, justify in pairs(self.justify) do
					line:SetJustifyH("LEFT")
				end
				
				self.justify = {}
			end
			if self.divider then
				for line, divider in pairs(self.divider) do
					divider:Hide()
				end
			end
		end)
		hooksecurefunc("SetTooltipMoney", function(self)
			if not self.shownMoneyFrames then
				return
			end

			for i = 1, self.shownMoneyFrames do
				local moneyFrame = _G[self:GetName().."MoneyFrame"..i]
				
				if moneyFrame then
					local name = moneyFrame:GetName()
					local prefix, suffix, gBtn, sBtn, cBtn = _G[name.."PrefixText"], _G[name.."SuffixText"], _G[name.."GoldButtonText"], _G[name.."SilverButtonText"], _G[name.."CopperButtonText"]
					
					for _, frame in pairs({prefix,suffix,gBtn,sBtn,cBtn}) do
						frame:SetFontObject(fontMedium)
					end
				end
			end
		end)
		hooksecurefunc("ToggleDropDownMenu", function(level)
			if not level then
				level = 1
			end
			local menu = _G["DropDownList"..level]
			local bgDrop = _G[menu:GetName().."MenuBackdrop"]

			OverwriteBackdrop(bgDrop)
			
			for index = 1, menu.numButtons do
				local button = _G[menu:GetName().."Button"..index]
				local arrow = _G[button:GetName().."ExpandArrow"]
				local icon = _G[button:GetName().."Icon"]
				local yPos = select(5, button:GetPoint())
				
				button:SetNormalFontObject(fontMedium)
				button:SetHighlightFontObject(fontMedium)
				button:SetDisabledFontObject(fontMedium)
				arrow:SetPoint("TOPRIGHT",menu,-9,yPos)
				icon:SetPoint("TOPRIGHT",menu,-11,yPos)
			end
		end)
	end
end
