
local _G = _G
local maxUnitAuras = 14
local maxBossAuras = 5
local maxPartyAuras = 4
local maxRaidAuras = 3
local maxReactiveAuras = 10
local HealFrameBuffFilter = {
	["DRUID"] = {
		[   774] = "Rejuvenation",
		[  8936] = "Regrowth",
		[ 33763] = "Life Bloom",
		[ 48438] = "Wild Growth",
		[ 48504] = "Living Seed",
		[155777] = "Germination",
	},
	["MONK"] = {
		[115175] = "Soothing Mist",
		[125950] = "Soothing Mist (Statue)",
		[119611] = "Renewing Mist",
		[132120] = "Enveloping Mist",
		[185158] = "Extend Life",
	},
	["PALADIN"] = {
		[ 86273] = "Illuminated Healing",
		[ 53563] = "Beacon of Light",
		[156322] = "Eternal Flame",
		[148039] = "Sacred Shield"
	},
	["PRIEST"] = {
		[    17] = "Powerword: Shield",
		[ 47753] = "Divine Aegis",
		[152118] = "Clarity of Will",
		[ 41635] = "PoM",
		[ 88684] = "Holy Word: Serenity",
		[   139] = "Renew",
		[ 77489] = "Echo of Light",
	},
	["SHAMAN"] = {
		[  974] = "Earth Shield",
		[61295] = "Riptide",
	},
}
local HealFrameDebuffFilter = {
	[  6788] = "Weakened Soul",
	[ 25771] = "Forbearance",
	[ 36032] = "Arcane Charge",
	[ 97821] = "Void Touched",
	[123981] = "Perdition",

	[ 57723] = "Heroism Debuff",
	[ 57724] = "Bloodlust Debuff",
	[ 80354] = "Time Warp Debuff",
	[ 95809] = "Hysteria Debuff",

	[ 26013] = "Deserter",
	[ 71041] = "Dungeon Deserter",
	[ 71328] = "Dungeon Cooldown",
	[ 95223] = "Mass Ressurrection",
}
local HealFrameReactiveAuras = {
	[102342] = "Iron Bark",
	[116849] = "Life Cocoon",
	[  1022] = "Hand of Protection",
	[  6940] = "Hand of Sacrifice",
	[ 47788] = "Guardian Spirit",
	[ 33206] = "Pain Suppression",
	[114030] = "Vigilance",
	--[[ Experimental
	[ 48707] = "Anti-Magic Shell",
	[ 48792] = "Icebound Fortitude",
	[171049] = "Rune Tap",
	
	[ 22812] = "Bark Skin",
	[ 61336] = "Survival Instincts",
	[155835] = "Bristling Fur",
	[ 62606] = "Savage Defense",
	--]]
}


local function GetColor(timer,point)
	local perc = timer/point
	local r,g,b

	if math.floor(timer) > point then
		return 0,1,0
	else
		if perc > 0.5 then
			return 0+(1-perc)*2,1,0
		else
			return 1,0+perc*2,0
		end
	end
end

local function SetAuraInfo(self,icon,charge,duration,expire,spellID,secure)
	local timeLeft = duration - expire + GetTime()
	
	self.spellID = spellID
	self.timer = GetTime() - timeLeft
	self.Cooldown:SetCooldown(GetTime() - timeLeft, duration)
	self.Icon:SetTexture(icon)
	
	if charge > 1 then
		self.Charge:SetText(charge)
	else
		self.Charge:SetText("")
	end
	
	if not secure then
		self:Show()
	end
end

local function GetAuraIndex(self,unitID,type)
	for index = 1, 40 do
		local spellID;
		
		if type == "Buff" then
			spellID = select(11, UnitBuff(unitID,index))
		elseif type == "Debuff" then
			spellID = select(11, UnitDebuff(unitID,index))
		end
		
		if self.spellID == spellID then return index end
	end
end

local function SetUnitAura(frame,unitID)
	for index = 1, maxUnitAuras do
		local buff = _G[frame.."Buff"..index]
		local debuff = _G[frame.."Debuff"..index]
		
		-- buffs
		if unitID ~= "player" and buff then
			buff:Hide()

			local _,_,icon,charge,_,duration,expire,caster,_,isRaidBuff,spellID = UnitBuff(unitID,index)
			
			if spellID then
				SetAuraInfo(buff,icon,charge,duration,expire,spellID)
			end
		end

		-- debuffs
		if debuff then
			debuff:Hide()

			local filter;
			
			if SyncUI_Data and SyncUI_Data.Options.Auras.DebuffFilter and unitID ~= "player" and UnitIsEnemy("player",unitID) then
				filter = "PLAYER"
			end
			if unitID == "targettarget" or unitID == "focustarget" then
				filter = nil
			end
			
			local _,_,icon,charge,_,duration,expire,caster,_,_,spellID = UnitDebuff(unitID,index,filter)
			
			if spellID then
				SetAuraInfo(debuff,icon,charge,duration,expire,spellID)
				
				if caster == "player" or caster == "pet" then
					debuff.Icon:SetDesaturated(false)
				elseif unitID ~= "player" then
					if UnitIsEnemy("player",unitID) then
						debuff.Icon:SetDesaturated(true)
					else
						debuff.Icon:SetDesaturated(false)
					end
				end
			end
		end
	end
end

local function SetBossAura(frame,unitID,index,max)
	local _, _, icon, charge, debuffType, duration, expire, _, isStealable, _, spellID = UnitDebuff(unitID,index,"PLAYER")
	
	if spellID then
		for i = 1, maxBossAuras do
			local debuff = _G[frame.."Debuff"..i]
			
			if debuff.spellID == nil then
				SetAuraInfo(debuff,icon,charge,duration,expire,spellID)
				
				max = max + 1
				return max
			end
		end
	end		

	return max
end

local function SetPartyAura(frame,unitID,type,index,max)
	if type == "debuff" then
		local _, _, icon, charge, debuffType, duration, expire, _, isStealable, _, spellID = UnitDebuff(unitID,index)
		
		if spellID then
			for i = 1, maxBossAuras do
				local debuff = _G[frame.."Debuff"..i]
				
				if debuff.spellID == nil then
					SetAuraInfo(debuff,icon,charge,duration,expire,spellID)
					
					max = max + 1
					return max
				end
			end
		end		
		return max
	end
	
	if type == "buff" then
		local _,_,icon,charge,_,duration,expire,caster,_,isRaidBuff,spellID = UnitBuff(unitID,index)
		
		if spellID and caster == "player" and not isRaidBuff then
			for i = 1, maxPartyAuras do
				local buff = _G[frame.."Buff"..i]
				
				if buff.spellID == nil then
					SetAuraInfo(buff,icon,charge,duration,expire,spellID)
					
					max = max + 1
					return max
				end
			end
		end		
		return max	
	end
end

local function SetReactiveAura(frame,unitID,index,max)
	local _, _, icon, charge, debuffType, duration, expire, _, isStealable, _, spellID = UnitBuff(unitID,index)
	
	if spellID then
		if not SyncUI_CharData or not SyncUI_CharData.WeakAuras then return end
		
		for _, AuraFilterID in pairs(SyncUI_CharData.WeakAuras) do
			if spellID == tonumber(AuraFilterID) then
				for i = 1, maxReactiveAuras do
					local aura = _G[frame.."Aura"..i]
					
					if aura.spellID == nil then
						SetAuraInfo(aura,icon,charge,duration,expire,spellID)
						
						max = max + 1
						return max
					end
				end
			end
		end
	end		

	return max
end

local function SetPlayerBuffs(frame,unitID,index,max)
	local _,_,icon,charge,_,duration,expire,_,_,isRaidBuff,spellID = UnitBuff(unitID,index)
	
	if GetNumGroupMembers() > 0 then
		if spellID and not isRaidBuff then
			for i = 1, maxUnitAuras do
				local buff = _G[frame.."Buff"..i]
				
				if buff.spellID == nil then
					SetAuraInfo(buff,icon,charge,duration,expire,spellID)
					
					max = max + 1
					return max
				end
			end
		end
	else
		if spellID then
			for i = 1, maxUnitAuras do
				local buff = _G[frame.."Buff"..i]
				
				if buff.spellID == nil then
					SetAuraInfo(buff,icon,charge,duration,expire,spellID)
					
					max = max + 1
					return max
				end
			end
		end
	end

	return max
end

local function SetRaidBuff(frame,unitID,index,max)
	local _,_,icon,charge,_,duration,expire,caster,_,_,spellID = UnitBuff(unitID,index)

	if spellID and caster == "player" then
		local class = select(2, UnitClass("player"))
		
		if HealFrameBuffFilter[class] then
			for buffID, description in pairs(HealFrameBuffFilter[class]) do
				if spellID == buffID then
					for i = 1, maxRaidAuras do
						local buff = _G[frame.."Buff"..i]
						
						if buff.spellID == nil then
							SetAuraInfo(buff,icon,charge,duration,expire,spellID)
							
							max = max + 1
							return max
						end
					end
				end
			end
		end
	end
	
	return max
end

local function SetRaidDebuff(frame,unitID,index,max)
	local _,_,icon,charge,debuffType,duration,expire,_,_,_,spellID,_,isBossDebuff = UnitDebuff(unitID,index)
	local isFilter
	
	for spell in pairs(HealFrameDebuffFilter) do
		if spell == spellID then
			isFilter = true
			break
		end
	end
	
	if spellID and not isFilter then
		for i = 1, maxRaidAuras do
			local debuff = _G[frame.."Debuff"..i]
			
			if debuff.spellID == nil then
				SetAuraInfo(debuff,icon,charge,duration,expire,spellID)
				
				max = max + 1
				return max
			end
		end
	end
	
	return max
end

local function SetReactiveRaidAura(frame,unitID,index)
	local _,_,icon,charge,_,duration,expire,_,_,_,spellID = UnitBuff(unitID,index)

	if spellID then
		for buffID in pairs(HealFrameReactiveAuras) do
			if spellID == buffID then
				local aura = _G[frame.."ReactiveAura"]

				SetAuraInfo(aura,icon,charge,duration,expire,spellID)
			end
		end
	end
end


function SyncUI_Aura_OnLoad(self)
	self.Cooldown:SetReverse(true)
	self.Cooldown:SetHideCountdownNumbers(true) 
	self.spellID = nil
	self.timer = 0
	
	SyncUI_Cooldown_OnRegister(self.Cooldown)
end

function SyncUI_Aura_OnEnter(self)
	local frame = _G[self:GetName()]
	local auraFrame = self:GetParent()
	local unitFrame = auraFrame:GetParent()
	local unitID = unitFrame.unitID
	local auraIndex = GetAuraIndex(self,unitID,"Buff") or GetAuraIndex(self,unitID,"Debuff")
	
	if not unitID or not auraIndex then return end
	
	-- buffs
	if strfind(self:GetName(),"Buff") then
		local spellID = select(11, UnitBuff(unitID,auraIndex))
		GameTooltip:SetOwner(frame, "ANCHOR_BOTTOMRIGHT",-35,0)
		GameTooltip:SetUnitBuff(unitID, auraIndex)
		GameTooltip:AddDivider()
		GameTooltip:AddLine("|cFF00FF99SpellID: |r"..spellID,1,1,1)
		GameTooltip:Show()
	end

	-- debuffs
	if strfind(self:GetName(),"Debuff") then
		local spellID = select(11, UnitDebuff(unitID,auraIndex))
		GameTooltip:SetOwner(frame, "ANCHOR_TOPLEFT",0,0)
		GameTooltip:SetUnitDebuff(unitID, auraIndex)
		GameTooltip:AddDivider()
		GameTooltip:AddLine("|cFF00FF99SpellID: |r"..spellID,1,1,1)
		GameTooltip:Show()
	end

	-- reactive Auras
	if strfind(self:GetName(),"Aura") then
		local spellID = select(11, UnitBuff(unitID,auraIndex))
		GameTooltip:SetOwner(frame, "ANCHOR_TOPLEFT",0,0)
		GameTooltip:SetUnitBuff(unitID, auraIndex)
		GameTooltip:AddDivider()
		GameTooltip:AddLine("|cFF00FF99SpellID: |r"..spellID,1,1,1)
		GameTooltip:Show()
	end
end

function SyncUI_Aura_OnUpdate(self)
	local durationTimer = SyncUI_OptionPanel_Aura.DurationTimer
	local durationColor = SyncUI_OptionPanel_Aura.DurationColor
	local durationSpiral = SyncUI_OptionPanel_Aura.DurationSpiral
	
	if durationSpiral:GetChecked() and durationSpiral:IsEnabled() then
		self.Cooldown:SetAlpha(0)
	else
		self.Cooldown:SetAlpha(1)
	end

	if not durationTimer:GetChecked() then
		self.Time:SetText("")
		return
	end

	if not self.timer then
		return
	end

	local duration = self.Cooldown:GetCooldownDuration() / 1000
	local timer = self.timer + duration - GetTime()

	if timer then
		if self.Icon:IsDesaturated() or not durationColor:GetChecked() then
			self.Time:SetVertexColor(1,1,1)
		else
			self.Time:SetVertexColor(GetColor(timer,10))
		end

		if timer >= 3600 then
			self.Time:SetText(math.floor(timer/60/60).."h")
		elseif  timer >= 60 then 
			self.Time:SetText(math.floor(timer/60).."m")
		elseif timer >= 5 then
			self.Time:SetText(math.floor(timer))
		elseif timer >= 0.1 then
			self.Time:SetText(string.format("%.1f",timer))
		else
			self.timer = 0
			self.Time:SetText("")
		end
	else
		self.timer = 0
		self.Time:SetText("")
	end
end

function SyncUI_AuraFrame_OnEvent(self,unitID)
	local frame = self:GetName()
	
	if unitID then
		-- Player
		if unitID == "player" and not self.isHeal then
			local max, max2 = 0, 0
			
			-- Reactive Auras
			for i = 1, maxReactiveAuras do
				local aura = _G[frame.."Aura"..i]
				aura.spellID = nil
				aura:Hide()		
			end

			--[[ Player Buffs
			for i = 1, maxUnitAuras do
				local buff = _G[frame.."Buff"..i]
				buff.spellID = nil
				buff:Hide()			
			end
			--]]

			for index = 1, 40 do
				max = SetReactiveAura(frame,unitID,index,max)
				--max2 = SetPlayerBuffs(frame,unitID,index,max2)
			end
			
			SetUnitAura(frame,unitID)
		end

		-- Target & Focus
		if unitID == "target" or unitID == "focus" then
			SetUnitAura(frame,unitID)
		end
		
		-- Target of Target & Target of Focus
		if unitID == "targettarget" or unitID == "focustarget" then
			SetUnitAura(frame,unitID)
		end

		-- Party Heal + Player
		if (unitID:find("party") or unitID:find("player")) and self.isHeal then
			local max, max2 = 0, 0
		
			do	-- pre hide
				local aura = _G[frame.."ReactiveAura"]
				
				aura:Hide()
			
				for i = 1, 3 do
					local debuff = _G[frame.."Debuff"..i]
					local buff = _G[frame.."Buff"..i]
					
					debuff.spellID = nil
					buff.spellID = nil
					debuff:Hide()
					buff:Hide()
				end
			end

			for index = 1, 40 do
				max = SetRaidDebuff(frame,unitID,index,max)
				max2 = SetRaidBuff(frame,unitID,index,max2)
				
				SetReactiveRaidAura(frame,unitID,index)
			end		
		end
		
		-- Party Normal
		if unitID:find("party") and not self.isHeal then
			do	-- debuffs
				local max = 0

				for i = 1, maxBossAuras do
					local debuff = _G[frame.."Debuff"..i]

					debuff.spellID = nil
					debuff:Hide()
				end
				
				for index = 1, 40 do
					max = SetPartyAura(frame,unitID,"debuff",index,max)
				end
			end
			
			do	-- buffs
				local max = 0

				for i = 1, maxPartyAuras do
					local buff = _G[frame.."Buff"..i]
					
					buff.spellID = nil
					buff:Hide()			
				end

				for index = 1, 40 do
					max = SetPartyAura(frame,unitID,"buff",index,max)
				end		
			end		
		end

		-- Raid Heal
		if unitID:find("raid") and self.isHeal then
			local max, max2 = 0, 0
		
			do	-- pre hide
				local aura = _G[frame.."ReactiveAura"]
				
				aura:Hide()
			
				for i = 1, 3 do
					local debuff = _G[frame.."Debuff"..i]
					local buff = _G[frame.."Buff"..i]
					
					debuff.spellID = nil
					buff.spellID = nil
					debuff:Hide()
					buff:Hide()
				end
			end

			for index = 1, 40 do
				max = SetRaidDebuff(frame,unitID,index,max)
				max2 = SetRaidBuff(frame,unitID,index,max2)
				
				SetReactiveRaidAura(frame,unitID,index)
			end
		end

		-- Boss
		if unitID:find("boss") then
			local max = 0
			
			for i = 1, maxBossAuras do
				local debuff = _G[frame.."Debuff"..i]
				debuff.spellID = nil
				debuff:Hide()			
			end

			for index = 1, 40 do
				max = SetBossAura(frame,unitID,index,max)
			end
		end
	end
end

function SyncUI_RegisterUnitAura(self,unitID)
	if not self.AuraFrame then
		--error(tostring(self:GetName()..".AuraFrame can not be found! XML-ERROR!"))
		return
	end

	if unitID == "target" then
		self.AuraFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
	end
	if unitID == "focus" then
		self.AuraFrame:RegisterEvent("PLAYER_FOCUS_CHANGED")
	end

	self.AuraFrame:RegisterUnitEvent("UNIT_AURA", unitID)
	self.AuraFrame:SetScript("OnEvent", function()
		SyncUI_AuraFrame_OnEvent(self,unitID)
	end)

	SyncUI_AuraFrame_OnEvent(self,unitID)	
end

function SyncUI_UnregisterUnitAuras(self)
	if not self.AuraFrame then
		return
	end
	
	self.AuraFrame:UnregisterAllEvents()
end


-- Secure Aura Header
local function AuraHeader_UpdateAura(header, button, unit)
	local name,_,icon,charge,_,duration,expire,_,_,_,spellID = UnitAura(unit, button:GetID(), header:GetAttribute("filter"))
	
	if not name then return end
	
	SetAuraInfo(button,icon,charge,duration,expire,spellID,true)
end
 
local function AuraHeader_OnEvent(self, event, unit)
	local secureUnit = SecureButton_GetUnit(self)
	local maxAuras = self:GetAttribute("wrapAfter") * self:GetAttribute("maxWraps")
	
	if not self:IsShown() or (unit and unit ~= secureUnit) then
		return
	end

	for index = 1, maxAuras do
		local aura = self:GetAttribute("child" .. index)
		
		if aura then
			AuraHeader_UpdateAura(self, aura, secureUnit)
		end
	end
end

function SyncUI_PlayerFrame_AuraHeader_Initialize()
	for i = 1, 2 do
		local header = CreateFrame("Frame", "SyncUI_PlayerFrame_BuffHeader"..i, SyncUI_UIParent, "SecureAuraHeaderTemplate")
		header:SetScale(SyncUI_UIScale*1.2)
		header:SetPoint("TOPRIGHT",SyncUI_Minimap,"TOPLEFT",-35,-1)
		header:HookScript("OnEvent", AuraHeader_OnEvent)
		header:HookScript("OnShow", AuraHeader_OnEvent)
		
		-- Settings
		header:SetAttribute("template", "SyncUI_SecureAuraTemplate")
		header:SetAttribute("unit", "player")
		header:SetAttribute("filter", "HELPFUL")
		header:SetAttribute("minWidth", 350)
		header:SetAttribute("minHeight", 140)
		header:SetAttribute("xOffset", -28)
		header:SetAttribute("wrapYOffset", -30)
		header:SetAttribute("wrapAfter", 10)
		header:SetAttribute("maxWraps", 2)

		if i == 1 then
			RegisterStateDriver(header, "visibility", "[@raid1,exists] hide; [@party1,exists] hide; show")
		else
			header:SetAttribute("consolidateTo",9)
			header:SetAttribute("consolidateDuration",-1)
			RegisterStateDriver(header, "visibility", "[@raid1,exists] show; [@party1,exists] show; hide")
		end
		
		header:Show()
		 
		AuraHeader_OnEvent(header)
	end
	
--[[	Secure Aura Header Template - Attribute List
	filter = [STRING] -- a pipe-separated list of aura filter options ("RAID" will be ignored)
	separateOwn = [NUMBER] -- indicate whether buffs you cast yourself should be separated before (1) or after (-1) others. If 0 or nil, no separation is done.
	sortMethod = ["INDEX", "NAME", "TIME"] -- defines how the group is sorted (Default: "INDEX")
	sortDirection = ["+", "-"] -- defines the sort order (Default: "+")
	groupBy = [nil, auraFilter] -- if present, a series of comma-separated filters, appended to the base filter to separate auras into groups within a single stream
	includeWeapons = [nil, NUMBER] -- The aura sub-stream before which to include temporary weapon enchants. If nil or 0, they are ignored.
	consolidateTo = [nil, NUMBER] -- The aura sub-stream before which to place a proxy for the consolidated header. If nil or 0, consolidation is ignored.
	consolidateDuration = [nil, NUMBER] -- the minimum total duration an aura should have to be considered for consolidation (Default: 30)
	consolidateThreshold = [nil, NUMBER] -- buffs with less remaining duration than this many seconds should not be consolidated (Default: 10)
	consolidateFraction = [nil, NUMBER] -- The fraction of remaining duration a buff should still have to be eligible for consolidation (Default: .10)
 
	template = [STRING] -- the XML template to use for the unit buttons. If the created widgets should be something other than Buttons, append the Widget name after a comma.
	weaponTemplate = [STRING] -- the XML template to use for temporary enchant buttons. Can be nil if you preset the tempEnchant1 and tempEnchant2 attributes, or if you don"t include temporary enchants.
	consolidateProxy = [STRING|Frame] -- Either the button which represents consolidated buffs, or the name of the template used to construct one.
	consolidateHeader = [STRING|Frame] -- Either the aura header which contains consolidated buffs, or the name of the template used to construct one.
 
	point = [STRING] -- a valid XML anchoring point (Default: "TOPRIGHT")
	minWidth = [nil, NUMBER] -- the minimum width of the container frame
	minHeight = [nil, NUMBER] -- the minimum height of the container frame
	xOffset = [NUMBER] -- the x-Offset to use when anchoring the unit buttons (Default: width)
	yOffset = [NUMBER] -- the y-Offset to use when anchoring the unit buttons (Default: height)
	wrapAfter = [NUMBER] -- begin a new row or column after this many auras
	wrapXOffset = [NUMBER] -- the x-offset from one row or column to the next
	wrapYOffset = [NUMBER] -- the y-offset from one row or column to the next
	maxWraps = [NUMBER] -- limit the number of rows or columns
--]]
end
