
local _G = _G

function SyncUI_ResourceBar_OnEvent(self,event,...)
	local class = select(2, UnitClass("player"))
	local specID; 
	
	if GetSpecialization() then
		specID = select(1, GetSpecializationInfo(GetSpecialization()))
	end

	if event == "PLAYER_TALENT_UPDATE" then
		if specID == self.specID then
			return
		end
	end

	if event == "PLAYER_ENTERING_WORLD" then
		if class == "DRUID" then
			self:RegisterEvent("UPDATE_SHAPESHIFT_FORM")
		end

		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	end
	
	for _, frame in pairs({self:GetChildren()})	do
		if frame:IsShown() then
			frame:Hide()
		end
	end

	self.specID = specID
	
	if class == "DEATHKNIGHT" then
		SyncUI_RuneBar:Show()
	end
	if class == "DRUID" or event == "UPDATE_SHAPESHIFT_FORM" then
		local form = GetShapeshiftForm()
		local isGlyphed;
		
		-- search for Glyph of the Stag to fix balance form value
		for i = 1, 5, 2 do
			local glyphID = select(4, GetGlyphSocketInfo(i, GetActiveSpecGroup()))

			if glyphID == 114338 then
				isGlyphed = true
				break
			end
		end
		
		-- Cat Form
		if form == 2 then
			SyncUI_ComboBar:Show()
		end
		
		-- Moonkin Form
		if specID == 102 then
			if (not isGlyphed and form == 4) or (isGlyphed and form == 5) then
				SyncUI_EclipseBar:Show()
			end
		end
		
		if specID == 105 then
			SyncUI_StatueBar:Show()
		end
	end
	if class == "MAGE" then
		if specID == 62 then
			SyncUI_ArcaneChargeBar:Show()
		end
	end
	if class == "MONK" then
		SyncUI_ChiBar:Show()

		if specID == 268 then
			SyncUI_StaggerBar:Show()
		end
		if specID == 269 then
			SyncUI_SpiritBar:Show()
		end
		if specID == 268 or specID == 270 then
			SyncUI_StatueBar:Show()
		end
	end
	if class == "PALADIN" then
		SyncUI_HolyBar:Show()		
	end
	if class == "PRIEST" then
		if specID == 258 then
			SyncUI_OrbBar:Show()			
		end
		if specID == 257 then
			SyncUI_StatueBar:Show()
		end
	end
	if class == "ROGUE" then
		SyncUI_ComboBar:Show()
		
		-- Anticipation
		for i = 1, 40 do
			local spellID = select(11, UnitBuff("player",i))
			
			if spellID == 110051 then
				local charge = select(4, UnitBuff("player",i))
				if charge and charge > 0 then
				
				else
					SyncUI_ComboBar.Anticipation:Hide()
				end	
				
				SyncUI_ComboBar.Anticipation:Show()
			else
				SyncUI_ComboBar.Anticipation:Hide()
			end
		end
	end
	if class == "SHAMAN" then
		SyncUI_TotemBar:Show()		
	end
	if class == "WARLOCK" then
		if specID == 265 then
			SyncUI_SoulShardBar:Show()
		end
		if specID == 266 then
			SyncUI_FuryBar:Show()
		end
		if specID == 267 then
			SyncUI_EmberBar:Show()
		end
	end
end

function SyncUI_RuneBar_OnUpdate(self)
	for rune = 1, 6 do
		local i;

			-- Fix Rune Index
			if rune >= 1 and rune <= 2 then i = rune
			elseif rune == 3 then i = 5
			elseif rune == 4 then i = 6
			elseif rune == 5 then i = 3
			elseif rune == 6 then i = 4 end

		local start, duration, runeReady = GetRuneCooldown(rune);
		local runeType = GetRuneType(rune);
		local frame = _G[self:GetName().."Rune"..i]
		local statusBar = frame.StatusBar
		
		-- Set Cooldown
		statusBar:SetMinMaxValues(0,duration)
		statusBar:SetValue(GetTime()-start)
		
		-- Ready Checker
		if not runeReady then
			frame.wasReady = nil
		end
		
		-- Set Rune Color
		if runeType == 1 then
			statusBar:SetStatusBarColor(1,0,0)
		elseif runeType == 2 then
			statusBar:SetStatusBarColor(0,1,0)
		elseif runeType == 3 then
			statusBar:SetStatusBarColor(0,0.7,1)
		elseif runeType == 4 then
			statusBar:SetStatusBarColor(0.8,0.3,1)
		end
	end
end

function SyncUI_RuneBar_OnEvent(self,event,...)
	if event == "RUNE_POWER_UPDATE" then
		for rune = 1, 6 do
			local i;

				-- Fix Rune Index
				if rune >= 1 and rune <= 2 then i = rune
				elseif rune == 3 then i = 5
				elseif rune == 4 then i = 6
				elseif rune == 5 then i = 3
				elseif rune == 6 then i = 4 end
			
			local start, duration, runeReady = GetRuneCooldown(rune);
			local frame = _G[self:GetName().."Rune"..i]
			
			if runeReady then
				if not frame.wasReady then
					frame.wasReady = true
					RuneButton_ShineFadeIn(frame.Shine)
				end
			end
		end
	end
end

function SyncUI_EclipseBar_OnUpdate(self)
	local value = UnitPower("player", SPELL_POWER_ECLIPSE)
	local statusBar = self.StatusBar
	local state;

	-- Get State
	if value <= 0 then
		value = value-(2*value)
		state = "lunar"
	elseif value > 0 then
		state = "solar"
	end
	
	-- Eclipse Flash
	if value >= 35 then
		if state == "lunar" then
			self.Lunar:Show()	
		end
		if state == "solar" then
			self.Solar:Show()	
		end
	else
		self.Lunar:Hide()
		self.Solar:Hide()
	end
	
	-- Set Bar Color
	if state == "lunar" and self.state ~= "lunar" then
		statusBar:SetReverseFill(true)
		statusBar:SetStatusBarColor(0.3,0.6,1)
		self.state = "lunar"
	elseif state == "solar" and self.state ~= "solar" then
		statusBar:SetReverseFill(false)
		statusBar:SetStatusBarColor(1,1,0.3)
		self.state = "solar"
	end
	
	-- Set Value
	statusBar:SetValue(value)
end

function SyncUI_ArcaneChargeBar_OnUpdate(self)
	local charge = select(4, UnitDebuff("player",GetSpellInfo(36032)))

	
	for i = 1, 4 do
		local frame = _G[self:GetName().."Charge"..i]
		local statusBar = frame.StatusBar
		
		if charge and i <= charge then
			statusBar:SetValue(1)
		else
			statusBar:SetValue(0)
		end
	end
end

function SyncUI_ChiBar_OnUpdate(self)
	local chi, maxChi = UnitPower("player", SPELL_POWER_CHI), UnitPowerMax("player", SPELL_POWER_CHI)
	local left, right, top, bottom

	-- Get TexCoords
	if maxChi == 4 then
		left, right, top, bottom = 0, 0.74609375, 0.3125, 0.45703125
	elseif maxChi == 5 then
		left, right, top, bottom = 0, 0.74609375, 0.46875, 0.61328125
	elseif maxChi == 6 then
		left, right, top, bottom = 0, 0.74609375, 0.625, 0.76953125
	end

	SyncUI_ChiBarBorder:SetTexCoord(left,right,top,bottom)
	
	for i = 1, 6 do
		local frame = _G[self:GetName().."Chi"..i]
		local statusBar = frame.StatusBar
	
		if i <= maxChi then
			frame:SetAlpha(1)

			if maxChi == 4 then
				frame:SetSize(38,15)
			elseif maxChi == 5 then
				frame:SetSize(29,15)
			elseif maxChi == 6 then
				frame:SetSize(23,15)
			end
			
			if i <= chi then
				statusBar:SetValue(1)
			else
				statusBar:SetValue(0)
			end
		else
			frame:SetAlpha(0)
		end
	end
end

function SyncUI_StaggerBar_OnUpdate(self)
	local stagger, maxStagger = UnitStagger("player"), UnitHealthMax("player")
	local perc = math.ceil(stagger*100/maxStagger)
	local statusBar = self.StatusBar
	
	--Update value + maxValue
	statusBar:SetMinMaxValues(0, maxStagger)
	SmoothBar(statusBar,stagger)
	
	--Update Color
	if perc > 60 then
		statusBar:SetStatusBarColor(1,0,0)
	elseif perc > 30 then
		statusBar:SetStatusBarColor(1,1,0)
	else
		statusBar:SetStatusBarColor(0,1,0)
	end
end

function SyncUI_HolyBar_OnUpdate(self)
	local holyPower = UnitPower("player", SPELL_POWER_HOLY_POWER)
	
	for i = 1, 5 do
		local frame = _G[self:GetName().."Holy"..i]
		local statusBar = frame.StatusBar
		
		if i <= holyPower then
			statusBar:SetValue(1)
		else
			statusBar:SetValue(0)
		end
	end
end

function SyncUI_OrbBar_OnUpdate(self)
	local shadowOrb, maxOrbs = UnitPower("player", SPELL_POWER_SHADOW_ORBS), UnitPowerMax("player", SPELL_POWER_SHADOW_ORBS)
	local left, right, top, bottom
	
	-- Get TexCoords
	if maxOrbs == 3 then
		left, right, top, bottom = 0,0.74609375,0.15625,0.30078125
	elseif maxOrbs == 5 then
		left, right, top, bottom = 0,0.74609375,0.46875,0.61328125
	end
	
	SyncUI_OrbBarBorder:SetTexCoord(left,right,top,bottom)
	
	for i = 1, 5 do
		local frame = _G[self:GetName().."Orb"..i]
		local statusBar = frame.StatusBar
		
		if i <= maxOrbs then
			frame:SetAlpha(1)
			
			if maxOrbs == 3 then
				frame:SetSize(53,15)
			elseif maxOrbs == 5 then
				frame:SetSize(29,15)
			end
			
			if i <= shadowOrb then
				statusBar:SetValue(1)
			else
				statusBar:SetValue(0)
			end		
		else
			frame:SetAlpha(0)
		end
	end
end

function SyncUI_TotemBar_OnUpdate(self)
	for i = 1, 4 do
		local frame = _G[self:GetName().."Totem"..i]
		local statusBar = frame.StatusBar
		local timer = frame.Timer
		
		if GetTotemInfo(i) then
			local haveTotem, name, start, duration, icon = GetTotemInfo(i)
			local timeLeft = math.ceil(start + duration - GetTime())
			
			frame.Icon:SetTexture(icon)

			statusBar:SetMinMaxValues(0,duration)
			statusBar:SetValue(GetTime()-start)
			statusBar:SetStatusBarColor(0,0,0,0.75)
			
			if timeLeft then
				if timeLeft >= 60 then
					timer:SetText(string.format("%.0f",timeLeft/60).."m")
				elseif timeLeft > 0 then
					timer:SetText(string.format("%.0f",timeLeft))
				else
					timer:SetText("")
				end
			end
		else
			frame.Icon:SetTexture()
			statusBar:SetStatusBarColor(0,0,0,0)
			timer:SetText("")
		end		
	end
end

function SyncUI_EmberBar_OnUpdate(self)
	local ember = UnitPower("player", SPELL_POWER_BURNING_EMBERS, true)

	for i = 1, 4 do
		local frame = _G[self:GetName().."Ember"..i]
		local statusBar = frame.StatusBar
		local minValue = i*10-10
		local maxValue = i*10
		
		statusBar:SetMinMaxValues(minValue,maxValue)
		statusBar:SetValue(ember)
		
		if IsPlayerSpell(101508) then
			statusBar:SetStatusBarColor(0.7,1,0)
		elseif not IsPlayerSpell(101508) then
			statusBar:SetStatusBarColor(1,0.5,0)
		end
	end
end

function SyncUI_SoulShardBar_OnUpdate(self)
	local shard = UnitPower("player", SPELL_POWER_SOUL_SHARDS)
	
	for i = 1, 4 do
		local frame = _G[self:GetName().."Shard"..i]
		local statusBar = frame.StatusBar	
		
		if i <= shard then
			statusBar:SetValue(1)
		else
			statusBar:SetValue(0)
		end
	end
end

function SyncUI_FuryBar_OnUpdate(self)
	local fury = UnitPower("player", SPELL_POWER_DEMONIC_FURY)
	
	--Update Value
	self.Text:SetText(fury)
	SmoothBar(self.StatusBar,fury)

	-- Show Glow
	if fury >= 900 then
		self.Metamorph:Show()
		self.StatusBar:SetStatusBarColor(0.5,1,0)
	else
		self.Metamorph:Hide()
		self.StatusBar:SetStatusBarColor(0.5,0.3,0.8)
	end
end

function SyncUI_ComboBar_OnEvent(self)
	local comboPoints = GetComboPoints("player", "target")

	for i = 1, 5 do
		local frame = _G[self:GetName().."Combo"..i]
		local statusBar = frame.StatusBar
		
		if i <= comboPoints then
			statusBar:SetValue(1)
		else
			statusBar:SetValue(0)
		end
	end
end

function SyncUI_StatueBar_OnUpdate(self)
	if GetTotemInfo(1) then
		local haveTotem, name, start, duration, icon = GetTotemInfo(1)
		local statusBar = self.Button.StatusBar
		local timer = self.Button.Timer
		local timeLeft = math.ceil(start + duration - GetTime())
		
		self:SetAlpha(1)
		self.Button.Icon:SetTexture(icon)

		statusBar:SetMinMaxValues(0,duration)
		statusBar:SetValue(GetTime()-start)
		statusBar:SetStatusBarColor(0,0,0,0.75)
		
		if timeLeft then
			if timeLeft >= 60 then
				timer:SetText(string.format("%.0f",timeLeft/60).."m")
			elseif timeLeft > 0 then
				timer:SetText(string.format("%.0f",timeLeft))
			else
				timer:SetText("")
			end
		end
	else
		self:SetAlpha(0)
	end
end

function SyncUI_SpiritBar_OnLoad(self)
	self.spirits = {self.Spirit1,self.Spirit2}
	self:RegisterEvent("PLAYER_TOTEM_UPDATE")
	
	for _, spirit in pairs(self.spirits) do
		spirit:SetAlpha(0)
		spirit.info = {}
		spirit.Health = spirit.ArtFrame.Health
		spirit.Name = spirit.ArtFrame.Name

		spirit:RegisterForClicks("RightButtonUp")
		spirit:SetAttribute("type2","destroytotem")
		spirit:SetAttribute("totem-slot",spirit:GetID())	
	end
end

function SyncUI_SpiritBar_OnEvent(self,event,...)
	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		local playerGUID = UnitGUID("player")
		local subEvent,_,source,_,_,_,target,_,_,_,spellID,_,_,damage,over = select(2,...)
		
		for _, spirit in pairs(self.spirits) do		
			-- Summon Spirit
			if source == playerGUID and (spellID == 137639 or spellID == 138122 or spellID == 138121 or spellID == 138123) then
				if subEvent == "SPELL_SUMMON" and not spirit.info.GUID then
					spirit.info.GUID = target
					spirit.info.maxHealth = math.ceil(UnitHealthMax("player") * 0.1)
					spirit.info.health = spirit.info.maxHealth
				end
			end    
			
			-- Update spirit health
			local strDMG = string.find(subEvent,"_DAMAGE")
			local strHEAL = string.find(subEvent,"_HEAL")

			if (strDMG or strHEAL) and target == spirit.info.GUID then
				if subEvent == "SWING_DAMAGE" then
					damage = spellID
				elseif strHEAL then
					damage = -(damage - over)
				end
				
				spirit.info.health = spirit.info.health - damage
			end
		end
	end
	
	if event == "PLAYER_TOTEM_UPDATE" then
		for _, spirit in pairs(self.spirits) do
			local isActive,name,_,_,icon = GetTotemInfo(spirit:GetID())
			
			if isActive then
				spirit.Name:SetText(name)
				spirit.Icon:SetTexture(icon)
				spirit:SetAlpha(1)
			else
				spirit:SetAlpha(0)
			end
		end
	end
end

function SyncUI_SpiritBar_OnUpdate(self)
	for _, spirit in pairs(self.spirits) do
		if spirit:GetAlpha() > 0 then
			local health = spirit.info.health
			local maxHealth = spirit.info.maxHealth
			
			if health and health > 0 then
				spirit.StatusBar:SetMinMaxValues(0,maxHealth)
				SmoothBar(spirit.StatusBar,health)
			end
		else
			if spirit.info.GUID then
				spirit.info = {}
			end
		end
	end
end
