
local _G = _G

local function GetDebuffColors(type)
	local r,g,b
	
	if type == "Curse" then
		r,g,b = 0.5,0.2,1.0
	elseif type == "Disease" then
		r,g,b = 1.0,0.5,0.0
	elseif type == "Magic" then
		r,g,b = 0.0,0.2,0.8
	elseif type == "Poison" then
		r,g,b = 0.0,1.0,0.0
	else
		r,g,b = 1.0,1.0,1.0
	end
	
	return r,g,b
end

local function UpdateBar(self,unitID)
	if self.ManaBar then
		local mana, maxMana = UnitMana(unitID), UnitManaMax(unitID)
		self.ManaBar:SetMinMaxValues(0, maxMana)
		SmoothBar(self.ManaBar,mana)
		
		if UnitMana(unitID) > 0 then
			local color = SYNCUI_POWER_COLORS[select(2, UnitPowerType(unitID))]

			if color then
				self.ManaBar:SetStatusBarColor(color.r,color.g,color.b)
			else
				self.ManaBar:SetStatusBarColor(0.5,0.5,0.5)
			end
		end
	end
	
	if self.HealthBar then
		local health, maxHealth = UnitHealth(unitID), UnitHealthMax(unitID)
		
		self.HealthBar:SetMinMaxValues(0, maxHealth)
		SmoothBar(self.HealthBar,health)

		if UnitIsPlayer(unitID) then
			local class = select(2, UnitClass(unitID))
			local color = SYNCUI_CLASS_COLORS[class]
			
			if color then
				self.HealthBar:SetStatusBarColor(color.r,color.g,color.b,0)
			end
		else
			self.HealthBar:SetStatusBarColor(0,0.8,0,0)
		end
	end

	if self.AbsorbBar then
		local health, maxHealth = UnitHealth(unitID), UnitHealthMax(unitID)
		local absorb = UnitGetTotalAbsorbs(unitID) or 0
		local barWidth, barHeight = self.HealthBar:GetSize()
			
		if health + absorb > maxHealth then
			self.AbsorbBar:SetSize((barWidth-(health*barWidth/maxHealth))+0.1,barHeight)
		else
			self.AbsorbBar:SetSize((absorb * barWidth / maxHealth),barHeight)
		end
		if absorb == 0 then
			self.AbsorbBar:SetAlpha(0)
		else
			self.AbsorbBar:SetPoint("TOPLEFT", self.HealthBar:GetStatusBarTexture(), "TOPRIGHT")
			self.AbsorbBar:SetAlpha(1)
		end
	end
end

local function UpdateBarText(self,unitID)
	if self.Mana then
		local mana = UnitMana(unitID)
		
		if UnitIsDeadOrGhost(unitID) then
			self.Mana:SetText()
		else
			if mana > 10000 then
				self.Mana:SetText(string.format("%.0f",(mana)/1000).."k")
			elseif mana > 0 then
				self.Mana:SetText(mana)
			else
				self.Mana:SetText()
			end
		end
	end

	if self.Health then
		local health, maxHealth = UnitHealth(unitID), UnitHealthMax(unitID)
		
		if not UnitIsConnected(unitID) or UnitIsDeadOrGhost(unitID) then
			self.Health:SetText("")
		else
			if health >= 10000000 then
				self.Health:SetText(string.format("%.0f",(health)/1000000).."m".." | "..string.format("%.0f",math.ceil((health)*100/maxHealth)).."%")
			elseif health >= 10000 then
				self.Health:SetText(string.format("%.0f",(health)/1000).."k".." | "..string.format("%.0f",math.ceil((health)*100/maxHealth)).."%")
			elseif health > 0 then
				self.Health:SetText(health.." | "..string.format("%.0f",math.ceil((health)*100/maxHealth)).."%")
			end
		end
	end
end

local function UpdateName(self,unitID)
	if self.Name then
		local name, server = UnitName(unitID)
		self.Name:SetText(name)
	end
end

local function UpdateLevel(self,unitID)
	if self.Level then
		local unitLvl = UnitLevel(unitID)
		
		if unitLvl >= 0 then
			local r,g,b = SyncUI_GetEffectiveLevelColor(unitLvl)
			
			self.Level:SetText(unitLvl)
			self.Level:SetVertexColor(r,g,b)
		else
			self.Level:SetText("??")
			self.Level:SetVertexColor(1,1,1)
		end
	end
end

local function UpdateStatusIcon(self,unitID)
	if self.Quest then
		if UnitIsQuestBoss(unitID) then
			self.Quest:Show()
		else
			self.Quest:Hide()
		end
	end

	if self.Tapped then
		if not UnitIsDeadOrGhost(unitID) then
			if UnitAffectingCombat(unitID) and UnitIsTapped(unitID) and not UnitIsTappedByPlayer(unitID) then
				self.Tapped:Show()
			else
				self.Tapped:Hide()
			end
		else
			self.Tapped:Hide()
		end
	end

	if self.Role then
		if UnitGroupRolesAssigned(unitID) == "TANK" then
			if not self.Role:IsShown() then
				self.Role:Show()
			end
			self.Role:SetTexCoord(0.625,0.90625,0,0.5625)
		elseif UnitGroupRolesAssigned(unitID) == "HEALER" then
			if not self.Role:IsShown() then
				self.Role:Show()
			end
			self.Role:SetTexCoord(0.3125,0.59375,0,0.5625)
		else
			if self.Role:IsShown() then
				self.Role:Hide()
			end
		end
	end

	if self.StatusIcon then
		if not UnitIsConnected(unitID) then
			self.StatusIcon:SetTexture("Interface\\CHARACTERFRAME\\Disconnect-Icon")
			self.StatusIcon:SetTexCoord(0.2,0.8,0.2,0.8)
			self.StatusIcon:Show()
		elseif UnitHasIncomingResurrection(unitID) then
			self.StatusIcon:SetTexture("Interface\\RaidFrame\\Raid-Icon-Rez")
			self.StatusIcon:SetTexCoord(0,1,0,1)
			self.StatusIcon:Show()		
		elseif UnitIsDeadOrGhost(unitID) then
			self.StatusIcon:SetTexture("Interface\\WorldStateFrame\\SkullBones")
			self.StatusIcon:SetTexCoord(0,0.5,0,0.5)
			self.StatusIcon:Show()
		else
			self.StatusIcon:Hide()
		end
	end
	
	if unitID == "target" or unitID == "focus" then
		if not UnitIsConnected(unitID) then
			self.Disconnect:Show()
			self.Boss:Hide()
			self.Death:Hide()
			self.Level:SetText("")
		else
			self.Disconnect:Hide()
			
			if UnitIsDeadOrGhost(unitID) then
				self.Death:Show()
				self.Boss:Hide()
				self.Level:SetText("")
			else
				self.Death:Hide()
				
				if UnitLevel(unitID) >= 0 then
					self.Boss:Hide()
					self.Level:SetText(UnitLevel(unitID))
				else
					self.Boss:Show()
					self.Level:SetText("")
				end
			end
		end
	end
	
end

local function UpdateSelection(self,unitID)
	if self.Selection then
		if UnitGUID(unitID) == UnitGUID("target") then
			self.Selection:Show()
		else
			self.Selection:Hide()
		end
	end
end

local function UpdateRaidMarks(self,unitID)
	if self.RaidMark then
		local index = GetRaidTargetIndex(unitID)
		
		if index then
			SetRaidTargetIconTexture(self.RaidMark,index)
			self.RaidMark:Show()
		else
			self.RaidMark:Hide()
		end
	end
end

local function UpdateGroupHeader(self,unitID)
	if self.GroupFrame then
		if GetNumGroupMembers() > 5 then
			self.GroupFrame:Show()

			for index = 1, 40 do
				local name, _,group = GetRaidRosterInfo(index)
				if name == UnitName("player") then
					self.Group:SetText("Group "..group)
				end
			end
		else
			self.GroupFrame:Hide()
		end
	end
end

local function UpdateRange(self,unitID)
	if self.isRaid or self.isHeal then
		if UnitInRange(unitID) then
			self:SetAlpha(1)
			self.HealthBar:SetAlpha(1)
		else
			self:SetAlpha(0.8)
			self.HealthBar:SetAlpha(0.5)
		end
	end
end

local function UpdateUnitClass(self,unitID)
	if self.UnitClass then
		local type = UnitClassification(unitID)
		local level = UnitLevel(unitID)
		
		if (type ~= "rare" and type ~= "rareelite" and type ~= "elite") or level == -1 or UnitIsDeadOrGhost(unitID) then
			self.UnitClass:Hide()
		else
			self.UnitClass:Show()
			
			if type == "rare" then
				self.UnitClass:SetVertexColor(0,1,0)
			elseif type == "rareelite" then
				self.UnitClass:SetVertexColor(0,0.5,1)
			elseif type == "elite" then
				self.UnitClass:SetVertexColor(1,0.8,0)
			end
		end
	end
end

local function UpdateDispel(self,unitID)
	if self.Dispel then
		local isDispel,r,g,b;
		
		for index = 1, 40 do
			local type = select(5, UnitDebuff(unitID,index))
			
			if type then
				r,g,b = GetDebuffColors(type)
				isDispel = true
				break
			end
		end

		if isDispel then
			self.Dispel:Show()
			self.Dispel:SetVertexColor(r,g,b)
		else
			self.Dispel:Hide()
		end
	end
end

local function UpdateThreat(self,unitID)
	if self.Threat then
		local feedbackUnit = "player"
		local status;
		
		if feedbackUnit ~= unitID then
			status = UnitThreatSituation(feedbackUnit, unitID)
		else
			status = UnitThreatSituation(feedbackUnit)
		end
		
		if status then
			local isTanking, status, perc = UnitDetailedThreatSituation(feedbackUnit, unitID)

			if isTanking or perc and perc > 0 then
				local role = UnitGroupRolesAssigned("player")
							
				if isTanking and role == "TANK" then
					self.Threat.StatusBar:SetStatusBarColor(0.2,0.5,1)
				else
					self.Threat.StatusBar:SetStatusBarColor(GetThreatStatusColor(status))
				end
				
				if perc < 100 then
					self.Threat.Text:SetText(format("%0.f",perc))
				else
					self.Threat.Text:SetText("")
				end

				self.Threat.StatusBar:SetValue(perc)
				
				self.Threat:Show()
			else
				self.Threat:Hide()
			end
		else
			self.Threat:Hide()
		end
	end
end

local function UpdatePvPState(self,unitID)
	if self.PvPIcon then
		local isPvP = UnitIsPVP(unitID)
		local faction = UnitFactionGroup(unitID)
		local horde = "Interface\\BattlefieldFrame\\Battleground-Horde"
		local alliance = "Interface\\BattlefieldFrame\\Battleground-Alliance"
		
		if faction == "Alliance" then
			if UnitIsMercenary(unitID) then
				self.PvPIcon:SetTexture(horde)
			else
				self.PvPIcon:SetTexture(alliance)
			end
		end
		if faction == "Horde" then
			if UnitIsMercenary(unitID) then
				self.PvPIcon:SetTexture(alliance)
			else
				self.PvPIcon:SetTexture(horde)
			end
		end
		
		if isPvP then
			self.PvPIcon:Show()
		else
			self.PvPIcon:Hide()
		end
	end
end


-- General Unit Frame Functions
function SyncUI_UnitFrame_OnRegister(self,force)
	local unitID = self.unitID
	
	self:SetAttribute("unit", unitID)
	self:SetAttribute("type1", "target")
	self:SetAttribute("*type2", "togglemenu")
	self:RegisterForClicks("AnyUp")
	self:RegisterForDrag("LeftButton")
	
	if force then RegisterUnitWatch(self) end
	
	SyncUI_UnitFrame_OnRegisterClickCasting(self)
end

function SyncUI_UnitFrame_OnRegisterClickCasting(self)		
	if not ClickCastFrames then ClickCastFrames = {} end

	ClickCastFrames[self] = true;
end

function SyncUI_UnitFrame_OnUpdate(self)
	local modUnit = SecureButton_GetModifiedUnit(self)
	local unitID = self.unitID
	
	if self.unitID ~= modUnit then
		self.unitID = modUnit
		unitID = modUnit
		
		SyncUI_UnregisterUnitAuras(self)
		SyncUI_RegisterUnitAura(self,self.unitID)
	end

	if UnitExists(unitID) then
		UpdateBar(self,unitID)
		UpdateBarText(self,unitID)
		UpdateName(self,unitID)
		UpdateLevel(self,unitID)
		UpdateStatusIcon(self,unitID)
		UpdateSelection(self,unitID)
		UpdateRaidMarks(self,unitID)
		UpdateGroupHeader(self,unitID)
		UpdateRange(self,unitID)
		UpdateUnitClass(self,unitID)
		UpdateDispel(self,unitID)
		UpdateThreat(self,unitID)
		UpdatePvPState(self,unitID)
	end
end

function SyncUI_UnitFrame_OnEnter(self)
	local unitID = self.unitID or SecureButton_GetModifiedUnit(self)
	local r, g, b = GameTooltip_UnitColor(unitID)
	
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	GameTooltip:SetUnit(unitID)
	GameTooltipTextLeft1:SetTextColor(r, g, b)
	GameTooltip:Show()
	
	-- StatusBarColor
	if UnitIsPlayer(self.unitID) then
		local color = SYNCUI_CLASS_COLORS[select(2, UnitClass(self.unitID))]
		
		if color then
			local r,g,b = color.r, color.g, color.b
		
			GameTooltipStatusBar:SetStatusBarColor(r,g,b)
		end
	end
end


-- Specific Unit Frame Load Functions
function SyncUI_PlayerFrame_OnLoad(self)
	self:SetScale(SyncUI_UIScale*1.2)
	self:RegisterForDrag("")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_UPDATE_RESTING")
	self:RegisterEvent("PLAYER_ENTER_COMBAT")
	self:RegisterEvent("PLAYER_LEAVE_COMBAT")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	
	self.unitID = "player"
	self.Group = self.GroupFrame.Group
	self.Health = self.ArtFrame.Health
	self.Mana = self.ArtFrame.Mana
	self.Level = self.ArtFrame.Level
	self.RaidMark = self.ArtFrame.RaidMark
	self.PvPIcon = self.ArtFrame.PvPIcon
	
	RegisterStateDriver(self, "visibility", "[petbattle] hide; show")

	SyncUI_RegisterDragFrame(self,"Player")
	SyncUI_UnitFrame_OnRegister(self)
	SyncUI_PlayerFrame_AuraHeader_Initialize()
	
	SyncUI_DisableFrame("PlayerFrame")
	SyncUI_DisableFrame("BuffFrame")
	SyncUI_DisableFrame("TemporaryEnchantFrame")
end

function SyncUI_PlayerFrame_OnEvent(self,event,...)
	if event == "PLAYER_ENTERING_WORLD" or event == "PLAYER_UPDATE_RESTING" then
		if InCombatLockdown() then
			return
		end
		
		if IsResting() then
			self.Resting_FadeIn:Play()
			self.Level_FadeOut:Play()
		else
			self.Resting_FadeOut:Play()
			self.Level_FadeIn:Play()
		end
	end
	
	if event == "PLAYER_ENTER_COMBAT" then
		self.ThreatGlow:Show()
	end
	
	if event == "PLAYER_LEAVE_COMBAT" then
		self.ThreatGlow:Hide()
	end
	
	if event == "PLAYER_REGEN_ENABLED" then
		self.Combat_FadeOut:Play()
		
		
		if IsResting() then
			self.Resting_FadeIn:Play()
		else
			self.Level_FadeIn:Play()
		end
	end
	
	if event == "PLAYER_REGEN_DISABLED" then
		self.Combat_FadeIn:Play()
		
		if IsResting() then
			self.Resting_FadeOut:Play()
		else
			self.Level_FadeOut:Play()
		end	
	end
end

function SyncUI_TargetFrame_OnLoad(self)
	self:SetScale(SyncUI_UIScale*1.2)
	self.unitID = "target"
	self.Name = self.NameFrame.Name
	self.Health = self.ArtFrame.Health
	self.Mana = self.ArtFrame.Mana
	self.Level = self.ArtFrame.Level
	self.Boss = self.ArtFrame.Boss
	self.Death = self.ArtFrame.Death
	self.Quest = self.ArtFrame.Quest
	self.Disconnect = self.ArtFrame.Disconnect
	self.Tapped = self.ArtFrame.Tapped
	self.RaidMark = self.ArtFrame.RaidMark
	self.UnitClass = self.ArtFrame.UnitClass
	
	RegisterStateDriver(self, "visibility", "[petbattle] hide; [@target,exists] show; hide")
	
	SyncUI_RegisterDragFrame(self,"Target")
	SyncUI_UnitFrame_OnRegister(self)
	SyncUI_DisableFrame("TargetFrame")	
end

function SyncUI_FocusFrame_OnLoad(self)
	self:SetScale(SyncUI_UIScale*1.2)
	self.unitID = "focus"
	self.Name = self.NameFrame.Name
	self.Health = self.ArtFrame.Health
	self.Mana = self.ArtFrame.Mana
	self.Level = self.ArtFrame.Level
	self.Boss = self.ArtFrame.Boss
	self.Death = self.ArtFrame.Death
	self.Quest = self.ArtFrame.Quest
	self.Disconnect = self.ArtFrame.Disconnect
	self.Tapped = self.ArtFrame.Tapped
	self.RaidMark = self.ArtFrame.RaidMark
	self.UnitClass = self.ArtFrame.UnitClass
	
	SyncUI_RegisterDragFrame(self,"Focus")
	SyncUI_UnitFrame_OnRegister(self,true)
	SyncUI_DisableFrame("FocusFrame")
end

function SyncUI_PetFrame_OnLoad(self)
	self:SetScale(SyncUI_UIScale*1.2)
	self.unitID = "pet"
	self.Name = self.ArtFrame.Name
	self.Health = self.ArtFrame.Health
	
	RegisterStateDriver(self, "visibility", "[petbattle][overridebar][vehicleui][possessbar,@vehicle,exists] hide; [@pet,exists] show; hide")
	
	SyncUI_RegisterDragFrame(self,"Pet")
	SyncUI_UnitFrame_OnRegister(self)
end

function SyncUI_ToTFrame_OnLoad(self)
	self.unitID = "targettarget"
	self.Name = self.ArtFrame.Name
	self.Health = self.ArtFrame.Health
	
	SyncUI_RegisterDragFrame(self,"Target Target")
	SyncUI_UnitFrame_OnRegister(self,true)
end

function SyncUI_ToFFrame_OnLoad(self)
	self.unitID = "focustarget"
	self.Name = self.ArtFrame.Name
	self.Health = self.ArtFrame.Health
	
	SyncUI_RegisterDragFrame(self,"Focus Target")
	SyncUI_UnitFrame_OnRegister(self,true)
end

function SyncUI_PartyFrame_OnLoad(self)
	if self:GetID() == 0 then
		self.unitID = "player"
	else
		self.unitID = "party"..self:GetID()
	end
	self.Name = self.ArtFrame.Name
	self.Role = self.ArtFrame.Role
	self.StatusIcon = self.ArtFrame.StatusIcon
	
	if self:GetName():find("Heal") then
		self.isHeal = true
		self.Dispel = self.ArtFrame.Dispel
	else
		self.Health = self.ArtFrame.Health
	end

	SyncUI_UnitFrame_OnRegister(self,true)
	SyncUI_DisableFrame("PartyMemberFrame"..self:GetID())
end

function SyncUI_RaidMember_OnLoad(self)
	self.Name = self.ArtFrame.Name
	self.Role = self.ArtFrame.Role
	self.StatusIcon = self.ArtFrame.StatusIcon
	self.isRaid = true
	
	if self:GetName():find("Heal") then
		self.isHeal = true
		self.Dispel = self.ArtFrame.Dispel
	else
		self.Selection = self.ArtFrame.Selection
	end
	
	SyncUI_UnitFrame_OnRegister(self,true)
end

function SyncUI_BossFrame_OnLoad(self)
	self.unitID = "boss"..self:GetID()
	self.Name = self.ArtFrame.Name
	self.Health = self.ArtFrame.Health
	self.RaidMark = self.ArtFrame.RaidMark
	
	SyncUI_UnitFrame_OnRegister(self,true)
	SyncUI_DisableFrame("Boss"..self:GetID().."TargetFrame")
end


-- Raid Frame Setup
local function UpdateHeaderSorting(header,sortBy)
	if sortBy == "CLASS" then
		header:SetAttribute("groupingOrder", "DEATHKNIGHT,DRUID,HUNTER,MAGE,MONK,PALADIN,PRIEST,ROGUE,SHAMAN,WARLOCK,WARRIOR")
		header:SetAttribute("sortMethod", "NAME")
		header:SetAttribute("groupBy", "CLASS")			
	elseif sortBy == "ROLE" then
		header:SetAttribute("groupingOrder", "TANK,HEALER,DAMAGER,NONE")
		header:SetAttribute("sortMethod", "NAME")
		header:SetAttribute("groupBy", "ASSIGNEDROLE")
	elseif sortBy == "GROUP" then
		header:SetAttribute("groupingOrder", "1,2,3,4,5,6,7,8")
		header:SetAttribute("sortMethod", "INDEX")
		header:SetAttribute("groupBy", "GROUP")
	end
end

local function PreCreateChildren(header)
	local index = header:GetAttribute("startingIndex")
	local maxColumns = header:GetAttribute("maxColumns") or 1
	local perColumn = header:GetAttribute("unitsPerColumn") or 5
	local maxFrames = maxColumns * perColumn
	local count = header.FrameCount
	
	if not count or count < maxFrames then
		header:Show()
		header:SetAttribute("startingIndex", 1-maxFrames)
		header:SetAttribute("startingIndex", index)
		header.FrameCount = maxFrames
	end
end

function SyncUI_RaidFrameContainer_OnLoad(self)
	self:SetScale(SyncUI_UIScale)
	
	do	-- Create Normal Header
		local header = CreateFrame("frame", "SyncUI_RaidFrameContainer_Normal", self, "SecureRaidGroupHeaderTemplate")

		header:SetScale(1.2)
		header:SetPoint("TOPLEFT",5,-40)
		header:SetAttribute("template","SyncUI_RaidMemberTemplate")
		header:SetAttribute("point","TOP")
		header:SetAttribute("xOffset",0)
		header:SetAttribute("yOffset",6)
		header:SetAttribute("maxColumns",2)
		header:SetAttribute("unitsPerColumn",20)
		header:SetAttribute("columnSpacing",5)
		header:SetAttribute("columnAnchorPoint","LEFT")
		header:SetAttribute("showRaid",true)
		header:SetAttribute("sortDir","ASC")

		UpdateHeaderSorting(header,"GROUP")
		PreCreateChildren(header)
		
		self.Normal = header
	end

	do	-- Create Heal Header
		local header = CreateFrame("frame", "SyncUI_RaidFrameContainer_Heal", self, "SecureRaidGroupHeaderTemplate")

		header:SetAttribute("template","SyncUI_RaidMemberTemplate2")
		header:SetAttribute("point","LEFT")
		header:SetAttribute("xOffset",-5)
		header:SetAttribute("yOffset",0)
		header:SetAttribute("maxColumns",8)
		header:SetAttribute("unitsPerColumn",5)
		header:SetAttribute("columnSpacing",-5)
		header:SetAttribute("columnAnchorPoint","BOTTOM")
		header:SetAttribute("showRaid",true)
		header:SetAttribute("sortDir","ASC")
		header:SetAttribute("minWidth", 400)
		header:SetAttribute("minHeight", 161)
		
		UpdateHeaderSorting(header,"GROUP")
		PreCreateChildren(header)
		
		self.Heal = header
		
		SyncUI_RegisterDragFrame(header,"Raid Frames (Heal)")
		header:Hide()
	end
end

function SyncUI_RaidFrameContainer_OnEvent(self,event,...)
	if event == "PLAYER_REGEN_ENABLED" then		
		SyncUI_RaidFrameContainer_UpdateSorting(self.sortBy)
		
		self.sortBy = nil
		self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	end
end

function SyncUI_RaidFrameContainer_UpdateSorting(sortBy)
	local self = SyncUI_RaidFrameContainer

	if InCombatLockdown() then
		if not self.sortBy then
			print("|cFF00FF99Sync|cFFFFFFFFOptions: Raid Frame Sort Method will be assigned after leaving combat.")
		end
		
		self.sortBy = sortBy
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
	else
		UpdateHeaderSorting(self.Normal,sortBy)
		UpdateHeaderSorting(self.Heal,sortBy)
	end
end
